%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: bash_lexer.l,v 1.1 2006/02/24 08:27:15 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: bash_lexer.l,v $
 * Revision 1.1  2006/02/24 08:27:15  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include "cuiterm.h"
#include "cuilexelements.h"
#include "bash_parser.h"
#include "terminal.h"
#include "history.h"

#undef YY_INPUT
extern Terminal *terminal_for_lex;
#define YY_INPUT(b, r, ms) (r = terminal_get_char(terminal_for_lex, b, ms))

%}

%s COMM ARG S_FOR S_FOR_LIST
%s ASSIGN_COMM

%%
	if (terminal_for_lex->parsed_chars == 0){
		#ifdef DEBUG
			g_message("%s(): Starting new scan...", __func__);
		#endif
		BEGIN(COMM);
	}

<COMM>for {
#ifdef DEBUG
	g_message("%s(): FOR: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = FOR;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "keyword",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	BEGIN(S_FOR);
  	return(FOR);
}

<COMM>done {
#ifdef DEBUG
	g_message("%s(): DONE: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = DONE;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "keyword",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(DONE);
}

<COMM>[_a-zA-Z][_a-zA-Z0-9]*/= {
	/*
	 * Environment variable at the beginning of the command.
	 */
#ifdef DEBUG
	g_message("%s(): VARIABLE: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = VARIABLE;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "variable", 
		bash_lval.sval.begin, 
		bash_lval.sval.end);
	/*
	 * Maybe we though it was a command.
	 */
	terminal_remove_tag(terminal_for_lex, "underline-error", 
		bash_lval.sval.begin, 
		bash_lval.sval.end);
	BEGIN (ASSIGN_COMM);
  	return VARIABLE;
}

<ASSIGN_COMM>= {
	/* * Assignment equal sign.  */
#ifdef DEBUG
	g_message("%s(): equal: '%s'", __func__, yytext);
#endif
	terminal_get_token_marks(terminal_for_lex, yytext, NULL, NULL); 
  	return yytext[0];
}

<ASSIGN_COMM>([^=\t ]+)|(\"[^\"]*[\"]) {
	/*
	 * String in assignment. 
	 */
#ifdef DEBUG
	g_message("%s(): STRING(ASSIGN_COMM): '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = VARIABLE;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "list_element", 
		bash_lval.sval.begin, 
		bash_lval.sval.end);
	/*
	 * Maybe we though it was a command.
	 */
	terminal_remove_tag(terminal_for_lex, "underline-error", 
		bash_lval.sval.begin, 
		bash_lval.sval.end);
	BEGIN (COMM);
  	return STRING;
}



<COMM>[^\t \n\r\$\(\);|=]+ {
#ifdef DEBUG
	g_message("%s(): COMMAND: '%s'", __func__, yytext);
#endif
	/* A simple string is a command in this state.*/
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = COMMAND;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "command_name", 
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	BEGIN(ARG);
  	return(COMMAND);
}

	/*
	 *
	 */
<S_FOR>"in" {
#ifdef DEBUG
	g_message("%s(): IN: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = IN;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "keyword",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
	BEGIN(S_FOR_LIST);
  	return(IN);
}

<S_FOR>"do" {
#ifdef DEBUG
	g_message("%s(): DO: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = DO;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "keyword",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	BEGIN(COMM);
  	return(DO);
}

<S_FOR_LIST>\; {
#ifdef DEBUG
	g_message("%s(): CS: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = CS;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	BEGIN(S_FOR);
  	return CS;
}

<S_FOR_LIST>([^\t \n\r\$\(\);|]+)|(\"[^\"]*\") {
#ifdef DEBUG
	g_message("%s(): LIST_ELEMENT: '%s'", __func__, yytext);
#endif
	/* Simple string is a list element*/
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = LIST_ELEMENT;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "list_element",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(LIST_ELEMENT);
}

<S_FOR>[^\t \n\r\$\(\);|]+ {
#ifdef DEBUG
	g_message("%s(): VARIABLE: '%s'", __func__, yytext);
#endif
	/* A simple string is a variable name in this state */
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = VARIABLE;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "variable", 
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(VARIABLE);
}


<ARG>\; {
#ifdef DEBUG
	g_message("%s(): COMM: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	BEGIN(COMM);
  	//return CS;
}


<ARG>\| {
#ifdef DEBUG
	g_message("%s(): PIPE: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	BEGIN(COMM);
  	return PIPE;
}


<ARG>"$(" {
#ifdef DEBUG
	g_message("%s(): ESTART: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = ESTART;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "embedded",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(ESTART);
}

<ARG>")" {
#ifdef DEBUG
	g_message("%s(): EEND: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = EEND;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "embedded",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(EEND);
}

<ARG>--[^ \t\n\r]+ {
	/* Long option option */
#ifdef DEBUG
	g_message("%s(): LONGOPT: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = LONGOPT;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "option",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(LONGOPT);
}

<ARG>-[^ \t\n\r]+ {
	/* Simple (short) option */
#ifdef DEBUG
	g_message("%s(): OPTION: '%s'", __func__, yytext);
#endif
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = OPTION;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "option",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(OPTION);
}

<ARG>\$[^\t \n\r\$\(\);|]+ {
	/* Variable value as an argument. */
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = STRING;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "varvalue",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(STRING);
}

<ARG>[^\t \n\r\$\(\);|]+("\\".)?[^\t \n\r\$\(\);|]* {
#ifdef DEBUG
	g_message("%s(): STRING: '%s'", __func__, yytext);
#endif
	/* Simple string as an argument. */
	bash_lval.sval.text = strdup(yytext);
	bash_lval.sval.type = STRING;
	terminal_get_token_marks(terminal_for_lex, 
		bash_lval.sval.text,
		&bash_lval.sval.begin,
		&bash_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "argument",
		bash_lval.sval.begin, 
		bash_lval.sval.end); 
  	return(STRING);
}

<*><<EOF>> {
#ifdef DEBUG
	g_message("%s(): END_OF_FILE: '%s'", __func__, yytext);
#endif
	return END_OF_FILE;
}

<*>\r|\n|(\r\n)|(\n\r) {
	/* Just for skipping to the next token */
	terminal_get_token_marks(terminal_for_lex, 
		yytext, NULL, NULL);	
}


<*>[ \t]+ {
	/* Just for skipping to the next token */
	terminal_get_token_marks(terminal_for_lex, 
		yytext, NULL, NULL);	
}

%%
