%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: bash_parser.y,v 1.1 2006/02/24 08:25:06 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: bash_parser.y,v $
 * Revision 1.1  2006/02/24 08:25:06  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "bash_parser_c.h"

Terminal *terminal_for_lex = NULL;

GNode* node_add_child(GNode *parent, GNode *child)
{
	g_node_append(parent, child);
	return parent;
}

GNode* node2_new(GNode *node1, GNode *node2, gint type)
{
	GNode *new_node;
	CuiLexElement *new_lex = g_malloc(sizeof(CuiLexElement));
	
	new_lex->type = type;
	new_lex->text = g_strdup("");
	new_lex->begin = NULL; //lex1->begin; FIXME:
	new_lex->end = NULL; //lex2->end;
	
	new_node = g_node_new(new_lex);
	node_add_child(new_node, node1);
	node_add_child(new_node, node2);
	return new_node;
}

GNode* node0_new(CuiLexElement *lex_element)
{
	GNode *new_node;
	CuiLexElement *saved = g_malloc(sizeof(CuiLexElement));
	
	saved->text = lex_element->text;
	saved->begin = lex_element->begin;
	saved->end = lex_element->end;
	saved->type = lex_element->type;
	new_node = g_node_new(saved);
	return new_node;
}



%}

%parse-param {Terminal *t}
%union { 
	CuiLexElement  sval; /* String with begin and end marks. */
	GNode         *nval; /* N-ary tree node */
}

%token        BOGUS
%token        CONCAT SERIAL_EXECUTION
%token        END_OF_FILE
%token <sval> STRING VARIABLE LIST_ELEMENT OPTION LONGOPT
%token <sval> FOR IN DO DONE
%token <sval> ESTART EEND
%token <sval> COMMAND CS PIPE
%type  <nval> commands command argument arglist

%%
commands
   : 
   | commands command { 
		$$ = node2_new($1, $2, SERIAL_EXECUTION); 
		t->parse_tree = $$; 
	}
   | commands PIPE command { 
		$$ = node2_new($1, $3, PIPE);
		t->parse_tree = $$; 
	}
   | command { 
		$$ = $1;
		t->parse_tree = $$; 
	}
   ;

command
   : COMMAND {
		t->cmd = g_string_assign(t->cmd, $1.text);
		$$ = node0_new(&$1);
		t->parse_tree = $$;
	}
   | COMMAND arglist {
		t->cmd = g_string_assign(t->cmd, $1.text);
		$$ = node0_new(&$1);
		node_add_child($$, $2);
		t->parse_tree = $$;
	}
   | FOR VARIABLE IN list CS DO commands DONE 
   | assignment COMMAND {
		t->cmd = g_string_assign(t->cmd, $2.text);
		$$ = node0_new(&$2);
		t->parse_tree = $$;
	}
   | assignment COMMAND arglist {
		t->cmd = g_string_assign(t->cmd, $2.text);
		$$ = node0_new(&$2);
		node_add_child($$, $3);
		t->parse_tree = $$;
	}
   | assignment FOR VARIABLE IN list CS DO commands DONE 
   ;

assignment
   : VARIABLE '=' STRING
   ;

list
   : list LIST_ELEMENT
   | LIST_ELEMENT
   ;

arglist
   : arglist argument { 
		$$ = node2_new($1, $2, CONCAT); 
		t->parse_tree = $$; 
	}
   | argument { 
		$$ = $1;
		t->parse_tree = $$; 
	}
   ;

argument
   : STRING        	{ $$ = node0_new(&$1); t->parse_tree = $$; }
   | OPTION		{ $$ = node0_new(&$1); t->parse_tree = $$; }
   | LONGOPT		{ $$ = node0_new(&$1); t->parse_tree = $$; }
   | ESTART command EEND   { $$ = $2;             t->parse_tree = $$; }
   ;
%%


int bash_error(Terminal *t, char *c)
{
#ifdef DEBUG
	g_warning("%s(): Error in BASH command: %s\n", __func__, c);
#endif
	return(1);
}


int bash_wrap(Terminal *t)
{
	return 1;
}

