/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: dirhistory.c,v $
 * Revision 1.1  2006/02/24 17:20:33  pipas
 * Initial revision
 *
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

#include "configure.h"
#include "support.h"
#include "interface.h"
#include "terminal.h"
#include "bookmarks.h"
#if ALL_DEBUG
#   define DEBUG
#endif

typedef struct bookmark {
	gchar     *name;
	gchar     *directory;
	gchar     *host;
	gboolean   latex;
	gboolean   software;
} bookmark;

GList *bookmark_list = NULL;


void
on_add_bookmarks_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *app;
	GtkWidget *window;
	GtkWidget *main_widget;
	GtkWidget *entry;
	Terminal  *t;
	const gchar     *wd;
	
	window = create_new_bookmark_window();
	entry = lookup_widget(window, "new_bookmark_directory_entry");
	main_widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	g_object_set_data(G_OBJECT(window), "main_widget", main_widget);
	app = lookup_widget(GTK_WIDGET(menuitem), "app1");
	g_object_set_data(G_OBJECT(window), "app", app);
	
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_assert(t != NULL);
	wd = terminal_get_working_directory(t);
	gtk_entry_set_text(GTK_ENTRY(entry), wd);
	
	gtk_widget_show(window);
}



void
on_new_bookmark_cancel_button_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window;
	window = lookup_widget(GTK_WIDGET(button), "new_bookmark_window");
	gtk_widget_destroy(window);
}

static void
_store_bookmarks(void)
{
	gchar *name;
	GList *actual = bookmark_list;
	bookmark *bm;
	gint n = 0;
	
	while(actual != NULL){
		bm = actual->data;
#ifdef DEBUG
		g_message("%s(): %d->'%s'/'%s'", __func__, n,
				bm->name, bm->directory);
#endif
		name = g_strdup_printf("/bookmarks/bookmark%d.name", n);
		bg_conf_set_string(name, bm->name);
		g_free(name);
		name = g_strdup_printf("/bookmarks/bookmark%d.directory", n);
		bg_conf_set_string(name, bm->directory);
		g_free(name);
		
next_item:
		actual = actual->next;
		++n;
	}
	
#ifdef DEBUG
	g_message("%s(): %d->''/''", __func__, n);
#endif
	name = g_strdup_printf("/bookmarks/bookmark%d.name", n);
	bg_conf_set_string(name, "");
	g_free(name);
	name = g_strdup_printf("/bookmarks/bookmark%d.directory", n);
	bg_conf_set_string(name, "");
	g_free(name);
}

static void
_read_bookmarks(void)
{
	gchar *name;
	GList *actual = bookmark_list;
	bookmark *bm;
	gint n = 0;
	/*
	 * First we remove the bookmarks from the list.
	 */
	actual = bookmark_list;
	while(actual != NULL){
		bm = actual->data;
		g_free(bm->name);
		g_free(bm->directory);
		g_free(bm->host);
		actual = actual->next;
	}
	g_list_free(bookmark_list);
	bookmark_list = NULL;

	for (n = 0; n < 15; ++n){
#ifdef DEBUG
		g_message("%s(): reading bookmark[%d]", __func__, n);
#endif
		bm = g_new0(bookmark, 1);
		name = g_strdup_printf("/bookmarks/bookmark%d.name", n);
		bm->name = bg_conf_get_string(name, "");
		if (strlen(bm->name) == 0){
			g_free(bm->name);
			g_free(bm);
			break;
		}
		name = g_strdup_printf("/bookmarks/bookmark%d.directory", n);
		bm->directory = bg_conf_get_string(name, "");
		bookmark_list = g_list_append(bookmark_list, bm);
	}
}

void
on_specific_bookmark_activate(GtkMenuItem     *menuitem,
                              gpointer         user_data)
{
	GtkWidget *text_view;
	Terminal *t;
	gchar *command;
	bookmark *bm = user_data;
	
	text_view = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(text_view), "terminal");
	g_assert(t != NULL);

	command = g_strdup_printf("cd %s", bm->directory);
	terminal_send_command(t, command, TRUE);
	g_free(command);
	
	command = g_strdup_printf("ls%s", 
			bg_conf_get_default_ls_options(FALSE));
	terminal_send_command(t, command, TRUE);
	g_free(command);

}


gint
add_bookmarks_to_menu(GtkWidget *app)
{
	GnomeUIInfo *menu;
	gint n;
	GList *actual;
	bookmark *bm;

	_read_bookmarks();
	
	menu = g_new0(GnomeUIInfo, 16);
	menu[0].type = GNOME_APP_UI_SEPARATOR;

	n = 1;
	actual = bookmark_list;
	while(actual != NULL) {
		bm = actual->data;
		menu[n].type = GNOME_APP_UI_ITEM;
		menu[n].label = bm->name;
		menu[n].hint = _("Select to jump to the bookmark.");
		menu[n].moreinfo = on_specific_bookmark_activate;
		menu[n].user_data = bm;
		menu[n].unused_data = NULL;
		menu[n].pixmap_type = GNOME_APP_PIXMAP_NONE;
		menu[n].pixmap_info = NULL;
		menu[n].accelerator_key = 0;
		menu[n].ac_mods = (GdkModifierType) 0;
		menu[n].widget = NULL;
next_item:
		actual = actual->next;
		n++;
	}
		
	menu[n].type = GNOME_APP_UI_ENDOFINFO;
	// FIXME: Why do we need to remove the menus? Who calls this function?
	gnome_app_remove_menus((GnomeApp *)app, _("Bookmarks/<Separator>"), 100);
	gnome_app_insert_menus((GnomeApp *)app, _("Bookmarks/Edit Bookmarks"), menu);
}

static void 
_refresh_menus(GtkWidget *app)
{
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif
	//gnome_app_remove_menus(app, _("Bookmarks/<Separator>"), 100);
	add_bookmarks_to_menu(app);
}

/*
 * New bookmark window functions.
 */
void
on_new_bookmark_ok_button_clicked      (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget    *app;
	GtkWidget    *window;
	GtkWidget    *entry;
	const gchar  *name;
	const gchar  *directory;
	bookmark     *new_entry;

	window = lookup_widget(GTK_WIDGET(button), "new_bookmark_window");
	app = g_object_get_data(G_OBJECT(window), "app");	
	
	entry = lookup_widget(GTK_WIDGET(button), 
			"new_bookmark_directory_entry");
	directory = gtk_entry_get_text(GTK_ENTRY(entry));
	entry = lookup_widget(GTK_WIDGET(button), 
			"new_bookmark_name_entry");
	name = gtk_entry_get_text(GTK_ENTRY(entry));
	if (strlen(name) == 0)
		name = directory;
		
	new_entry = g_new0(bookmark, 1);
	new_entry->name = g_strdup(name);
	new_entry->directory = g_strdup(directory);
	
	bookmark_list = g_list_append(bookmark_list, new_entry);
	_store_bookmarks();
	/*
	 * Refreshing menus.
	 */
	_refresh_menus(app);
	/*
	 * Destroying window.
	 */
	gtk_widget_destroy(window);
}

/*
 * Edit bookmarks window functions.
 */
void
on_edit_bookmarks_close_button_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window;
	window = lookup_widget(GTK_WIDGET(button), "edit_bookmarks_window");
	gtk_widget_destroy(window);
}


void
on_edit_bookmarks_treeview_realize     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkTreeView *tree_view = GTK_TREE_VIEW(widget);
	GtkTreeViewColumn *column;
	GtkCellRenderer   *renderer;
	GtkTreeStore *tree_store;
	GtkTreeIter iter_level0;
	
	gint n;
	GList *actual;
	bookmark *bm;
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif
	/*
	 * Tree model
	 */
	tree_store = gtk_tree_store_new(6,
			G_TYPE_STRING,    /* Long Name.               */
			G_TYPE_STRING,    /* Directory name.          */
			G_TYPE_STRING,    /* Host name.               */
			G_TYPE_BOOLEAN,   /* LaTeX direcory?          */
			G_TYPE_BOOLEAN,   /* Software directory?      */
			G_TYPE_POINTER    /* The list item to handle. */
			);

	_read_bookmarks();
	
	n = 1;
	actual = bookmark_list;
	while(actual != NULL) {
#ifdef DEBUG
		g_message("%s(): reading into the store: %d", __func__, n);
#endif
		bm = actual->data;
		
		gtk_tree_store_append(tree_store, &iter_level0, NULL);
		gtk_tree_store_set(tree_store, &iter_level0, 
	  		0, bm->name, 
	  		1, bm->directory, 
			2, bm->host,
			3, bm->latex,
			4, bm->software,
			5, actual,
			-1);
next_item:
		actual = actual->next;
		n++;
	}
	gtk_tree_view_set_model(tree_view, GTK_TREE_MODEL(tree_store));
	
	/*
	 * The columns.
	 */
  	/* Column0: the name */
  	renderer = gtk_cell_renderer_text_new ();
  	column = gtk_tree_view_column_new_with_attributes(
		  _("Designation"), 
		  renderer, "text", 0, NULL);
  	gtk_tree_view_append_column(tree_view, column);
  	/* Column1: the directory */
  	renderer = gtk_cell_renderer_text_new ();
  	column = gtk_tree_view_column_new_with_attributes(
		  _("Directory"), 
		  renderer, "text", 1, NULL);
  	gtk_tree_view_append_column(tree_view, column);
  	/* Column2: the hostname */
  	renderer = gtk_cell_renderer_text_new ();
  	column = gtk_tree_view_column_new_with_attributes(
		  _("Host"), 
		  renderer, "text", 2, NULL);
  	gtk_tree_view_append_column(tree_view, column);
  	/* Column3: software directory */
  	renderer = gtk_cell_renderer_toggle_new ();
  	column = gtk_tree_view_column_new_with_attributes(
		  _("Softw."), 
		  renderer, "active", 3, NULL);
  	//g_signal_connect (renderer, "toggled",
	//	  G_CALLBACK (enable_toggled), tree_view);
  	gtk_tree_view_append_column(tree_view, column);
  	/* Column4: latex directory */
  	renderer = gtk_cell_renderer_toggle_new ();
  	column = gtk_tree_view_column_new_with_attributes(
		  _("LaTeX"), 
		  renderer, "active", 4, NULL);
  	//g_signal_connect (renderer, "toggled",
	//	  G_CALLBACK (enable_toggled), tree_view);
  	gtk_tree_view_append_column(tree_view, column);

	gtk_tree_view_set_rules_hint(tree_view, TRUE);
}


void
on_edit_bookmarks_button_up_clicked    (GtkButton       *button,
                                        gpointer         user_data)
{
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif

}


void
on_edit_bookmarks_button_down_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif

}


void
on_edit_bookmarks_button_left_clicked  (GtkButton       *button,
                                        gpointer         user_data)
{
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif

}


void
on_edit_bookmarks_button_right_clicked (GtkButton       *button,
                                        gpointer         user_data)
{
#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif

}


void
on_edit_bookmarks_button_delete_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkTreeView *tree_view = (GtkTreeView *)lookup_widget(GTK_WIDGET(button), "edit_bookmarks_treeview");
	GtkTreeSelection *tree_selection = 
		gtk_tree_view_get_selection(tree_view);
	GtkWidget *window;
	GtkWidget *app;
	GList *list;
	GtkTreeModel *tree_model;
	GtkTreeIter iter;
	
	window = lookup_widget(GTK_WIDGET(button), "edit_bookmarks_window");
	app = g_object_get_data(G_OBJECT(window), "app");	

#ifdef DEBUG
	g_message("%s(): Start.", __func__);
#endif
	
	_store_bookmarks();
	_refresh_menus(app);
	
	tree_model = gtk_tree_view_get_model(tree_view);
	list = gtk_tree_selection_get_selected_rows(tree_selection, NULL);
	
	if (list == NULL)
		return;
	
	gtk_tree_model_get_iter(tree_model, &iter, list->data);
	gtk_tree_store_remove(GTK_TREE_STORE(tree_model), &iter);
        g_list_foreach(list, (GFunc)gtk_tree_path_free, NULL);
	g_list_free (list);
}


