/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: callbacks.c,v 1.1 2006/02/24 17:25:02 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: callbacks.c,v $
 * Revision 1.1  2006/02/24 17:25:02  pipas
 * Initial revision
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <stdlib.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "configure.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "terminal.h"     /* This file contains more callback functions */
#include "dirhistory.h"   /* This file contains more callback functions */
#include "bookmarks.h"    /* This file contains more callback functions */
#include "cuiwindow.h"
#include "history.h"      /* This file contains more callback functions */

#include "gui_completion.h"

#if DEBUG_ALL
#  define DEBUG_CALL
#endif

#ifdef __CYGWIN__
	#define TTY_NAME "/dev/ptmx"
#endif

void 
unimplemented(const char *function_name, 
		const char *filename,
		gint linenumber)
{
	GtkWidget *dialog;
	dialog = gtk_message_dialog_new (NULL,
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
        			  "Unimplementes function "
				  "'%s()' at line %d of source file '%s'.",
                                  function_name,
				  linenumber,
				  filename);
	
	gtk_dialog_run (GTK_DIALOG (dialog));
	gtk_widget_destroy (dialog);
}

void
on_new1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget = create_app1();
	gtk_widget_show(widget);
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_save_as1_activate(menuitem, user_data);
}


int 
save_textview_to_file(
		GtkWidget *parent,
		GtkTextView *text_view, 
		char *filename){
	
	int output_file; 
	int written;
	GtkWidget *dialog;
	GtkTextBuffer *text_buffer;
	GtkTextIter start;
	GtkTextIter end;
	gchar *text;
	size_t size;
	
	/*
	 */
	output_file = open(filename, 
			O_WRONLY | O_CREAT, 
			S_IRUSR | S_IWUSR);
	
	if (output_file == -1){
  		dialog = gtk_message_dialog_new (GTK_WINDOW(parent),
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  _("Can not open for writing: "
				  "'%s': %m."),
                                  filename);
 		gtk_dialog_run (GTK_DIALOG (dialog));
  		gtk_widget_destroy (dialog);
		close(output_file);
		return -1;
	}
	
	/*
	 */
	text_buffer = gtk_text_view_get_buffer(text_view);
	gtk_text_buffer_get_start_iter(text_buffer, &start);
	gtk_text_buffer_get_end_iter(text_buffer, &end);
	text = gtk_text_buffer_get_text(text_buffer,
                                        &start,
                                        &end,
                                        TRUE);
	/*
	 * A szöveg írása. Az strlen() függvény itt nem a karakterek
	 * számát, hanem a foglalt memória méretét adja meg.
	 */
	size = strlen(text);
	if ((written = write(output_file, text, size)) != size){
  		dialog = gtk_message_dialog_new (GTK_WINDOW(parent),
                                  GTK_DIALOG_DESTROY_WITH_PARENT,
                                  GTK_MESSAGE_ERROR,
                                  GTK_BUTTONS_CLOSE,
                                  _("Error writing file: "
				  "'%s': %m"),
                                  filename);
 		gtk_dialog_run(GTK_DIALOG (dialog));
  		gtk_widget_destroy(dialog);
		written = -1;
	}

	/*
	 */
	g_free(text);
	close(output_file);
	return written;
}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkWidget *window = lookup_widget(GTK_WIDGET(menuitem), "app1");
  GtkWidget *text_view = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
  GtkWidget *dialog;
  gchar *filename;
  
  dialog = gtk_file_chooser_dialog_new("Save as...",
                     GTK_WINDOW(window),
                     GTK_FILE_CHOOSER_ACTION_SAVE, 
		     GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		     GTK_STOCK_OK, GTK_RESPONSE_OK,
                     NULL);
  
  if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_OK){
  	gtk_widget_destroy (dialog);
	return;
  }
  
  filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
  gtk_widget_destroy (dialog);
  
  if (filename == NULL){
		return;
  }
	  
  if (save_textview_to_file(window, 
			  GTK_TEXT_VIEW(text_view), 
			  filename) == 0){
	  gtk_window_set_title(GTK_WINDOW(window), filename);
  }
  g_free (filename);
}


void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	if (widget == NULL)
		widget = lookup_widget(GTK_WIDGET(menuitem), "application_popup_menu");
	if (widget == NULL)
		goto not_found;
	
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	if (t == NULL)
		goto not_found;
	
	terminal_send_command(t, "exit", TRUE);
	return;

not_found:
	g_warning("%s(): Terminal could not found.", __func__);
	gtk_exit(EXIT_SUCCESS);
}


void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}


void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}

void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	widget = create_properties_window();
	gtk_widget_show(widget);
}




void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *about_window;
	const gchar *authors[] = {
		"László Pere <pipas@linux.pte.hu>",
		NULL
	};
	about_window = gnome_about_new("cuiterm", 
			VERSION,
                        "Copyright 2005 László Pere",
                        "The cuiterm is a composite user interface\n"
			"frontend for the BASH shell. The program can be\n"
			"used, modified and distributed as stated in the\n"
			"GNU General Public License.",
                        authors, 
                        NULL, //**documenters,
                        NULL, //*translator_credits,
                        NULL //GdkPixbuf *logo_pixbuf);
	);
	gtk_widget_show(about_window);
}

/*
 * This callback function will solve a lot of problems in the terminal character
 * handling.
 */
static void        
text_buffer_changed(GtkTextBuffer *textbuffer, 
		gpointer user_data)
{
	GtkTextView *text_view = GTK_TEXT_VIEW(user_data);
	GtkTextMark *cursor;
	static gint n = 0;
#ifdef DEBUG_CALL
	g_message("%s(): start: %08d", __func__, n++);	
#endif

	terminal_changed(g_object_get_data(G_OBJECT(text_view), "terminal"));
	cursor = gtk_text_buffer_get_insert(textbuffer);
	gtk_text_view_scroll_mark_onscreen(text_view, cursor);
}

void
on_main_scrolledwindow_scroll_child    (GtkScrolledWindow *scrolledwindow,
                                        GtkScrollType    scroll,
                                        gboolean         horizontal,
                                        gpointer         user_data)
{
}


static void
start_bash(gchar *binary) 
{
	char *argv[] = { "/bin/bash", 
			 "--noprofile", /* Do not read profile files */
			 "--norc",      /* Do not read the rc files  */
		         "-i",          /* Interactive shell         */
			 NULL };
	
	g_setenv("TERM", "xterm", TRUE);
	g_setenv("COLUMNS", g_strdup_printf("%.0f",
				bg_conf_get_double("/settings/shell/columns",
					80)), TRUE);
	g_setenv("ROWS", g_strdup_printf("%.0f",
				bg_conf_get_double("/settings/shell/rows",
					25)), TRUE);
	g_setenv("ROWS", "120", TRUE);
	g_setenv("PAGER", "cat", TRUE);
	g_setenv("PS1", "\\[\\033]0;\\w\\007\\][\\u@\\h \\W]# ", TRUE);
	
	execv(binary, argv);
}

static void
start_ksh(gchar *binary) 
{
	char *argv[] = { "/bin/ksh", 
		         "-i",          /* Interactive shell         */
			 NULL };
	
	g_setenv("TERM", "xterm", TRUE);
	g_setenv("COLUMNS", g_strdup_printf("%.0f",
				bg_conf_get_double("/settings/shell/columns",
					80)), TRUE);
	g_setenv("ROWS", g_strdup_printf("%.0f",
				bg_conf_get_double("/settings/shell/rows",
					25)), TRUE);
	g_setenv("ROWS", "120", TRUE);
	g_setenv("PAGER", "cat", TRUE);
	g_setenv("PS1", "\\[\\033]0;\\w\\007\\][\\u@\\h \\W]# ", TRUE);
	execv(binary, argv);
}

static void
start_zsh(gchar *binary) 
{
	char *argv[] = { "/bin/zsh", 
		         "-i",          /* Interactive shell         */
			 NULL };
	
	g_setenv("TERM", "xterm", TRUE);
	g_setenv("COLUMNS", g_strdup_printf("%.0f",
				bg_conf_get_double("/settings/shell/columns",
					80)), TRUE);
	g_setenv("ROWS", g_strdup_printf("%.0f",
				bg_conf_get_double("/settings/shell/rows",
					25)), TRUE);
	g_setenv("ROWS", "120", TRUE);
	g_setenv("PAGER", "cat", TRUE);
	g_setenv("PROMPT", "%{\033]0;%d\007%}[%n@%m %1d]# ", TRUE);
	execv(binary, argv);
}

static void
terminal_widget_set_font(GtkWidget *widget,
		const gchar    *fontname,
		gboolean       set_widget_size)
{
	PangoFontDescription *font_desc;
	PangoContext     *context;
	PangoFontMetrics *metrics;
	gint	          width, height;
	gint              columns, rows;
	/*
	 * Getting the font description.
	 */
	font_desc = pango_font_description_from_string(fontname);
	pango_font_description_set_family(font_desc, "monospace");
	gtk_widget_modify_font (widget, font_desc);
	/*
	 * Calculating the size of the widget.
	 */
	if (!set_widget_size)
		return;
	columns = (gint)bg_conf_get_double("/settings/shell/columns", 80);
	rows = (gint)bg_conf_get_double("/settings/shell/rows", 25);
	
	context = gtk_widget_get_pango_context(widget);
	metrics = pango_context_get_metrics(context, font_desc, NULL);
	width = pango_font_metrics_get_approximate_char_width(metrics) / PANGO_SCALE;
	height = (pango_font_metrics_get_ascent(metrics) /*+ pango_font_metrics_get_descent(metrics)*/) / PANGO_SCALE;
 	pango_font_metrics_unref(metrics);
	/*
	 * Setting the size of the widget.
	 */
	gtk_widget_set_size_request(widget, width * columns + LEFT_MARGIN + 5, height * rows);
}


void
on_main_widget_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gchar *fontname;
	GtkTextBuffer *text_buffer;
	Terminal *t = NULL;
	int tty_handler;
	char *tty;
	pid_t pid;
	gchar *shellname;
	gchar *binary;
	
	t = terminal_new(GTK_TEXT_VIEW(widget));
	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	g_signal_connect((gpointer) text_buffer, "changed", 
			G_CALLBACK(text_buffer_changed), 
			(gpointer) widget);
	
	/*
	 * Setting the default font.
	 */
	fontname = bg_conf_get_string("/settings/basefont", BASE_FONT);
	terminal_widget_set_font(widget, fontname, TRUE);
	g_free(fontname);
	
	/*
	 * Getting the next available pseudo terminal, granting it to the
	 * user and unlocking it.
	 */
#ifdef HAVE_GETPT
	tty_handler = getpt();
#else
	tty_handler = open(TTY_NAME, O_RDWR | O_NOCTTY);
#endif
	if (grantpt(tty_handler) < 0 || unlockpt(tty_handler) < 0) {
		g_warning("%s(): %m\n", __func__);
		gtk_exit(EXIT_FAILURE);
	}

	/*
	 * Getting the tty name. 
	 */
	tty = ptsname(tty_handler);
#ifdef DEBUG_CALL
	if (tty == NULL) {
		g_error("%s(): error getting tty: %m\n", __func__);
	} else {
		g_message("%s(): Terminal: '%s'\n", __func__, tty);
	}
#endif
	/*
	 * Starting a new process with fork.
	 */
	pid = fork();
	if (pid == -1) 
		g_error("%s(): %m", __func__);

	if (pid != 0) {
		/*
		 * The parent process.
		 */
		// Here we write,
		t->shell_stream = g_io_channel_unix_new(tty_handler);
		g_io_channel_set_encoding(t->shell_stream, NULL, NULL);
		g_io_channel_set_buffered(t->shell_stream, FALSE);
		g_io_add_watch_full(t->shell_stream,
				    G_PRIORITY_HIGH,
				    G_IO_IN | G_IO_HUP | G_IO_PRI,
				    read_from_shell, 
				    (gpointer) t, 
				    NULL);
#if 0
		g_io_add_watch_full(t->shell_stream,
				    G_PRIORITY_DEFAULT_IDLE,
				    G_IO_PRI,
				    read_from_shell_priority, 
				    (gpointer) t, 
				    NULL);
#endif
#ifdef DEBUG_CALL
		fprintf(stderr, "%s: Child on PID %d\n", __func__, pid);
#endif
	} else {
		/*
		 * The child process.
		 */
		/*
		 * New session with the shell as process group leader.
		 */
		setsid();
		setpgid(0, 0);
		/*
		 * Opening slave side of the terminal.
		 */
		tty_handler = open(tty, O_RDWR);
		if (tty_handler < 0) {
			fprintf(stderr, "%s(): child can not open '%s'\n",
				__func__, tty);
			fflush(stderr);
			gtk_exit(EXIT_FAILURE);
		}
		/*
		 * Duplicating the terminal for the standard streams.
		 */
		dup2(tty_handler, STDIN_FILENO);
		dup2(tty_handler, STDERR_FILENO);
		dup2(tty_handler, STDOUT_FILENO);
		shellname = bg_conf_get_string("/settings/shell_name", "Bourne Again Shell");
		binary = bg_conf_get_string("/settings/shell_binary", "/bin/bash");
		if (strcmp(shellname, "Bourne Again Shell") == 0)
			start_bash(binary);
		else if (strcmp(shellname, "Korn Shell") == 0)
			start_ksh(binary);
		else if (strcmp(shellname, "Z Shell") == 0)
			start_zsh(binary);
		else 
			start_bash(binary);
	}
}

void
on_secondary_widget_realize            (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gchar *fontname;

#ifdef DEBUG_CALL
	g_message("%s(): Start.", __func__);
#endif
	/*
	 * Setting the default font.
	 */
	fontname = bg_conf_get_string("/settings/basefont", BASE_FONT);
	terminal_widget_set_font(widget, fontname, FALSE);
	g_free(fontname);	
	dnd_init_destination(widget);
}

void
on_secondary_widget_map                (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget     *main_widget;
	Terminal      *t;
#ifdef DEBUG_CALL
	g_message("%s(): Start.", __func__);
#endif
	
	main_widget = lookup_widget(widget, "main_widget");
	g_assert(main_widget != NULL);
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_assert(t != NULL);
	
	background_set_secondary_text_view(t->background,
			GTK_TEXT_VIEW(widget));
}


void
on_app1_map                            (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget     *main_widget, *secondary_widget;
	GtkTextBuffer *text_buffer;
	Terminal      *t;
#ifdef DEBUG_CALL
	g_message("%s(): Start.", __func__);
#endif
	
	main_widget = lookup_widget(widget, "main_widget");
	secondary_widget = lookup_widget(widget, "secondary_widget");
	g_assert(GTK_WIDGET_REALIZED(main_widget));
	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(main_widget));
	gtk_text_view_set_buffer(GTK_TEXT_VIEW(secondary_widget), text_buffer);

gboolean
on_main_widget_key_release_event         (GtkWidget	*widget,
					GdkEventKey	*event,
					gpointer	user_data)
{
	return on_terminal_key_release_event(widget, event, user_data);
}

gboolean
on_secondary_widget_key_release_event    (GtkWidget	*widget,
					GdkEventKey	*event,
					gpointer	user_data)
{
	return on_main_widget_key_release_event(widget, event, user_data);
}
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_object_set_data(G_OBJECT(secondary_widget), "terminal", t);
	
	background_set_secondary_text_view(t->background,
			GTK_TEXT_VIEW(secondary_widget));
}

gboolean
on_main_widget_key_press_event         (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	return on_terminal_key_press_event(widget, event, user_data);
}

gboolean
on_secondary_widget_key_press_event    (GtkWidget       *widget,
                                        GdkEventKey     *event,
                                        gpointer         user_data)
{
	return on_main_widget_key_press_event(widget, event, user_data);
}




gboolean
on_app1_delete_event                   (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	GtkWidget *main_widget;
	Terminal *t;
	
	main_widget = lookup_widget(GTK_WIDGET(widget), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(main_widget), "terminal");
	terminal_send_command(t, "exit", TRUE);
	return TRUE;
}

/***************************************************************************
 * Callback functions for the main_widget.                                 *
 *                                                                         *
 ***************************************************************************/
gboolean
on_main_widget_button_press_event(
		GtkWidget       *widget, 
		GdkEventButton  *event, 
		gpointer         user_data)
{
	Terminal        *t;
	Background 	*bg;

	t = g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	bg = t->background;
	g_assert(bg != NULL);
	
	background_button_press_event(bg, widget, event, user_data);
	gtk_widget_grab_focus(widget);
	return TRUE;
}

gboolean
on_secondary_widget_button_press_event (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	return on_main_widget_button_press_event(widget, event, user_data);
}


gboolean
on_main_widget_button_release_event    (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	Terminal        *t;
	Background      *bg;
	
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	bg = t->background;
	g_assert(bg != NULL);	
 
	background_button_release_event(bg, widget, event, user_data);
	return FALSE;
}

gboolean
on_secondary_widget_button_release_event
                                        (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
	return on_main_widget_button_release_event(widget, event, user_data);
}


gboolean
on_main_widget_motion_notify_event     (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
	Terminal        *t;
	Background      *bg;
	int              x, y; 
	GdkModifierType  state;
	
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	bg = t->background;
	g_assert(bg != NULL);	
	background_motion_notify_event(bg, widget, event, user_data);
	/*
	 * If we call this function we will get another motion notify
	 * next time. This is strange!
	 */
	gdk_window_get_pointer(event->window, &x, &y, &state);
	return TRUE;
}

gboolean
on_secondary_widget_motion_notify_event
                                        (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
	return on_main_widget_motion_notify_event(widget, event, user_data);
}

gboolean
on_main_widget_leave_notify_event      (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	Terminal        *t;
	Background      *bg;
	
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	bg = t->background;
	g_assert(bg != NULL);	
	
	background_leave_notify_event(bg, widget, event, user_data);
	return FALSE;
}

gboolean
on_secondary_widget_leave_notify_event (GtkWidget       *widget,
                                        GdkEventCrossing *event,
                                        gpointer         user_data)
{
	return on_main_widget_leave_notify_event(widget, event, user_data);
}

gboolean
on_main_widget_expose_event            (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
	background_expose_event(widget, event, user_data);
  	return FALSE;
}

gboolean
on_secondary_widget_expose_event       (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
#ifdef DEBUG
	g_message("%s(): start.", __func__);	
#endif
	/*
	 * FIXME: The expose event handlers must be prepared to draw on multiple
	 * GtkTextView widgets.
	 */
	on_main_widget_expose_event(widget, event, user_data);
	return FALSE;
}



GtkWidget*
create_history_combobox (gchar *widget_name, 
		gchar *string1, 
		gchar *string2,
                gint int1, 
		gint int2)
{
	GtkWidget*  combo = gtk_combo_box_entry_new();
	
	g_signal_connect ((gpointer) combo , "changed", G_CALLBACK (on_combobox_changed), NULL);
	return combo;
}


void
on_repeat_toolbutton_clicked           (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gchar *command;
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	
	command = g_strdup_printf("ls%s",
			bg_conf_get_default_ls_options(FALSE));
	terminal_send_command(t, command, TRUE);
	g_free(command);

}


void
on_abort_toolbutton_clicked            (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	gsize written;
	GError *error = NULL;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);

	g_io_channel_write_chars(t->shell_stream, "\032", -1, &written, &error);
}


void
on_home_toolbutton_clicked             (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	gchar *command;
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	terminal_send_command(t, "cd", TRUE);
	command = g_strdup_printf("ls%s",
			bg_conf_get_default_ls_options(FALSE));
	terminal_send_command(t, command, TRUE);
	g_free(command);
}


void
on_filesystem_radiotoolbutton_clicked  (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_network_radiotoolbutton_clicked     (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_programmer_radiotoolbutton_clicked  (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


/*****************************************************************************
 * The callback functions for the tools menu containing the basic UNIX       *
 * commands for novice users.                                                *
 *                                                                           *
 *****************************************************************************/
void
on_directory_listing_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_ls_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	terminal_send_command(t, "ls -p ", FALSE);
}


void
on_tree_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	terminal_send_command(t, "tree -FC ", FALSE);
}


void
on_basic_operations_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	
	UNIMPLEMENTED;
}


void
on_cp_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	terminal_send_command(t, _("cp SOURCE DEST "), FALSE);
}


void
on_dd_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");
	terminal_send_command(t, _("dd ibs=BYTES if=INPUT of=OUTPUT "), FALSE);
}


void
on_install_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mv_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_rm_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_shred_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_special_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_ln_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mkdir_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_rmdir_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mkfifo_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mknod_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_changing_file_attributes1_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_chgrp_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_chmod_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_chown_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_touch_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_disk_usage_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_df_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_du_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_stat_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_sync_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_user_information_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_who_activate                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_w_activate                          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_id_activate                         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_logname_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_whoami_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_group_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}



gboolean
on_app1_focus_out_event                (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  return FALSE;
}

/*****************************************************************************
 * Callbacks for toolbar buttons.                                            *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
void
on_radio_button_arrow_realize(
		GtkWidget *widget, 
		gpointer   user_data)
{
	GtkWidget *icon;
	icon = gtk_image_new_from_file(PIXMAP_DIR "stock_draw-selection.png");
	gtk_widget_show (icon);
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(widget), icon);
}

void
on_radio_button_arrow_toggled(GtkToggleToolButton *toggletoolbutton, 
		gpointer user_data)
{
	GtkWidget *main_widget;
	Terminal *t;

	if (!gtk_toggle_tool_button_get_active(toggletoolbutton))
		return;
	main_widget = lookup_widget(GTK_WIDGET(toggletoolbutton), 
			"main_widget");
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_assert(t != NULL && t->background != NULL);
	background_set_method(t->background, MethodGraphics);
}


void
on_radio_button_text_realize           (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *icon;
	icon = gtk_image_new_from_file(PIXMAP_DIR "stock_directcursor.png");
	gtk_widget_show (icon);
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(widget), icon);
}


void
on_radio_button_text_toggled           (GtkToggleToolButton *toggletoolbutton,
                                        gpointer         user_data)
{
	GtkWidget *main_widget;
	Terminal *t;

	if (!gtk_toggle_tool_button_get_active(toggletoolbutton))
		return;
	main_widget = lookup_widget(GTK_WIDGET(toggletoolbutton), 
			"main_widget");
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_assert(t != NULL && t->background != NULL);
	background_set_method(t->background, MethodEditor);
}


void
on_radio_button_objects_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *icon;
	icon = gtk_image_new_from_file(PIXMAP_DIR "stock_handles-simple.png");
	gtk_widget_show (icon);
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(widget), icon);
}


void
on_radio_button_objects_toggled(
		GtkToggleToolButton *toggletoolbutton, 
		gpointer user_data)
{
	GtkWidget *main_widget;
	Terminal *t;

	if (!gtk_toggle_tool_button_get_active(toggletoolbutton))
		return;
	main_widget = lookup_widget(GTK_WIDGET(toggletoolbutton), 
			"main_widget");
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_assert(t != NULL && t->background != NULL);
	background_set_method(t->background, MethodSelection);
}


gboolean
on_properties_window_focus_out_event   (GtkWidget       *widget,
                                        GdkEventFocus   *event,
                                        gpointer         user_data)
{
  	return FALSE;
}


void
on_slice_help_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}


void
on_slice_close_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	widget = lookup_widget(GTK_WIDGET(button), "slice_window");
	gtk_widget_destroy(widget);
}


void 
on_slice_text_view_realize(
		GtkWidget * widget, 
		gpointer user_data)
{
	GtkTextBuffer *text_buffer;
	GtkTextIter begin, end;
	
	g_message("%s(): start", __func__);
	gtk_text_view_set_editable(GTK_TEXT_VIEW(widget), FALSE);
	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_create_tag(text_buffer, "normal",
				   "family", "monospace",
				   //"foreground", "DarkGray",
				   NULL);
	
	gtk_text_buffer_get_start_iter(text_buffer, &begin);
	gtk_text_buffer_get_end_iter(text_buffer, &end);
	gtk_text_buffer_apply_tag_by_name(text_buffer, "normal", &begin, &end);
}



void
on_view_size_bars_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean initialized = FALSE;
	GtkWidget *text_view;
	Terminal *t;

	if (!initialized){
		initialized = TRUE;
	}else{
		text_view = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
		t = g_object_get_data(G_OBJECT(text_view), "terminal");
		background_toggle_view_flag(t->background, Sizebars);
		gtk_widget_queue_draw(text_view);
	}
}


void
on_view_window_frames_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean initialized = FALSE;
	if (!initialized)
		initialized = TRUE;
	else
		UNIMPLEMENTED;


}


void
on_view_syntax_highlight_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean initialized = FALSE;
	if (!initialized)
		initialized = TRUE;
	else
		UNIMPLEMENTED;

}


void
on_view_active_areas_activate(GtkMenuItem *menuitem, 
		gpointer user_data)
{
	GtkWidget *text_view;
	Terminal *t;
	
	g_message("%s(): start.", __func__);
	
	text_view = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = g_object_get_data(G_OBJECT(text_view), "terminal");
	background_toggle_view_flag(t->background, ActiveAreas);
	gtk_widget_queue_draw(text_view);
}


void
on_view_menu_areas_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean initialized = FALSE;
	if (!initialized)
		initialized = TRUE;
	else
		UNIMPLEMENTED;

}


void
on_view_entry_areas_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	static gboolean initialized = FALSE;
	if (!initialized)
		initialized = TRUE;
	else
		UNIMPLEMENTED;

}


void
on_view_font_settings_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	Terminal                *t;
	GtkWidget     		*main_widget, *secondary_widget;
	GtkWidget     		*dialog;
	gchar         		*fontname;
	
	main_widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	secondary_widget = lookup_widget(GTK_WIDGET(menuitem), "secondary_widget");
	t = g_object_get_data(G_OBJECT(main_widget), "terminal");
	g_assert(t != NULL);
	
	fontname = bg_conf_get_string("/settings/basefont", BASE_FONT);
	
	dialog = gtk_font_selection_dialog_new(_("Select Font"));
	gtk_font_selection_dialog_set_font_name(
			GTK_FONT_SELECTION_DIALOG(dialog),
			fontname);
	g_free(fontname);

	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_OK){
		gtk_widget_destroy(dialog);
		return;
	}

	/*
	 * Erasing everithing from the terminal.
	 */
	//terminal_sequence_erase(t);

	fontname = gtk_font_selection_dialog_get_font_name(GTK_FONT_SELECTION_DIALOG(dialog));

#ifdef DEBUG_CALL
	g_message("The font is: '%s'\n", fontname);
#endif
	terminal_widget_set_font(main_widget, fontname, TRUE);
	terminal_widget_set_font(secondary_widget, fontname, FALSE);
	
	/*
	 * Saving the new font setting.
	 */
	bg_conf_set_string("/settings/basefont", fontname);

	g_free(fontname);
	gtk_widget_destroy (dialog);
}




void
on_file_convert_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_convert_to_png_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_main_widget_insert_at_cursor        (GtkTextView     *textview,
                                        gchar           *string,
                                        gpointer         user_data)
{
#ifdef DEBUG_CALL
	g_message("%s(): Start.", __func__);
#endif
}



void
on_main_widget_drag_begin              (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gpointer         user_data)
{
#ifdef DEBUG_CALL
	g_message("%s(): start", __func__);
#endif
}

/*
 * The search window and its parts.
 *
 */
void
on_search_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *window;
	window = create_search_window();
	gtk_widget_show(window);
}

void
on_search_window_realize               (GtkWidget       *widget,
                                        gpointer         user_data)
{
	bg_conf_restore_window_coord(widget, "/settings/search_window");
}


void
on_cancel_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window = lookup_widget(GTK_WIDGET(button), "search_window");

	bg_conf_save_window_coord(window, "/settings/search_window");
	gtk_widget_destroy(window);
}


void
on_search_button_clicked               (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window = lookup_widget(GTK_WIDGET(button), "search_window");
	
	bg_conf_save_window_coord(window, "/settings/search_window");
	UNIMPLEMENTED;
	
	gtk_widget_destroy(window);
}


/*****************************************************************************
 * Toolbar for software projects.                                            *
 *                                                                           *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
void
on_show_software_project_toolbar_activate(
		GtkMenuItem     *menuitem, 
		gpointer         user_data)
{
	GtkWidget *toolbar;
	GtkCheckMenuItem *item;
	gboolean visible;
#ifdef DEBUG        
	g_message("%s(): start", __func__);
#endif

	item = GTK_CHECK_MENU_ITEM(menuitem);
	toolbar = lookup_widget(GTK_WIDGET(menuitem), "softw_proj_toolbar");
	visible = gtk_check_menu_item_get_active(item);
	
	if (visible)
		gtk_widget_show(toolbar->parent);
	else
		gtk_widget_hide(toolbar->parent);

	bg_conf_set_bool("/settings/sw_toolbar_visible", visible);
}

void
on_softw_proj_toolbar_realize(
		GtkWidget       *widget,
                gpointer         user_data)
{
	gboolean visible;
	GtkWidget *app;
	GtkCheckMenuItem *item;
	/*
	 * The software toolbar...
	 */
	visible = bg_conf_get_bool("/settings/sw_toolbar_visible", FALSE);
	if (visible)
		gtk_widget_show(widget->parent);
	else
		gtk_widget_hide(widget->parent);
	/* now we assume the menus are already realized. */
	item = (GtkCheckMenuItem *) lookup_widget(widget, "show_software_project_toolbar");
	if (item == NULL)
		return;
	gtk_check_menu_item_set_active(item, visible);
	/*
	 * Filling the bookmarks. This should not be here...
	 */
	app = lookup_widget(widget, "app1");
	add_bookmarks_to_menu(app);
}

void
on_latex_project_toolbar_realize       (GtkWidget       *widget,
                                        gpointer         user_data)
{
	gboolean visible;
	GtkWidget *app;
	GtkCheckMenuItem *item;
	/*
	 * The latex toolbar...
	 */
	visible = bg_conf_get_bool("/settings/latex_toolbar_visible", FALSE);
	if (visible)
		gtk_widget_show(widget->parent);
	else
		gtk_widget_hide(widget->parent);
	/* now we assume the menus are already realized. */
	item = (GtkCheckMenuItem *) lookup_widget(widget, "show_latex_project_toolbar");
	if (item == NULL)
		return;
	gtk_check_menu_item_set_active(item, visible);
	/*
	 * Filling the bookmarks. This should not be here...
	 */
	app = lookup_widget(widget, "app1");
	add_bookmarks_to_menu(app);
}



static void
_push_icon_to_toolbutton(GtkWidget *widget, gchar *filename)
{
	GtkWidget *icon;
	
	icon = gtk_image_new_from_file(filename);
	//icon = gtk_image_new_from_icon_name(filename,
	//		GTK_ICON_SIZE_SMALL_TOOLBAR);
	gtk_tool_button_set_icon_widget(GTK_TOOL_BUTTON(widget), icon);
	gtk_widget_show (icon);
}

void
on_make_toolbutton_realize             (GtkWidget       *widget,
                                        gpointer         user_data)
{
	//_push_icon_to_toolbutton(widget, PIXMAP_DIR "make_toolbutton.png");
}


void
on_make_toolbutton_clicked             (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	terminal_send_command(t, "make", TRUE);
}


void
on_configure_toolbutton_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	_push_icon_to_toolbutton(widget, PIXMAP_DIR "configure_toolbutton.png");
	//_push_icon_to_toolbutton(widget, PIXMAP_DIR "configure");
}


void
on_configure_toolbutton_clicked        (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	terminal_send_command(t, "./configure", TRUE);
}


void
on_make_dist_toolbutton_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	//_push_icon_to_toolbutton(widget, PIXMAP_DIR "makedist_toolbutton.png");
}


void
on_make_dist_toolbutton_clicked        (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	terminal_send_command(t, "make dist", TRUE);
}


void
on_make_install_toolbutton_realize     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	_push_icon_to_toolbutton(widget, PIXMAP_DIR "makeinstall_toolbutton.png");
}


void
on_make_install_toolbutton_clicked     (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	terminal_send_command(t, "make install", TRUE);
}


void
on_make_clean_toolbutton_clicked       (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	terminal_send_command(t, "make clean", TRUE);
}



/************************************************************************
 * color settings
 */
void
on_set_colors_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkTextBuffer *text_buffer;
	GtkWidget     *main_widget;
	GtkWidget     *window;

	main_widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	window = create_syntax_highlight_window();
	g_object_set_data(G_OBJECT(window), "main_widget", main_widget);

	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(main_widget));
	g_object_set_data(G_OBJECT(window), "text_buffer", text_buffer);
	gtk_widget_show(window);
}


void
on_syntax_highlight_window_realize     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *checkbox;
	
	checkbox = lookup_widget(widget, "use_highlight_checkbutton");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox),
                bg_conf_get_bool("/settings/syntax_highlight/enabled", TRUE));
	
	checkbox = lookup_widget(widget, "mouse_over_checkbutton");
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox),
                bg_conf_get_bool("/settings/syntax_highlight/highlight",TRUE));
}


void
on_syntax_highlight_close_button_clicked
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GtkWidget *window;
	window = lookup_widget(GTK_WIDGET(button),
			"syntax_highlight_window");
	gtk_widget_destroy(window);
}

static void
settings_enable(GtkWidget *widget, gboolean enable)
{
	GtkWidget *w;
	w = lookup_widget(widget, "color_colorselection");
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(widget, "background_colorselection");
	gtk_widget_set_sensitive(w, enable);
	w = lookup_widget(widget, "font_fontselection");
	gtk_widget_set_sensitive(w, enable);
}

void
on_style_categories_treeview_cursor_changed
                                        (GtkTreeView     *treeview,
                                        gpointer         user_data)
{
	GtkColorSelection *color_sel;	
	GtkColorSelection *bg_color_sel;	
	GtkWidget         *window;
	GtkTextBuffer     *text_buffer;
	GtkTextTagTable   *tag_table;
	GtkTextTag        *tag;
	GtkTreeSelection  *selection;
	GtkTreeModel      *model;
	GtkTreeIter        iter;
	gboolean           def, enabled;	
	gchar             *tag_name;
	GdkColor          *color;
	gboolean          *color_set;
	gchar             *conf_name;
	
	g_message("%s(): start", __func__);
	window = lookup_widget(GTK_WIDGET(treeview), "syntax_highlight_window");
	color_sel = (GtkColorSelection *) lookup_widget(GTK_WIDGET(treeview), "color_colorselection");
	bg_color_sel = (GtkColorSelection *) lookup_widget(GTK_WIDGET(treeview), 
			"background_colorselection");
	
	selection = gtk_tree_view_get_selection(treeview);
	gtk_tree_selection_get_selected(selection, &model, &iter);
	gtk_tree_model_get(model, &iter, 
			1, &enabled, 
			2, &def, 
			3, &conf_name, // FIXME: This should be freed.
			4, &tag_name,  // FIXME: This should be freed.
			-1);
	settings_enable(GTK_WIDGET(treeview), enabled && !def);
	/*
	 * If the tag name is NULL, then we set the color selectors to NULL 
	 * tag.
	 */
	if (tag_name == NULL){
		g_object_set_data(G_OBJECT(color_sel), "tag", NULL);
		return;
	}

	text_buffer = g_object_get_data(G_OBJECT(window), "text_buffer");
	g_assert(text_buffer != NULL);
	tag_table = gtk_text_buffer_get_tag_table(text_buffer);
	tag = gtk_text_tag_table_lookup(tag_table, tag_name);
	g_object_set_data(G_OBJECT(color_sel), "tag", tag);
	g_object_set_data(G_OBJECT(color_sel), "conf_name", conf_name);
	g_object_set_data(G_OBJECT(bg_color_sel), "tag", tag);
	g_object_set_data(G_OBJECT(bg_color_sel), "conf_name", conf_name);
	
	g_object_get(G_OBJECT(tag), 
			"foreground-set", &color_set,
			"foreground-gdk", &color, 
			NULL);
	if (color_set)
		gtk_color_selection_set_current_color(color_sel, color);
	
	g_object_get(G_OBJECT(tag), 
			"background-set", &color_set,
			"background-gdk", &color, 
			NULL);
	if (color_set)	
		gtk_color_selection_set_current_color(bg_color_sel, color);
}

void
on_color_colorselection_color_changed(
		GtkColorSelection *colorselection, 
		gpointer           user_data)
{
	GdkColor    color;
	GtkTextTag *tag;
	gchar      *conf_name;
	
	tag = g_object_get_data(G_OBJECT(colorselection), "tag");
	conf_name = g_object_get_data(G_OBJECT(colorselection), "conf_name");
	if (tag == NULL)
		return;

	gtk_color_selection_get_current_color(
			GTK_COLOR_SELECTION(colorselection),
                        &color);	
	g_object_set(G_OBJECT(tag), 
			"foreground-set", TRUE,
			"foreground-gdk", &color, 
			NULL);

	bg_conf_set_color(conf_name, &color);

	/*
	typedef struct {
		guint32 pixel;
  		guint16 red;
  		guint16 green;
  		guint16 blue;
	} GdkColor;
	*/
}

void
on_background_colorselection_color_changed(
		GtkColorSelection *colorselection,
                gpointer         user_data)
{
	GdkColor    color;
	GtkTextTag *tag;
	
	tag = g_object_get_data(G_OBJECT(colorselection), "tag");
	if (tag == NULL)
		return;

	gtk_color_selection_get_current_color(
			GTK_COLOR_SELECTION(colorselection),
                        &color);	
	g_object_set(G_OBJECT(tag), 
			"background-set", TRUE,
			"background-gdk", &color, 
			NULL);
}


void
on_style_categories_treeview_row_activated(
		GtkTreeView     *treeview, 
		GtkTreePath     *path, 
		GtkTreeViewColumn *column, 
		gpointer         user_data)
{
}

static void
enable_toggled(GtkCellRendererToggle * cell,
	       gchar * path_str, gpointer data)
{
	GtkTreeView *tree_view = GTK_TREE_VIEW(data);
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	gboolean enabled;
	gboolean def;

	model = gtk_tree_view_get_model(tree_view);
	gtk_tree_model_get_iter(model, &iter, path);
	enabled = gtk_cell_renderer_toggle_get_active(cell);
	enabled ^= 1;
	gtk_tree_store_set(GTK_TREE_STORE(model), &iter, 1, enabled, -1);
	gtk_tree_path_free(path);
	/*
	 * We enable the settings if the category is enabled but not with the
	 * default settings.
	 */
	gtk_tree_model_get(model, &iter, 2, &def, -1);
	settings_enable(GTK_WIDGET(tree_view), enabled && !def);
}


static void
default_toggled(GtkCellRendererToggle * cell,
	       gchar * path_str, gpointer data)
{
	GtkTreeView *tree_view = GTK_TREE_VIEW(data);
	GtkTreeModel *model;
	GtkTreeIter iter;
	GtkTreePath *path = gtk_tree_path_new_from_string(path_str);
	gboolean enabled;
	gboolean def;

	model = gtk_tree_view_get_model(tree_view);
	gtk_tree_model_get_iter(model, &iter, path);
	//gtk_tree_model_get(model, &iter, 1, &enabled, -1);
	def = gtk_cell_renderer_toggle_get_active(cell);
	def ^= 1;
	gtk_tree_store_set(GTK_TREE_STORE(model), &iter, 2, def, -1);
	gtk_tree_path_free(path);
	/*
	 * We enable the settings if the category is enabled but not with the
	 * default settings.
	 */
	gtk_tree_model_get(model, &iter, 1, &enabled, -1);
	settings_enable(GTK_WIDGET(tree_view), enabled && !def);
}


static void 
_add_header_to_categories_treeview(GtkTreeView *tree_view)
{
  GtkTreeViewColumn *column;
  GtkCellRenderer   *renderer;
#if 0
  GtkTreeModel *model;
  model = gtk_tree_view_get_model (tree_view);
#endif
 
  /* Column0: the name */
  renderer = gtk_cell_renderer_text_new ();
  column = gtk_tree_view_column_new_with_attributes(
		  _("Category"), 
		  renderer, "text", 0, NULL);
  gtk_tree_view_append_column(tree_view, column);
  
  /* Column1: enabled/disabled */
  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes(
		  _("Enabled"), 
		  renderer, "active", 1, NULL);
  g_signal_connect (renderer, "toggled",
		    G_CALLBACK (enable_toggled), tree_view);
  gtk_tree_view_append_column(tree_view, column);

  /* Column2: default */
  renderer = gtk_cell_renderer_toggle_new ();
  column = gtk_tree_view_column_new_with_attributes(
		  _("Default"), 
		  renderer, "active", 2, NULL);
  g_signal_connect (renderer, "toggled",
		    G_CALLBACK (default_toggled), tree_view);
  
  gtk_tree_view_append_column(tree_view, column);
}

#define CONF_PREFIX "/settings/syntax_highlight"
static void 
_add_store_to_categories_treeview(GtkTreeView *tree_view)
{
	GtkTreeStore *tree_store = gtk_tree_store_new(5, 
			G_TYPE_STRING,  /* Description */ 
			G_TYPE_BOOLEAN, /* Enabled     */
			G_TYPE_BOOLEAN, /* Default     */
			G_TYPE_STRING,  /* GConfd name */
			G_TYPE_STRING   /* tag name    */
			);
	GtkTreeIter iter_level0;
	GtkTreeIter iter_level1;
	GtkTreeIter iter_level2;
	GtkTreeIter iter_level3;
  
	gtk_tree_store_append(tree_store, &iter_level0, NULL);
	gtk_tree_store_set(tree_store, &iter_level0, 
	  	0, _("Syntax highlight"), 
	  	1, bg_conf_get_bool(CONF_PREFIX "/enabled", TRUE), 
	  	2, FALSE,
	  	3, CONF_PREFIX "/enabled",
		4, NULL,
	  	-1);
	
	gtk_tree_store_append(tree_store, &iter_level1, &iter_level0);
	gtk_tree_store_set(tree_store, &iter_level1, 
		0, _("Directory entries"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/enable",
		4, NULL,
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Directory name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/dir/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/dir/enable",
		4, "directory",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/dir/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/dir/highlight",
		4, "directory_active",
		-1);
	    
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Executable file name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/exec/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/exec/enable",
		4, "executable",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/exec/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/exec/highlight",
		4, "executable_active",
		-1);
	    
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Archive file name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/arch/enable", TRUE),
		2, FALSE, 
		3, CONF_PREFIX "/dir_ent/arch/enable",
		4, "archive",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/arch/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/arch/highlight",
		4, "archive_active",
		-1);
	  
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Block device file name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/block/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/block/enable",
		4, "block_device",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/block/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/block/highlight",
		4, "block_device_active",
		-1);
	  
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Other file names"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/other/enable", TRUE),
		3, CONF_PREFIX "/dir_ent/other/enable",
		4, "file",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/other/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/dir_ent/other/highlight",
		4, "file_active",
		-1);
	/*
	 * User and group names
	 */
	gtk_tree_store_append(tree_store, &iter_level1, &iter_level0);
	gtk_tree_store_set(tree_store, &iter_level1, 
		0, _("User and group names"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/user_group/enable",
		4, NULL,
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("User name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/user_group/user/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/user_group/user/enable",
		4, "username",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/dir/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/user_group/user/highlight",
		4, "username_active",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Group name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/user_group/user/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/user_group/group/enable",
		4, "groupname",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/dir/highlight", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/user_group/group/highlight",
		4, "groupname_active",
		-1);
	/*
	 * Logical values.
	 */
	gtk_tree_store_append(tree_store, &iter_level1, &iter_level0);
	gtk_tree_store_set(tree_store, &iter_level1, 
		0, _("Logical values"), 
		1, bg_conf_get_bool(CONF_PREFIX "/log_val/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/log_val/enable",
		4, NULL,
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("True/Yes"), 
		1, bg_conf_get_bool(CONF_PREFIX "/log_val/true/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/log_val/true/enable",
		4, "yes",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/log_val/true/highlight/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/log_val/true/highlight/enable",
		4, "yes_active",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("False/No"), 
		1, bg_conf_get_bool(CONF_PREFIX "/log_val/false/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/log_val/false/enable",
		4, "no",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level3, &iter_level2);
	gtk_tree_store_set(tree_store, &iter_level3, 
		0, _("Highlighted"), 
		1, bg_conf_get_bool(CONF_PREFIX "/log_val/false/highlight/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/log_val/false/highlight/enable",
		4, "no_active",
		-1);

	/*
	 * BASH Syntax Highlight 
	 */
	gtk_tree_store_append(tree_store, &iter_level1, NULL);
	gtk_tree_store_set(tree_store, &iter_level1, 
		0, _("BASH Syntax Highlight"), 
		1, bg_conf_get_bool(CONF_PREFIX "/dir_ent/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/user_group/enable",
		4, NULL,
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Command or Program Name"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/command/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/command/enable",
		4, "command_name",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Option"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/command/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/option/enable",
		4, "option",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Argument"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/argument/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/argument/enable",
		4, "argument",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Variable"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/variable/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/variable/enable",
		4, "variable",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Variable Substitution"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/varvalue/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/varvalue/enable",
		4, "varvalue",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("Keyword"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/keyword/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/keyword/enable",
		4, "keyword",
		-1);
	
	gtk_tree_store_append(tree_store, &iter_level2, &iter_level1);
	gtk_tree_store_set(tree_store, &iter_level2, 
		0, _("List Item"), 
		1, bg_conf_get_bool(CONF_PREFIX "/bash/list/enable", TRUE),
		2, FALSE,
		3, CONF_PREFIX "/bash/list/enable",
		4, "list_element",
		-1);
	
	gtk_tree_view_set_model(tree_view, GTK_TREE_MODEL(tree_store));
}


void
on_style_categories_treeview_realize   (GtkWidget       *widget,
                                        gpointer         user_data)
{
	_add_store_to_categories_treeview(GTK_TREE_VIEW(widget));
	_add_header_to_categories_treeview(GTK_TREE_VIEW(widget));
	gtk_tree_view_expand_all(GTK_TREE_VIEW(widget));
}






void
on_show_latex_project_toolbar_activate (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *toolbar;
	GtkCheckMenuItem *item;
	gboolean visible;
#ifdef DEBUG        
	g_message("%s(): start", __func__);
#endif

	item = GTK_CHECK_MENU_ITEM(menuitem);
	toolbar = lookup_widget(GTK_WIDGET(menuitem), "latex_project_toolbar");
	visible = gtk_check_menu_item_get_active(item);
	
	if (visible)
		gtk_widget_show(toolbar->parent);
	else
		gtk_widget_hide(toolbar->parent);

	bg_conf_set_bool("/settings/latex_toolbar_visible", visible);
}


void
on_tex_files_toolbuton_realize         (GtkWidget       *widget,
                                        gpointer         user_data)
{
	_push_icon_to_toolbutton(widget, PIXMAP_DIR "latex.png");
}


void
on_tex_files_toolbuton_clicked         (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}


void
on_dvi_files_toolbutton_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_dvi_files_toolbutton_clicked        (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}


void
on_ps_files_toolbutton_realize         (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_ps_files_toolbutton_clicked         (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}


void
on_pdf_files_toolbutton_realize        (GtkWidget       *widget,
                                        gpointer         user_data)
{

}


void
on_pdf_files_toolbutton_clicked        (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{

}



void
on_edit_bookmarks_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *app;
	GtkWidget *window;
	
	window = create_edit_bookmarks_window();
	app = lookup_widget(GTK_WIDGET(menuitem), "app1");
	g_object_set_data(G_OBJECT(window), "app", app);
	
	gtk_widget_show(window);
}


void
on_show_history_window_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *app;
	GtkWidget *window;
	
	window = create_history_window();
	app = lookup_widget(GTK_WIDGET(menuitem), "app1");
	g_object_set_data(G_OBJECT(window), "app", app);
	
	gtk_widget_show(window);

}


void
on_shell_combobox_changed              (GtkComboBox     *combobox,
                                        gpointer         user_data)
{
	gchar *name = gtk_combo_box_get_active_text(combobox);
	GtkWidget *chooser = lookup_widget(GTK_WIDGET(combobox), 
			"shell_filechooserbutton");
	gchar *executable = NULL;

	if (strcmp(name, "Bourne Again Shell") == 0)
		executable = "/bin/bash";
	else if (strcmp(name, "Korn Shell") == 0)
		executable = "/bin/ksh";
	else if (strcmp(name, "Z Shell") == 0)
		executable = "/bin/zsh";

	if (executable != NULL)
		gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(chooser), 
				executable);
}


void
on_shell_filechooserbutton_realize     (GtkWidget       *widget,
                                        gpointer         user_data)
{
	/*
	 * The default is the /bin/bash
	 */
	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER(widget), "/bin");
	gtk_file_chooser_set_filename(GTK_FILE_CHOOSER(widget), "/bin/bash");
}


void
on_shell_combobox_realize              (GtkWidget       *widget,
                                        gpointer         user_data)
{
	/*
	 * The default is the BASH shell.
	 */
	gtk_combo_box_set_active(GTK_COMBO_BOX(widget), 0);
}


void
on_different_language_toggled          (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_text_editor_toggled                 (GtkToggleButton *togglebutton,
                                        gpointer         user_data)
{

}


void
on_gui_completion_button_clicked       (GtkToolButton   *toolbutton,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal  *t;	
	
	widget = lookup_widget(GTK_WIDGET(toolbutton), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	//g_thread_create((GThreadFunc)history_read_bash_history, 
	//		t, FALSE, NULL);
	gui_completion_activate(t);
	//cui_completion_activate(t);
	//terminal_send_command(t, "command", FALSE);
}


void
on_command_dialog_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	
	GtkWidget *widget;
	Terminal  *t;	
	
	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = (Terminal *) g_object_get_data(G_OBJECT(widget), "terminal");

	//g_thread_create((GThreadFunc)history_read_bash_history, 
	//		t, FALSE, NULL);
	gui_completion_activate(t);
	//cui_completion_activate(t);
	//terminal_send_command(t, "command", FALSE);
}


void
on_app1_realize                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	/*
	 * Reading the .bash_history file from the users home directory.
	 */
	g_thread_create((GThreadFunc)history_read_bash_history, widget, FALSE, NULL);
	//history_read_bash_history(widget);
}


GtkWidget*
cui_terminal_widget_new (gchar *widget_name, gchar *string1, gchar *string2,
                gint int1, gint int2)
{
	GtkWidget *widget;
	//widget = gtk_text_view_new(); 
	widget = cui_terminal_new();
}


void
on_new2_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_open2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_save_as2_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_quit2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_cut2_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_copy2_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_paste2_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_clear2_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_preferences2_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about2_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}



void
on_save_as3_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}



