%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: cat_lexer.l,v 1.1 2006/02/24 09:07:05 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: cat_lexer.l,v $
 * Revision 1.1  2006/02/24 09:07:05  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define DEBUG

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "cat_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%option yylineno

%%
	cui_window_set_title(window_for_lex, _("File-system Table"));
#.* {
	#ifdef DEBUG
	g_message("%s(): COMMENT: '%s'", __func__, yytext);
	#endif
	cat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		cat_lval.sval.text,
		&cat_lval.sval.begin,
		&cat_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "comment", 
		cat_lval.sval.begin, 
		cat_lval.sval.end); 
	return COMMENT;
  }


^[A-Za-z]+/= {
	#ifdef DEBUG
	g_message("%s(): LABEL: '%s'", __func__, yytext);
	#endif
	cat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		cat_lval.sval.text,
		&cat_lval.sval.begin,
		&cat_lval.sval.end);
	return LABEL;
}

\/[A-Za-z0-9/]* {
	#ifdef DEBUG
	g_message("%s(): FILENAME: '%s'", __func__, yytext);
	#endif
	cat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		cat_lval.sval.text,
		&cat_lval.sval.begin,
		&cat_lval.sval.end);
	return FILENAME;
}

[A-Za-z0-9/]+ {
	#ifdef DEBUG
	g_message("%s(): WORD: '%s'", __func__, yytext);
	#endif
	cat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		cat_lval.sval.text,
		&cat_lval.sval.begin,
		&cat_lval.sval.end);
	return WORD;
}

[ \t]+ {
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}


\n {
	#ifdef DEBUG
	g_message("%s(): NEW_LINE: '%s'", __func__, yytext);
	#endif
	cat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		cat_lval.sval.text,
		&cat_lval.sval.begin,
		&cat_lval.sval.end);
	return NEW_LINE;
}

<<EOF>> {
	return FILE_END;
}

. {
	#ifdef DEBUG
	g_message("%s(): OTHER_CAR: '%s'", __func__, yytext);
	#endif
	cat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		cat_lval.sval.text,
		&cat_lval.sval.begin,
		&cat_lval.sval.end);
	return yytext[0];
}


%%
