/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: command_generator.c,v 1.1 2006/02/24 17:23:54 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: command_generator.c,v $
 * Revision 1.1  2006/02/24 17:23:54  pipas
 * Initial revision
 *
 */
#define _GNU_SOURCE 
/* FIXME: Why do I need this? */
#ifndef FNM_CASEFOLD
  #define FNM_CASEFOLD	 (1 << 4)
#endif

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#undef DEBUG_GENERATOR	

#include <gnome.h>
#include "configure.h"
#include "filename.h"
#include "command_generator.h"
#include "cuiwindow.h"

#define PROPERTY_OS 1
#define PROPERTY_TERMINAL 2

static void
cui_generator_set_property(GObject *object, 
		guint           property_id, 
		const GValue    *value, 
		GParamSpec      *pspec)
{
	CuiGenerator *self = (CuiGenerator *) object;
	
	//g_message("%s(): start: %05u", __func__, property_id);
	switch(property_id){
		case PROPERTY_OS:
			g_free(self->os);
			self->os = g_value_dup_string(value);
			break;
		case PROPERTY_TERMINAL:
			self->terminal = g_value_get_pointer(value);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(
					object, property_id, pspec);
	}
}


static void
cui_generator_get_property(GObject *object, 
		guint            property_id, 
		GValue           *value, 
		GParamSpec       *pspec)
{
	CuiGenerator *self = (CuiGenerator *) object;
	
	switch(property_id){
		case PROPERTY_OS:
			g_value_set_string(value, self->os);
			break;
		case PROPERTY_TERMINAL:
			g_value_set_pointer(value, self->terminal);
			break;
		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(
					object, property_id, pspec);
	}
}

static void 
cui_generator_class_init(
		gpointer g_class, 
		gpointer g_class_data) 
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
	CuiGeneratorClass *klass = CUI_GENERATOR_CLASS(g_class);
	GParamSpec *pspec;
	
	//g_message("%s(): start.", __func__);
	gobject_class->set_property = cui_generator_set_property;
  	gobject_class->get_property = cui_generator_get_property;

	/*
	 * Properties.
	 */
	pspec = g_param_spec_string (
			"os", "operating-system",
                        "The name of the operating system.",
                        "Linux",
                        G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROPERTY_OS, pspec);
	
	pspec = g_param_spec_pointer (
			"terminal", "cui-terminal",
                        "The terminal to send the generated commands.",
                        G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROPERTY_TERMINAL, 
			pspec);
}

static void
cui_generator_instance_init(
		GTypeInstance   *instance, 
		gpointer         g_class){
	CuiGenerator *self = (CuiGenerator *)instance;
	CuiGeneratorClass *klass = (CuiGeneratorClass *)g_class;
}

GType 
cui_generator_get_type(void) 
{
	static GType type = 0;
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof(CuiGeneratorClass),
			(GBaseInitFunc) NULL, 
			(GBaseFinalizeFunc) NULL,                  
			(GClassInitFunc) cui_generator_class_init,   
			NULL,	
			NULL,
			sizeof(CuiGenerator),
			0,
			(GInstanceInitFunc) cui_generator_instance_init 
		};
		type = g_type_register_static(G_TYPE_OBJECT, 
				"CuiGenerator", &info, 0);
	}
	return type;
}

/**
 * cui_generator_new:
 * @returns: A newly allocated and initialized command generator
 *
 */
CuiGenerator *
cui_generator_new(void)
{
	CuiGenerator *gen;
	gen = g_object_new(CUI_GENERATOR_TYPE, NULL);
	return gen;
}

/**
 * cui_generator_set_terminal:
 * @generator: a #CuiGenerator
 * @terminal: a #Terminal we send the commands to
 *
 * Sets the terminal for the command generator.
 */
void
cui_generator_set_terminal(CuiGenerator *generator, const gpointer *terminal)
{
	GValue val = {0,};
	g_return_if_fail(CUI_IS_GENERATOR(generator));
	g_value_init(&val, G_TYPE_POINTER);
	g_value_set_pointer (&val, (gpointer)terminal);
	g_object_set_property (G_OBJECT(generator), "terminal", &val);
}


/**
 * cui_generator_send_shell_command:
 * @generator: a #CuiGenerator
 * @command: a simple shell command
 *
 * Sends a command directly to the terminal. This function does not modify the
 * string just sends it to execute. 
 */
void
cui_generator_send_shell_command(
		CuiGenerator *generator, 
		const char *command)
{
	g_return_if_fail(CUI_IS_GENERATOR(generator) && command != NULL);
	terminal_send_command(generator->terminal, (gchar *)command, TRUE);

}

/*
 * Individual command generators:
 */

static gchar *
generate_command_copy(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	gchar *fullname;

	fullname = g_build_filename(file->directory, file->filename, NULL);
	if (cui_filename_is_dir(file))
		command = g_strdup_printf("PASTE_COMMAND=\"cp -r %s .\"", 
				fullname);
	else
		command = g_strdup_printf("PASTE_COMMAND=\"cp %s .\"", 
				fullname);

	g_free(fullname);
	return command;
}

static gchar *
generate_command_cut(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	gchar *fullname;

	fullname = g_build_filename(file->directory, file->filename, NULL);
	command = g_strdup_printf("PASTE_COMMAND=\"mv %s .\"", fullname);
	g_free(fullname);
	return command;
}

static gchar *
generate_command_paste(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	gchar *fullname;

	command = g_strdup("$PASTE_COMMAND");
	return command;
}

static gchar *
generate_command_file_delete(CuiGenerator *generator,
		CuiFilename *file,
		Terminal *terminal)
{
	if (cui_filename_is_dir(file))
		return "rm -fr";
	else
		return "rm -f";
}

/*
 * The return value is a newly allocated string which should be freed.
 *
 * Knoledge base: We will not try to create the ~/.Trash if we did it already.
 */
static gchar *
generate_command_move_trash(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	static gboolean trash_ready = FALSE;
	gchar *command;
	
	if (!trash_ready){
		command = "[ -d ~/.Trash ] || mkdir ~/.Trash";
		terminal_send_command(terminal, command, TRUE);
		trash_ready = TRUE;
	}

	return g_strdup_printf("mv %s ~/.Trash", filename);
}

/*
 * File open command generator functions.
 */
static gchar *
generate_command_open(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	gchar *tmp;
	/*
	 * If the file is a directory entry we change the working directory and
	 * list the entries in it.
	 */
	if (cui_filename_is_dir(file)){
		command = g_strdup_printf("cd %s", filename);
		terminal_send_command(terminal, command, TRUE);
		g_free(command);
		
		tmp = bg_conf_get_default_ls_options(TRUE);
		command = g_strdup_printf("ls%s", tmp);
		g_free(tmp);
		
		return command;
	}
	
	/*
	 * Otherwise if we can process the file with running it with some
	 * application, we do that.
	 */
	command = cui_filename_get_processor(file, filename);
	if (command != NULL)
		return command;	
	/*
	 * Otherwise if we can open the file with some editor, we do that.
	 */
	command = cui_filename_get_editor(file, filename);
	if (command != NULL)
		return command;
	/*
	 * Otherwise we try to open it with a viewer application.
	 */
	command = cui_filename_get_viewer(file, filename);
	if (command != NULL)
		return command;
	/*
	 * As a failsafe we open the file with the default text editor for
	 * it can open _any_ file.
	 */
	return g_strconcat(bg_conf_get_default_text_editor(), 
			" ", filename, NULL); 
}

static gchar *
generate_command_edit(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	
	if (cui_filename_is_dir(file)){
		command = g_strdup_printf("cd %s", filename);
		terminal_send_command(terminal, command, TRUE);
		g_free(command);
		return g_strdup("ls -lh");
	}
	
	command = cui_filename_get_editor(file, filename);
	if (command != NULL)
		return command;
	else
		return g_strconcat(bg_conf_get_default_text_editor(), 
				" ", filename, NULL); 
}

static gchar *
generate_command_view(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	
	if (cui_filename_is_dir(file)){
		command = g_strdup_printf("cd %s", filename);
		terminal_send_command(terminal, command, TRUE);
		g_free(command);
		return g_strdup("ls -lh");
	}
	
	command = cui_filename_get_viewer(file, filename);
	if (command != NULL)
		return command;
	else
		return g_strdup_printf("gview %s", filename);
}

static gchar *
generate_command_process(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *command;
	
	if (cui_filename_is_dir(file)){
		command = g_strdup_printf("cd %s", filename);
		terminal_send_command(terminal, command, TRUE);
		g_free(command);
		return g_strdup("ls -lh");
	}else{
		return cui_filename_get_processor(file, filename);
	}
}

/*
 * Knowledge base: Uncompress commands for the following name patterns:
 *   o *.tar.gz
 *   o *.tar.bz2
 *   o *.tgz
 *   o *.tar
 */
static gchar *
generate_command_uncompress(CuiGenerator *generator,
		CuiFilename *file,
		gchar *filename,
		Terminal *terminal)
{
	gchar *name;
	gchar *command;
	
	name = cui_filename_get_filename(file);
	/*
	 * Files uncompressable by tar.
	 */
	if (fnmatch("*.tar.gz", name, FNM_CASEFOLD) == 0 ||
			fnmatch("*.tgz", name, FNM_CASEFOLD) == 0)
		return g_strdup_printf("tar xzf %s", name);
	
	if (fnmatch("*.tar.bz2", name, FNM_CASEFOLD) == 0){
		return g_strdup_printf("tar xjf %s", name);
	}
	
	if (fnmatch("*.tar", name, FNM_CASEFOLD) == 0)
		return g_strdup_printf("tar xf %s", name);
	
	if (fnmatch("*.gz", name, FNM_CASEFOLD) == 0)
		return g_strdup_printf("gunzip %s", name);

	if (fnmatch("*.bz2", name, FNM_CASEFOLD) == 0)
		return g_strdup_printf("bunzip2 %s", name);

	if (fnmatch("*.zip", name, FNM_CASEFOLD) == 0)
		return g_strdup_printf("unzip %s", name);

	return g_strdup_printf("");
}

/**
 * cui_generator_emit_file_command:
 * @generator: a #CuiGenerator
 * @command: the code of the command to send
 * @file: the file argument for the command
 *
 * Sends a file command with the command generator to the shell.
 */
void
cui_generator_emit_file_command(
		CuiGenerator *generator,
		CuiCommandOpcode command,
		CuiFilename *file)
{
	Terminal *terminal;
	gchar *filename;
	gchar *commandline = NULL;
	gchar *prot;
	/*
	 * Deciding if we need to use the whole path or it is enough to use the
	 * basename of the file.
	 */
	terminal = generator->terminal;
	filename = cui_filename_get_canonical_filename(file, 
			terminal->wd->str);
	if (*filename == '-')
		prot = "-- ";
	else
		prot = "";
	/*
	 * Generating the command.
	 */
	switch (command){
		case FileOpen:
			commandline = generate_command_open(
					generator, 
					file, 
					filename,
					terminal);
			break;
		case FileRefresh:
			commandline = g_strdup_printf("touch %s%s", 
					prot,
					filename);
			break;

		case FileOpenForEdit:
			commandline = generate_command_edit(
					generator, 
					file, 
					filename,
					terminal);
			break;
		
		case FileOpenForView:
			commandline = generate_command_view(
					generator, 
					file, 
					filename,
					terminal);
			break;
		
		case FileProcess:
			commandline = generate_command_process(
					generator, 
					file, 
					filename,
					terminal);
			break;
			
		case FileOpenWithVim:
			commandline = g_strdup_printf("gvim %s%s", 
					prot,
					filename);
			break;
			
		case FileOpenWithEmacs:
			commandline = g_strdup_printf("emacs %s%s", 
					prot,
					filename);
			break;
			
		case FileNewRegularFile:
			commandline = g_strdup_printf("touch NewFile");
			break;
			
		case FileNewDirectory:
			commandline = g_strdup_printf("mkdir NewDirectory");
			break;
			
		case FileDelete:
			commandline = g_strdup_printf("%s %s", 
				generate_command_file_delete(generator, 
					file,
					terminal),
				filename);
			break;
			
		case FileMoveToTrash:
			commandline = generate_command_move_trash(
					generator, 
					file, 
					filename,
					terminal);
			break;
			
		case FileUncompress:
			commandline = generate_command_uncompress(
					generator, 
					file, 
					filename,
					terminal);
			break;
			
		case FileNewTarArchive:
			commandline = g_strdup_printf("tar cf %s.tar %s", 
					filename, filename);
			break;
		
		case FileNewTarGzArchive:
			if (cui_filename_is_dir(file)) {
				commandline = g_strdup_printf(
						"tar czf %s.tar.gz %s", 
						filename, filename);
			}else{
				commandline = g_strdup_printf(
						"gzip %s", filename);
			}
			break;
		
		case FileNewTarBzArchive:
			if (cui_filename_is_dir(file)) {
				commandline = g_strdup_printf(
						"tar cjf %s.tar.bz2 %s", 
						filename, filename);
			}else{
				commandline = g_strdup_printf(
						"bzip2 %s", filename);
			}
			break;
		
		case FileShowProperties:
			commandline = g_strdup_printf("stat %s", filename);
			break;
		
		case FileClipBoardCut:
			commandline = generate_command_cut(
					generator, 
					file, 
					filename,
					terminal);
			break;
			
		case FileClipBoardCopy:
			commandline = generate_command_copy(
					generator, 
					file, 
					filename,
					terminal);
			break;
			
		case FileClipBoardPaste:
			commandline = generate_command_paste(
					generator, 
					file, 
					filename,
					terminal);
			break;
		/*
		 * Mode setting commands...
		 */
		case FileSetModeToFile:	
			commandline = g_strdup_printf("chmod 644 %s", filename);
			break;
		
		case FileSetModeToDir:	
			commandline = g_strdup_printf("chmod 700 %s", filename);
			break;
		
		case FileSetModeToProgram:	
			commandline = g_strdup_printf("chmod 755 %s", filename);
			break;
			
		case FileGrantReadUser:
			commandline = g_strdup_printf("chmod u+r %s", filename);
			break;
			
		case FileRevokeReadUser:
			commandline = g_strdup_printf("chmod u-r %s", filename);
			break;
			
		case FileGrantWriteUser:
			commandline = g_strdup_printf("chmod u+w %s", filename);
			break;
			
		case FileRevokeWriteUser:
			commandline = g_strdup_printf("chmod u-w %s", filename);
			break;
			
		case FileGrantExecuteUser:
			commandline = g_strdup_printf("chmod u+x %s", filename);
			break;
			
		case FileRevokeExecuteUser:
			commandline = g_strdup_printf("chmod u-x %s", filename);
			break;
			
		case FileGrantReadGroup:
			commandline = g_strdup_printf("chmod g+r %s", filename);
			break;
			
		case FileRevokeReadGroup:
			commandline = g_strdup_printf("chmod g-r %s", filename);
			break;
			
		case FileGrantWriteGroup:
			commandline = g_strdup_printf("chmod g+w %s", filename);
			break;
			
		case FileRevokeWriteGroup:
			commandline = g_strdup_printf("chmod g-w %s", filename);
			break;
			
		case FileGrantExecuteGroup:
			commandline = g_strdup_printf("chmod g+x %s", filename);
			break;
			
		case FileRevokeExecuteGroup:
			commandline = g_strdup_printf("chmod g-x %s", filename);
			break;
			
		case FileGrantReadOthers:
			commandline = g_strdup_printf("chmod o+r %s", filename);
			break;
			
		case FileRevokeReadOthers:
			commandline = g_strdup_printf("chmod o-r %s", filename);
			break;
			
		case FileGrantWriteOthers:
			commandline = g_strdup_printf("chmod o+w %s", filename);
			break;
			
		case FileRevokeWriteOthers:
			commandline = g_strdup_printf("chmod o-w %s", filename);
			break;
			
		case FileGrantExecuteOthers:
			commandline = g_strdup_printf("chmod o+x %s", filename);
			break;
			
		case FileRevokeExecuteOthers:
			commandline = g_strdup_printf("chmod o-x %s", filename);
			break;
		
		case FileViewACLList:
			commandline = g_strdup_printf("getfacl %s%s", 
					prot,
					filename);
			break;

		default:
			g_warning("%s(): Unknown command code: %d",
					__func__, command);
			g_return_if_reached();
	}
	/*
	 * Sending the command to the terminal.
	 */
	terminal_send_command(terminal, commandline, TRUE);
	g_free(filename);
	g_free(commandline);
}


/*
 * Knowledge base:
 *   o The commands can handle dash at the begin of the filenames.
 */
void
cui_generator_emit_file_command_with_arg(
		CuiGenerator *generator,
		CuiCommandOpcode command,
		CuiFilename *file,
		const gchar *text)
{
	Terminal *terminal;
	gchar *filename;
	gchar *newname;
	gchar *commandline = NULL;
	/*
	 * Deciding if we need to use the whole path or it is enough to use the
	 * basename of the file.
	 */
	terminal = generator->terminal;
	filename = cui_filename_get_canonical_filename(file, 
			terminal->wd->str);
	newname  = cui_filename_get_canonical_for_newname(file, 
			terminal->wd->str,
			(gchar *)text);
	/*
	 * Creating the command.
	 */
	switch (command){
		/*
		 * Moving, copying, renaming etc.
		 */
		case FileRename:
			if (*filename == '-' || *newname == '-')
				commandline = g_strdup_printf("mv -- %s %s", 
						filename, 
						newname);
			else
				commandline = g_strdup_printf("mv %s %s", 
						filename, 
						newname);

			break;
		case FileCopy:
			if (cui_filename_is_dir(file))
				commandline = g_strdup_printf("cp -r %s %s",
						filename,
						text);
			else
				commandline = g_strdup_printf("cp %s %s",
						filename,
						text);
			break;
			
		case FileCopyMode:
			if (cui_filename_is_dir(file))
				commandline = g_strdup_printf("chmod -R --reference=%s %s", filename, text);
			else
				commandline = g_strdup_printf("chmod --reference=%s %s", filename, text);
			break;
			
		case FileCopyOwner:
			if (cui_filename_is_dir(file))
				commandline = g_strdup_printf("chown -R --reference=%s %s", filename, text);
			else
				commandline = g_strdup_printf("chown --reference=%s %s", filename, text);
			break;
			
		case FileCopyGroupOwner:
			if (cui_filename_is_dir(file))
				commandline = g_strdup_printf("chgrp -R --reference=%s %s", filename, text);
			else
				commandline = g_strdup_printf("chgrp --reference=%s %s", filename, text);
			break;
			
		case FileMove:
			commandline = g_strdup_printf("mv %s %s",
						filename,
						text);
			break;
		/*
		 * Changing owner and group owner.
		 */
		case FileChangeOwner:
			if (*filename == '-')
				commandline = g_strdup_printf("chown %s -- %s", 
						text,
						filename);
			else
				commandline = g_strdup_printf("chown %s %s", 
						text,
						filename);
			break;

		case FileChangeGroup:
			if (*filename == '-')
				commandline = g_strdup_printf("chgrp %s -- %s", 
						text,
						filename);
			else
				commandline = g_strdup_printf("chgrp %s %s", 
						text,
						filename);
				
			break;
		/*
		 * Changing dates.
		 */
		case FileChangeAccessDate:
			if (*filename == '-')
				commandline = g_strdup_printf(
						"touch -a --date=%s -- %s", 
						text,
						filename);
			else
				commandline = g_strdup_printf(
						"touch -a --date=%s %s", 
						text,
						filename);
			break;
		
		case FileChangeModifyDate:
			if (*filename == '-')
				commandline = g_strdup_printf(
						"touch -m --date=%s -- %s", 
						text,
						filename);
			else
				commandline = g_strdup_printf(
						"touch -m --date=%s %s", 
						text,
						filename);
			break;
		
		case FileChangeChangeDate:
			commandline = g_strdup_printf(
					"touch --date=%s %s", 
					text,
					filename);
			g_warning("%s(): There is no command for changing "
					"only the inode modification time!",
					__func__);
			break;
		/*
		 * Changing ACL entries.
		 */
		case FileChangeACL:
			commandline = g_strdup_printf("setfacl -m %s %s",
					text, filename);
			break;
		case FileOpenWithLine:
			commandline = g_strdup_printf("gvim +%s %s",
					text, filename);
			break;
		default:
			g_warning("%s(): Unknown command code: %d",
					__func__, command);
			g_return_if_reached();
	}
	/*
	 * Sending the command to the terminal.
	 */
	terminal_send_command(terminal, commandline, TRUE);
	g_free(filename);
	g_free(commandline);
}

void
cui_generator_emit_window_command(
		CuiGenerator *generator,
		CuiCommandOpcode opcode,
		CuiWindow *window)
{
	Terminal *terminal;
	gchar *command = NULL;
	const gchar *terminal_wd;
	const gchar *window_wd;
	gboolean jump = FALSE; /* If we need jump back. */
	
	g_return_if_fail(CUI_IS_GENERATOR(generator));
	g_return_if_fail(CUI_IS_WINDOW(window));
#ifdef DEBUG
	g_message("%s(): start.", __func__);
#endif
	
	terminal = generator->terminal;
	terminal_wd = terminal_get_working_directory(terminal);
	window_wd = cui_window_get_working_directory(window);
	if (terminal_wd == NULL || window_wd == NULL || 
			strcmp(terminal_wd, window_wd) != 0)
		jump = TRUE;
	
	switch (opcode){
		case WindowRefresh:
			/*
			 * If we changed the working directory we jump back.
			 */
			if (jump){
				command = g_strdup_printf(
						"pushd %s", window_wd);
				terminal_send_command(terminal, command, TRUE);
				g_free(command);	
			}
			/*
			 * The original command to refresh the window.
			 */
			terminal_send_command(terminal, window->command->str, 
					TRUE);
			/*
			 * If we had to jump back we return.
			 */
			if (jump)
				terminal_send_command(terminal, "popd", TRUE);
			
			break;
		default:
			g_warning("%s(): Unknown opcode: %d", __func__, opcode);
	}
}

void
cui_generator_emit_window_command_with_arg(
		CuiGenerator *generator,
		CuiCommandOpcode opcode,
		CuiWindow *window,
		gpointer   arg)
{
	Terminal *terminal;
	gchar *command = NULL;
	const gchar *terminal_wd;
	const gchar *window_wd;
	gboolean jump = FALSE; /* If we need jump back. */
	
	g_return_if_fail(CUI_IS_GENERATOR(generator));
	g_return_if_fail(CUI_IS_WINDOW(window));
#ifdef DEBUG
	g_message("%s(): start.", __func__);
#endif
	
	terminal = generator->terminal;
	terminal_wd = terminal_get_working_directory(terminal);
	window_wd = cui_window_get_working_directory(window);
	if (terminal_wd == NULL || window_wd == NULL || 
			strcmp(terminal_wd, window_wd) != 0)
		jump = TRUE;
	
	switch (opcode){
		case WindowDirectCommand:
			/*
			 * If we changed the working directory we jump back.
			 */
			if (jump){
				command = g_strdup_printf(
						"pushd %s", window_wd);
				terminal_send_command(terminal, command, TRUE);
				g_free(command);	
			}
			/*
			 * The original command to refresh the window.
			 */
			terminal_send_command(terminal, arg, TRUE);
			/*
			 * If we had to jump back we return.
			 */
			if (jump)
				terminal_send_command(terminal, "popd", TRUE);
			
			break;
		default:
			g_warning("%s(): Unknown opcode: %d", __func__, opcode);
	}
}

/**
 * cui_generator_emit_commands:
 * @command_generator: a #CuiGenerator to generate the commands
 * @list: a list of commands
 * @returns: the number of commands executed
 *
 * Emits a series of commands to the terminal to execute them.
 */
gint
cui_generator_emit_commands(CuiGenerator *command_generator, GList *list)
{
	CuiCommand *command;
	gint n;
	
	g_return_val_if_fail(CUI_IS_GENERATOR(command_generator), -1);
	if (list == NULL)
		return 0;
#ifdef DEBUG	
	g_message("%s(): Starting.", __func__);
#endif
	
	while (list != NULL){
		command = list->data;
		switch (command->type){
			case FileCommand:
				cui_generator_emit_file_command(
						command_generator, 
						command->opcode, 
						CUI_FILENAME(command->parg1));
				++n;
				break;
			case FileCommandWithArg:
				cui_generator_emit_file_command_with_arg(
						command_generator, 
						command->opcode, 
						CUI_FILENAME(command->parg1),
						(const gchar *)command->parg2);
				++n;
				break;
			case WindowCommand:
				cui_generator_emit_window_command(
						command_generator,
						command->opcode,
						CUI_WINDOW(command->parg1));
				++n;
				break;
			case WindowCommandWithArg:
				cui_generator_emit_window_command_with_arg(
						command_generator,
						command->opcode,
						CUI_WINDOW(command->parg1),
						command->parg2);
				++n;
				break;
			default:
				g_warning("%s(): Unknown command type: %d",
						__func__, command->type);
		}
next_command:
		list = list->next;
	}
	return n;
}
		
