/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: configure.c,v 1.1 2006/02/24 17:23:21 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: configure.c,v $
 * Revision 1.1  2006/02/24 17:23:21  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

/****************************************************************************
 * Functions to read and change the configuration of this program.          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <bonobo.h>
#include <gnome.h>
#include <gconf/gconf-client.h>

#include "support.h"
#include "configure.h"
#include "history.h"   /* We learn from the shell history file. */

#define NODEBUG

GConfClient *gconfclient = NULL; 

/**
 *bg_conf_init:
 *
 * Initializes the configuration subsystem.
 */
void bg_conf_init(void)
{
	gconfclient = gconf_client_get_default();
}

gchar *
bg_conf_get_default_text_editor(void)
{
	gboolean initialized = FALSE;
	gchar *editor = NULL;
	gchar *tmp;
	/* 
	 * For the fastest execution we cache the text editor, because this
	 * information is frequently needed.
	 */
	if (initialized)
		return editor;
	/*
	 * The editor name the user set. This is the first priority.
	 */
	editor = bg_conf_get_string("/general/applications/text-editor", "");
	if (editor[0] == '\0') 
		g_free(tmp);
	else
		goto finalize;
	/*
	 * The 2nd option is to find out which text editor is the most
	 * freqently used according to the users shell history file.
	 */
	editor = g_strdup(handle_text_editors(NULL, OpSearch));
	if (editor != NULL)
		goto finalize;
	/*
	 * The 3rd option is to find the name of the text editor in the
	 * environment.
	 */
	editor = g_getenv("EDITOR");
	if (editor != NULL) {
		editor = g_strdup(editor);
		goto finalize;
	}
	editor = g_getenv("VISUAL");
	if (editor != NULL) {
		editor = g_strdup(editor);
		goto finalize;
	}
	/*
	 * As last last resort we use the gvim text editor for it is simple,
	 * small, and graphical. Oh, and I like it... :-)
	 */
	editor = g_strdup("gvim");
finalize:
	initialized = TRUE;
	return editor;
}


gchar *bg_conf_set_string(const gchar *name, gchar *value)
{
	GError *error = NULL;
	gchar *fullname;

	if (gconfclient == NULL)
		bg_conf_init();

	fullname = g_strdup_printf("/apps/cuiterm%s", name);
	gconf_client_set_string(gconfclient, fullname, value, &error);
	if (error != NULL)
		g_warning("%s(): Error setting '%s': %s", __func__,
				name, error->message);
	g_free(fullname);
	return value;
}

gchar *bg_conf_get_string(const gchar *name, gchar *def)
{
	gchar *value;
	GError *error = NULL;
	gchar *fullname;

	if (gconfclient == NULL)
		bg_conf_init();

	fullname = g_strdup_printf("/apps/cuiterm%s", name);
	
	value = gconf_client_get_string(gconfclient, fullname, &error);
	if (error != NULL){
		g_warning("%s(): Error getting string '%s': %s\n", __func__, fullname, error->message);
		error = NULL;
	}

	if (value == NULL){
		gconf_client_set_string(gconfclient, fullname, def, &error);
		g_free(fullname);
		return g_strdup(def);
	}
	
	g_free(fullname);
	return value;
}


gboolean 
bg_conf_get_bool(const gchar *name, 
		gboolean def)
{
	gboolean value;
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s", name);
        v = gconf_client_get(gconfclient, fullname, &error);	
	if (v == NULL){
		gconf_client_set_bool(gconfclient, fullname, def, &error);
		g_free(fullname);
		return def;
	}
	
	value = gconf_client_get_bool(gconfclient, fullname, &error);

	g_free(fullname);
	return value;
}

gboolean 
bg_conf_set_bool(const gchar *name, 
		gboolean value)
{
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s", name);
	gconf_client_set_bool(gconfclient, fullname, value, &error);

	g_free(fullname);
	return value;
}

gdouble
bg_conf_get_double(const gchar *name, 
		gdouble def)
{
	gboolean value;
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s", name);
        v = gconf_client_get(gconfclient, fullname, &error);	
	if (v == NULL){
		gconf_client_set_float(gconfclient, fullname, def, &error);
		g_free(fullname);
		return def;
	}
	
	value = gconf_client_get_float(gconfclient, fullname, &error);

	g_free(fullname);
	return value;
}

gdouble
bg_conf_set_double(const gchar *name, 
		gdouble value)
{
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s", name);
	gconf_client_set_float(gconfclient, fullname, value, &error);
	g_free(fullname);
	return value;
}

gint 
bg_conf_get_int(const gchar *name, 
		gint def)
{
	gboolean value;
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;

	g_message("%s(): name: '%s'", __func__, name);
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s", name);
        v = gconf_client_get(gconfclient, fullname, &error);	
	if (v == NULL){
		gconf_client_set_int(gconfclient, fullname, def, &error);
		g_free(fullname);
		return def;
	}
	
	value = gconf_client_get_int(gconfclient, fullname, &error);

	g_message("'%s'-->%d", fullname, value);
	g_free(fullname);
	return value;
}

gint 
bg_conf_set_int(const gchar *name, 
		gint value)
{
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s", name);
	gconf_client_set_int(gconfclient, fullname, value, &error);
	g_free(fullname);
	return value;
}

GdkColor *
bg_conf_set_color(const gchar *name,
		  const GdkColor *color)
{
	GError *error = NULL;
	gchar *fullname;
	GConfValue *v;
	
	if (gconfclient == NULL)
		bg_conf_init();
	
	fullname = g_strdup_printf("/apps/cuiterm%s-red", name);
	gconf_client_set_int(gconfclient, fullname, color->red, &error);
	g_free(fullname);
	
	fullname = g_strdup_printf("/apps/cuiterm%s-green", name);
	gconf_client_set_int(gconfclient, fullname, color->green, &error);
	g_free(fullname);
	
	fullname = g_strdup_printf("/apps/cuiterm%s-blue", name);
	gconf_client_set_int(gconfclient, fullname, color->blue, &error);
	g_free(fullname);
	
	return color;
}

gboolean
bg_conf_save_window_coord(GtkWidget *widget, 
		gchar *name)
{
	gint       x, y;
	gchar      *tmp;
	
	g_assert(widget != NULL);

	gtk_window_get_position(widget, &x, &y);
	/*
	 * Saving the values.
	 */
	tmp = g_strdup_printf("%s/geometry_x", name);
	bg_conf_set_int(tmp, x);
	g_free(tmp);
	tmp = g_strdup_printf("%s/geometry_y", name);
	bg_conf_set_int(tmp, y);
	g_free(tmp);

	return TRUE;
}

gboolean
bg_conf_restore_window_coord(GtkWidget *widget, 
		gchar *name)
{
	gint       x, y;
	gchar      *tmp;
	
	g_assert(widget != NULL);
	
	/*
	 * Loading the values.
	 */
	tmp = g_strdup_printf("%s/geometry_x", name);
	x = bg_conf_get_int(tmp, 0);
	g_free(tmp);
	tmp = g_strdup_printf("%s/geometry_y", name);
	y = bg_conf_get_int(tmp, 0);
	g_free(tmp);

	gtk_window_move(widget, x, y);

	return TRUE;
}

/**
 * bg_conf_get_default_ls_options:
 * @longl: it is nothing. really!
 * @returns: an option set for the ls program.
 *
 * The return value is a newly allocated string, which must be freed when no
 * longer needed.
 */
gchar *
bg_conf_get_default_ls_options(gboolean longl)
{
	gboolean long_listing, show_hidden_files, human_readable;
	gboolean show_filetype, quote;

	long_listing = bg_conf_get_bool("/listings/long_listing", TRUE);
	show_hidden_files = bg_conf_get_bool("/listings/show_hidden_files",
			TRUE);
	human_readable = bg_conf_get_bool("/listings/human_readable", FALSE);
	show_filetype = bg_conf_get_bool("/listings/show_filetype", FALSE);
	quote = bg_conf_get_bool("/listings/quote", FALSE);

	if (!long_listing && !show_hidden_files && !human_readable &&
			!show_filetype && !quote)
		return g_strdup("");
	
	return g_strdup_printf(" -%s%s%s%s%s",
			long_listing ? "l" : "",
			human_readable ? "h" : "",
			show_hidden_files ? "a" : "",
			show_filetype ? "p" : "",
			quote ? "Q" : ""
			);
}

/*
 * In the future this function will be a much more sophisticated 
 * with learning capabilities. I promise.
 */
gchar *
bg_conf_get_default_editor(gchar *filename)
{
	int n;
	gchar *types[] = {
		"*.pdf",       "*.png",      "*.jpg",
		"*.dvi",
		"*.avi",       "*.mpg",
		"*.ps", "*.eps",
		NULL
	};
	
	gchar *apps[] = {
		"acroread", "gimp",    "gimp",
		"xdvi",
		"mplayer",  "mplayer",  
		"gv", "gv",
		NULL
	};

	n = 0;
	while (types[n] != NULL){
		if (fnmatch(types[n], filename, 0) == 0)
			return apps[n];
		++n;
	}

	return "gvim";
}

/*****************************************************************************
 * Callback functions for properties dialog.                                 *
 *                                                                           *
 *                                                                           *
 *****************************************************************************/
inline static gboolean
load_toggle_button(GtkWidget *widget,
		const gchar *widget_name,
		const gchar *gconf_name,
		gboolean default_value)
{
	GtkWidget *button = lookup_widget(widget, widget_name);
	gboolean b = bg_conf_get_bool(gconf_name, default_value);
	g_assert(button != NULL);
	
	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), b);
	return b;
}

inline static gdouble
load_spin_button(GtkWidget *widget,
		const gchar *widget_name,
		const gchar *gconf_name,
		gdouble default_value)
{
	GtkWidget *button = lookup_widget(widget, widget_name);
	gdouble value = bg_conf_get_double(gconf_name, default_value);
	g_assert(button != NULL);
	
	gtk_spin_button_set_value(GTK_SPIN_BUTTON(button), value);
	return value;
}

void
on_properties_window_realize           (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *button;
	gboolean b;
	
	bg_conf_restore_window_coord(widget, "/settings/properties_window");

	/*
	 * Setting the checkbuttons as stored in the configuration. 
	 */
	load_toggle_button(widget, "long_listing", "/listings/long_listing", TRUE);
	load_toggle_button(widget, "show_hidden_files", "/listings/show_hidden_files", TRUE);
	load_toggle_button(widget, "human_readable", "/listings/human_readable", FALSE);
	load_toggle_button(widget, "show_filetype", "/listings/show_filetype", FALSE);
	load_toggle_button(widget, "quote", "/listings/quote", TRUE);
	load_toggle_button(widget, "splash_checkbutton", "/init/splash_screen", TRUE);
	load_toggle_button(widget, "distribution_checkbutton", "/init/distr_logo", TRUE);
	load_toggle_button(widget, "different_language", "/settings/use_diff_language", TRUE);
	load_toggle_button(widget, "color_highlight", "/settings/use_color_highlight", TRUE);
	load_toggle_button(widget, "syntax_highlight", "/settings/bash_systax_highlight", TRUE);
	load_toggle_button(widget, "dynamic_colors", "/settings/dynamic_colors", TRUE);
	load_toggle_button(widget, "window_frames", "/settings/draw_window_frames", TRUE);
	load_toggle_button(widget, "window_shadow", "/settings/draw_window_shadow", TRUE);
	load_toggle_button(widget, "title_text", "/settings/title_text", TRUE);
	load_toggle_button(widget, "draw_sizebars", "/settings_draw_sizebars", TRUE);
	load_toggle_button(widget, "spell_check", "/settings/spell_check", TRUE);
	load_toggle_button(widget, "learn_manual", "/settings/learn_manual", TRUE);
	load_toggle_button(widget, "detach_man_pages", "/settings/detach_man_pages", TRUE);
	load_toggle_button(widget, "dnd_move", "/dnd/move", TRUE);
	load_toggle_button(widget, "dnd_copy", "/dnd/copy", TRUE);
	load_toggle_button(widget, "dnd_link", "/dnd/link", TRUE);
	load_toggle_button(widget, "dnd_ask", "/dnd/ask", TRUE);
	load_toggle_button(widget, "learn_text_editor", "/applications/text_editor/learn", TRUE);
	load_toggle_button(widget, "text_editor", "/applications/text_editor/given", TRUE);
	/*
	 * FIXME: If the user changed these settings we should execute a command
	 * to immediately validate the changes, but we have no pointer to the
	 * terminal here.
	 */
	load_spin_button(widget, "columns_spinbutton", "/settings/shell/columns", 80);
	load_spin_button(widget, "rows_spinbutton", "/settings/shell/rows", 25);
}

void
on_properties_cancel_button_clicked(
		GtkButton *button, 
		gpointer   user_data)
{
	GtkWidget *widget;
	widget = lookup_widget(GTK_WIDGET(button), "properties_window");

	bg_conf_save_window_coord(widget, "/settings/properties_window");
	gtk_widget_destroy(widget);
}


inline static gboolean
save_toggle_button(GtkWidget *widget,
		const gchar *widget_name,
		const gchar *gconf_name)
{
	GtkWidget *button = lookup_widget(widget, widget_name);
	gboolean b = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button));
	bg_conf_set_bool(gconf_name, b);
	return b;
}

inline static gdouble
save_spin_button(GtkWidget *widget,
		const gchar *widget_name,
		const gchar *gconf_name)
{
	GtkWidget *button = lookup_widget(widget, widget_name);
	gboolean value = gtk_spin_button_get_value(GTK_SPIN_BUTTON(button));
	bg_conf_set_double(gconf_name, value);
	return value;
}

void
on_properties_ok_button_clicked(
		GtkButton *button, 
		gpointer   user_data)
{
	GtkWidget *widget;
	GtkTextBuffer *text_buffer;
	GtkTextIter begin, end;
	gchar *text;
	gboolean b;
	/*
	 * Saving the checkbutton widgets to the gconf.
	 */
	save_toggle_button(button, "long_listing", "/listings/long_listing");
	save_toggle_button(button, "show_hidden_files", "/listings/show_hidden_files");
	save_toggle_button(button, "human_readable", "/listings/human_readable");
	save_toggle_button(button, "show_filetype", "/listings/show_filetype");
	save_toggle_button(button, "quote", "/listings/quote");
	save_toggle_button(button, "splash_checkbutton", "/init/splash_screen");
	save_toggle_button(button, "distribution_checkbutton", "/init/distr_logo");
	save_toggle_button(button, "different_language", "/settings/use_diff_language");
	save_toggle_button(button, "color_highlight", "/settings/use_color_highlight");
	save_toggle_button(button, "syntax_highlight", "/settings/bash_systax_highlight");
	save_toggle_button(button, "dynamic_colors", "/settings/dynamic_colors");
	save_toggle_button(button, "window_frames", "/settings/draw_window_frames");
	save_toggle_button(button, "window_shadow", "/settings/draw_window_shadow");
	save_toggle_button(button, "title_text", "/settings/title_text");
	save_toggle_button(button, "draw_sizebars", "/settings_draw_sizebars");
	save_toggle_button(button, "spell_check", "/settings/spell_check");
	save_toggle_button(button, "learn_manual", "/settings/learn_manual");
	save_toggle_button(button, "detach_man_pages", "/settings/detach_man_pages");
	save_toggle_button(button, "dnd_move", "/dnd/move");
	save_toggle_button(button, "dnd_copy", "/dnd/copy");
	save_toggle_button(button, "dnd_link", "/dnd/link");
	save_toggle_button(button, "dnd_ask", "/dnd/ask");
	save_toggle_button(button, "learn_text_editor", "/applications/text_editor/learn");
	save_toggle_button(button, "text_editor", "/applications/text_editor/given");
	
	save_spin_button(button, "columns_spinbutton", "/settings/shell/columns");
	save_spin_button(button, "rows_spinbutton", "/settings/shell/rows");

	widget = lookup_widget(GTK_WIDGET(button), "init_commands_textview");
	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_get_start_iter(text_buffer, &begin);
	gtk_text_buffer_get_end_iter(text_buffer, &end);
	text = gtk_text_buffer_get_text(text_buffer, &begin, &end, TRUE);
	bg_conf_set_string("/init/commands", text);
	g_free(text);
	
	widget = lookup_widget(GTK_WIDGET(button), "shell_combobox");
	text = gtk_combo_box_get_active_text(GTK_COMBO_BOX(widget));
	bg_conf_set_string("/settings/shell_name", text);

	widget = lookup_widget(GTK_WIDGET(button), "shell_filechooserbutton");
	text = gtk_file_chooser_get_filename(GTK_FILE_CHOOSER(widget));
	bg_conf_set_string("/settings/shell_binary", text);
	
	
	widget = lookup_widget(GTK_WIDGET(button), "properties_window");
	bg_conf_save_window_coord(widget, "/settings/properties_window");
	gtk_widget_destroy(widget);
}

void
on_init_commands_textview_realize      (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkTextBuffer *text_buffer;
	gchar *text;

	text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
	gtk_text_buffer_set_text(text_buffer,
			bg_conf_get_string("/init/commands", "ls -lha"),
			-1);
}

