%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: df_lexer.l,v 1.1 2006/02/24 09:05:33 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: df_lexer.l,v $
 * Revision 1.1  2006/02/24 09:05:33  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include <stdlib.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "df_parser.h"



#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%%
	
	cui_window_set_title(window_for_lex, _("Disk Usage"));

("Filesystem".*)|("Fájlrendszer".*) {
	#ifdef DEBUG
	g_message("%s: HEADER: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "Header",
		df_lval.sval.begin, 
		df_lval.sval.end); 
	return HEADER;
}

^[^ \t]+ {
	#ifdef DEBUG
	g_message("%s: BLOCK_DEVICE: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "block_device",
		df_lval.sval.begin, 
		df_lval.sval.end); 
	return BLOCK_DEVICE;
}

\/[^ \t\n]* {
	#ifdef DEBUG
	g_message("%s: MOUNT_POINT: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "directory",
		df_lval.sval.begin, 
		df_lval.sval.end); 
	return MOUNT_POINT;
}

[0-9.,]+ 	{
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	df_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	return NUMBER;
}

[0-9.,]+[Gg] {
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	df_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	df_lval.sval.double_val *= 1024 * 1024 * 1024;
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	return NUMBER;
}

[0-9.,]+[Mm] {
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	df_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	df_lval.sval.double_val *= 1024 * 1024;
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	return NUMBER;
}

[0-9.,]+[Kk] {
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	df_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	df_lval.sval.double_val *= 1024;
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	return NUMBER;
}

[0-9.,]+[%] {
	#ifdef DEBUG
	g_message("%s: PERCENT: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	df_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	df_lval.sval.double_val /= 100;
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	return PERCENT;
}

[a-zA-Z][a-zA-Z0-9]* {
	#ifdef DEBUG
	g_message("%s: FILESYSTEM: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		df_lval.sval.text,
		&df_lval.sval.begin,
		&df_lval.sval.end);
	return FILESYSTEM;
}


[ \t\n]+ { 
	/* Dropping it away */
	//g_message("%s: spaces: <%s>", __func__, yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}
	

.	{
	#ifdef DEBUG
	g_message("%s: OTHER_CHAR: '%s'", __func__, yytext);
	#endif
	df_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	return yytext[0];
}

%%

