/*
 * Copyright (C) 2004, 2005, 2006 Laszlo Pere
 * $Id: dnd.c,v 1.1 2006/02/24 17:19:56 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: dnd.c,v $
 * Revision 1.1  2006/02/24 17:19:56  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#ifdef ALLDEBUG
#  define DND_DEBUG
#endif

#include <stdlib.h>
#include <gnome.h>
#include "configure.h"
#include "widgets.h"
#include "terminal.h"
#include "background.h"
#include "dnd.h"

static GtkTargetEntry target_table[] = {
	{"text/uri-list", 0, TargetTextUriList},
	{"file/inode", 0, TargetFileForInode},
	{"file/mode", 0, TargetFileForMode},
	{"file/owner", 0, TargetFileForOwner},
	{"file/group", 0, TargetFileForGroupOwner},
	{"file/links", 0, TargetFileForLinks},
	{"user/name", 0, TargetUserName},
	{"group/name", 0, TargetGroupName},
	{"file/size", 0, TargetFileForSize},
	{"file/date", 0, TargetFileForDate},
	{"file/name", 0, TargetFileForName},
	{"string/option", 0, TargetStringOption},
	{"string/longoption", 0, TargetStringLongOption},
	{"string/program", 0, TargetStringProgram},
	NULL
};

static GtkTargetEntry target_table_dir[] = {
	{"text/uri-list", 0, TargetTextUriList},
	{"file/inode", 0, TargetFileForInode},
	{"file/mode", 0, TargetFileForMode},
	{"file/owner", 0, TargetFileForOwner},
	{"file/group", 0, TargetFileForGroupOwner},
	{"file/links", 0, TargetFileForLinks},
	{"user/name", 0, TargetUserName},
	{"group/name", 0, TargetGroupName},
	{"file/size", 0, TargetFileForSize},
	{"file/date", 0, TargetFileForDate},
	{"file/name", 0, TargetFileForName},
	NULL
};

static GtkTargetEntry target_table_file[] = {
	{"file/mode", 0, TargetFileForMode},
	{"file/owner", 0, TargetFileForOwner},
	{"file/group", 0, TargetFileForGroupOwner},
	{"user/name", 0, TargetUserName},
	{"group/name", 0, TargetGroupName},
	{"file/date", 0, TargetFileForDate},
	NULL
};

static void dnd_debug_print_targets(GList * list)
{
	gint n = 0;

	while (list) {
		char *name =
		    gdk_atom_name(GDK_POINTER_TO_ATOM(list->data));
		g_message("%s(): [%d] %s", __func__, n, name);
		g_free(name);
		list = list->next;
		n++;
	}
}


/**
 * dnd_get_file_target_list:
 * @returns: a #GtkTargetList list for file target sources
 *
 * Returns a #GtkTargetList usefull for DND target for regular files.
 */
GtkTargetList *dnd_get_file_target_list(void)
{
	static GtkTargetList *target_list = NULL;

	if (target_list == NULL)
		target_list = gtk_target_list_new(target_table_file, 6);
	return target_list;
}

/**
 * dnd_get_dir_target_list:
 * @returns: a #GtkTargetList list for directory target sources
 *
 * Returns a #GtkTargetList usefull for DND target for regular directories.
 */
GtkTargetList *dnd_get_dir_target_list(void)
{
	static GtkTargetList *target_list = NULL;

	if (target_list == NULL)
		target_list = gtk_target_list_new(target_table_dir, 11);
	return target_list;
}

gint dnd_init_destination(GtkWidget * widget)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	g_return_val_if_fail(GTK_IS_WIDGET(widget), -1);

	gtk_drag_dest_set(widget,	    /* The widget.            */
			  0,	            /* Auto action flags.     */
			  target_table,	    /* What types to accept.  */
			  14,	            /* How much dest types.   */
			  GDK_ACTION_COPY); /* Method to accept.      */

}


/*
 * The following function is coming from gtk+-2.8.
 */
#define DRAG_ICON_MAX_WIDTH 250
#define DRAG_ICON_LAYOUT_BORDER 2
#define DRAG_ICON_MAX_LINES 7
/**
 * _gtk_text_util_create_drag_icon
 * @widget: #GtkWidget to extract the pango context
 * @text: a #gchar to render the icon
 * @len: length of @text, or -1 for NUL-terminated text
 *
 * Creates a drag and drop icon from @text.
 **/
GdkPixmap*
_gtk_text_util_create_drag_icon (GtkWidget *widget, 
                                 gchar     *text,
                                 gsize      len)
{
  GdkDrawable  *drawable = NULL;
  PangoContext *context;
  PangoLayout  *layout;
  gint          pixmap_height, pixmap_width;
  gint          layout_width, layout_height;
  gint          n_lines;

  g_return_val_if_fail (widget != NULL, NULL);
  g_return_val_if_fail (text != NULL, NULL);

  context = gtk_widget_get_pango_context (widget);
  layout  = pango_layout_new (context);

  pango_layout_set_text (layout, text, len);
  pango_layout_set_wrap (layout, PANGO_WRAP_WORD_CHAR);
  pango_layout_get_size (layout, &layout_width, &layout_height);

  layout_width = MIN (layout_width, DRAG_ICON_MAX_WIDTH * PANGO_SCALE);
  pango_layout_set_width (layout, layout_width);
  n_lines = pango_layout_get_line_count (layout);

  //limit_layout_lines (layout);

  /* get again layout extents, they may have changed */
  pango_layout_get_size (layout, &layout_width, &layout_height);

  pixmap_width  = layout_width  / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;
  pixmap_height = layout_height / PANGO_SCALE + DRAG_ICON_LAYOUT_BORDER * 2;

  drawable = gdk_pixmap_new (widget->window,
                             pixmap_width  + 2,
                             pixmap_height + 2,
                             -1);

  gdk_draw_rectangle (drawable,
                      widget->style->base_gc [GTK_WIDGET_STATE (widget)],
                      TRUE,
                      0, 0,
                      pixmap_width + 1,
                      pixmap_height + 1);

  gdk_draw_layout (drawable,
                   widget->style->text_gc [GTK_WIDGET_STATE (widget)],
                   1 + DRAG_ICON_LAYOUT_BORDER,
                   1 + DRAG_ICON_LAYOUT_BORDER,
                   layout);

  gdk_draw_rectangle (drawable,
                      widget->style->black_gc,
                      FALSE,
                      0, 0,
                      pixmap_width + 1,
                      pixmap_height + 1);

  g_object_unref (layout);

  return drawable;
}


gint dnd_drag_begin(GdkEventMotion *event, gint button, GtkWidget *widget)
{
	Terminal *t;
	Background *bg;
	GList *list_item;
	CuiWidget *cui_widget;
	CuiWidget *chosen_widget;
	gint chosen_size;
	gint x, y;
	GdkDragContext *drag_context;
	GtkTargetList *target_list;
	TargetType t_type;

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif

	/*
	 * Lets find the background and the last mouse press coordinates
	 */
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	g_assert(t != NULL);
	bg = t->background;
	gtk_text_view_window_to_buffer_coords(GTK_TEXT_VIEW(widget),
					      GTK_TEXT_WINDOW_TEXT,
					      bg->last_mouse_x,
					      bg->last_mouse_y, &x, &y);
	/*
	 * Let's find the dnd source with the smallest area.
	 */
	chosen_widget = NULL;
	list_item = bg->bg_widgets;
	while (list_item != NULL) {
		cui_widget = list_item->data;

		if (!cui_widget_is_mouse_over(cui_widget, x, y))
			goto next_item;

		if (!CUI_IS_RECTANGLE(cui_widget) ||
		    !cui_rectangle_is_dnd_source(CUI_RECTANGLE
						 (cui_widget)))
			goto next_item;

		if (chosen_widget == NULL ||
		    cui_widget_get_size(cui_widget) < chosen_size) {
			chosen_widget = cui_widget;
			chosen_size = cui_widget_get_size(chosen_widget);
		}
	      next_item:
		list_item = list_item->next;
	}
	/*
	 * We need to remember the last dnd source.
	 */
	bg->dnd_source_widget = chosen_widget;
	/*
	 * Let's start the DnD operation. If we found a DnD source we offer that
	 * type first, otherwise we offer something other.
	 */
	if (chosen_widget != NULL) {
		t_type = CUI_RECTANGLE(chosen_widget)->source_type;
		target_list = gtk_target_list_new(&target_table[t_type], 1);
	} else {
		target_list = gtk_target_list_new(&target_table[0], 1);
	}
	drag_context = gtk_drag_begin(widget, target_list,
				      GDK_ACTION_COPY, button, 
				      (GdkEvent *)event);
	if (chosen_widget != NULL) {
		GdkPixmap *pixmap = NULL;
		GtkTextBuffer *text_buffer = gtk_text_view_get_buffer(GTK_TEXT_VIEW(widget));
		GtkTextIter start, end;
		gchar *text;
		
		if (chosen_widget != NULL && 
				chosen_widget->begin_mark != NULL && 
				chosen_widget->end_mark != NULL) {
			gtk_text_buffer_get_iter_at_mark(text_buffer, &start, chosen_widget->begin_mark);
			gtk_text_buffer_get_iter_at_mark(text_buffer, &end, chosen_widget->end_mark);
			text = gtk_text_buffer_get_text(text_buffer, &start, &end, FALSE);	
			pixmap = _gtk_text_util_create_drag_icon (widget, text, -1);
			g_free(text);
		}

		if (pixmap)
			gtk_drag_set_icon_pixmap (drag_context, 
					gdk_drawable_get_colormap(pixmap), 
					pixmap, NULL, -24, -2);
		else
			gtk_drag_set_icon_default (drag_context);
	}
}


static gchar *
dnd_create_file_list(gchar * file_list)
{
	gunichar c;
	GString *str = g_string_new("");

	do {
		if (g_ascii_strncasecmp("file://", file_list, 7) == 0) {
			file_list += 7;
			continue;
		}


		c = g_utf8_get_char(file_list);
		file_list = g_utf8_next_char(file_list);

		if (c == '\n')
			continue;

		if (c == '\r') {
			g_string_append_unichar(str, ' ');
			continue;
		}

		g_string_append_unichar(str, c);
	} while (c != '\0');

	return g_string_free(str, FALSE);
}


void
on_main_widget_drag_data_received(GtkWidget        *widget,
				  GdkDragContext   *drag_context,
				  gint              x, 
				  gint              y,
				  GtkSelectionData *data,
				  guint             info,
				  guint             time, 
				  gpointer          user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
	Background *bg = t->background;
	CuiRectangle *target;
	gchar *file_list;
	gchar *wd;
	gchar *command;

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	g_assert(t != NULL);
	/*
	 * We need this because we don't want the GtkTextView to handle
	 * this event.
	 */
	g_signal_stop_emission_by_name(widget, "drag-data-received");
	/*
	 * We ask which target was the first during the motion, then
	 * send the data to it.
	 */
	target = CUI_RECTANGLE(background_get_last_dnd_target(bg));
	g_signal_emit_by_name(target, "drop-data-received", data, NULL);
	/*
	 * This conversation is over (7 of 9).
	 */
	gtk_drag_finish(drag_context, TRUE, FALSE, time);
	
}

void
on_secondary_widget_drag_data_received (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        GtkSelectionData *data,
                                        guint            info,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	on_main_widget_drag_data_received(widget, drag_context, x, y, data, info, time, user_data);
}

/**
 * on_main_widget_drag_motion:
 * @widget: the #GtkWidget where the motion has been found
 * @drag_context: the drag context of the operation
 * @x: the x coordinate of the mouse
 * @y: the y coordinate of the mouse
 * @time: the time when the motion was detected
 * @user_data: the data given by the signal connection 
 * @returns: TRUE if the motion could be handled (always returns TRUE)
 *
 */
gboolean
on_main_widget_drag_motion(GtkWidget * widget,
			   GdkDragContext *drag_context,
			   gint x, gint y, 
			   guint time, 
			   gpointer user_data)
{
	GtkTextView *text_view = GTK_TEXT_VIEW(widget);
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
  	gint bx, by;
	GtkTextIter newplace;
	
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
	dnd_debug_print_targets(drag_context->targets);
#endif
	/*
	 * Let's scroll the text view during the dnd motion. I get this
	 * method from the GTK+ 2.8 source, however I'm not fully satisfied.
	 */
	gtk_text_view_window_to_buffer_coords (text_view, 
			GTK_TEXT_WINDOW_WIDGET, x, y, &bx, &by);
	gtk_text_layout_get_iter_at_pixel (text_view->layout, 
			&newplace, bx, by);  
	gtk_text_view_scroll_to_iter(text_view, &newplace, 0.2, FALSE, 0.0, 0.0);

	background_dnd_motion_notify(widget, t->background, drag_context, x, y);
	/*
	 * If we not call this function we will never get a motion notify again.
	 * I hate this.
	 */
	gdk_drag_status(drag_context, GDK_ACTION_COPY, time);
	return TRUE;
}


gboolean
on_secondary_widget_drag_motion        (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	return on_main_widget_drag_motion(widget, drag_context, x, y, time, user_data);
}


void
on_main_widget_drag_leave(GtkWidget * widget,
			  GdkDragContext * drag_context,
			  guint time, gpointer user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif

	background_dnd_motion_notify(widget, t->background, drag_context, -1, -1);
}

void
on_secondary_widget_drag_leave         (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	on_main_widget_drag_leave(widget, drag_context, time, user_data);
}


/**
 * on_main_widget_drag_data_get:
 * @widget: the #GtkTextView widget where the DnD was initiated 
 * @drag_context: the #GdkDragContext we created at the begin
 * @data: the #GtkSelectionData we have to fill
 * @info: the size of the data units
 * @time: what was the time when the data was received
 * @user_data: pointer to the data gave by the signal connection command
 *
 * Called when we initialized a DnD as a source widget and the target is asking
 * the data. There are some problems with this function, namely:
 *
 * (1) We store the DnD source CuiWidget with the main_widget object. This is
 * not a nice solution IMHO.
 * 
 */
void
on_main_widget_drag_data_get(GtkWidget         *widget,
			     GdkDragContext    *drag_context,
			     GtkSelectionData  *data,
			     guint              info, 
			     guint              time, 
			     gpointer           user_data)
{
	gchar        *send_this, *tmp;
	CuiFilename  *cui_filename;
	CuiRectangle *cui_widget;
	Terminal     *t;

#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	/*
	 * It is possible we drag nothing?
	 */
	cui_widget = CUI_RECTANGLE(background_get_last_dnd_source(t->background));
	if (cui_widget == NULL)
		return;
	/*
	 * Here we send the data to the dnd target. However we can not simply
	 * send a CuiFilename object since this object would contain pointers
	 * to which the dnd target -- if it is a separate process -- would not
	 * reach.
	 */
	if (CUI_IS_FILENAME(cui_widget->source_data)) {
		cui_filename = cui_widget->source_data;
		tmp = cui_filename_get_canonical_filename(cui_filename, "/");
		send_this = cui_remove_tilde(tmp);
		g_free(tmp);
		gtk_selection_data_set(data, data->target, 8, send_this, g_utf8_strlen(send_this, -1));
		g_free(send_this);
		return;
	} else {
		g_warning("%s(): Unknown data type.", __func__);
	}
#if 0
	gtk_selection_data_set(data, data->target, 8,
			       cui_widget->source_data,
			       cui_widget->source_size);
#endif
}


void
on_secondary_widget_drag_data_get(GtkWidget       *widget,
                                  GdkDragContext  *drag_context,
                                  GtkSelectionData *data,
                                  guint            info,
                                  guint            time,
                                  gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	on_main_widget_drag_data_get(widget, drag_context, data, info, time, user_data);
}


/**
 * on_main_widget_drag_drop:
 *
 * Here we have to decide if the DnD is in the "drop zone".
 */
gboolean
on_main_widget_drag_drop(GtkWidget * widget,
			 GdkDragContext * drag_context,
			 gint x, gint y, guint time, gpointer user_data)
{
	Terminal *t = g_object_get_data(G_OBJECT(widget), "terminal");
	gboolean found;
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	/*
	 * This is not nice to stop the emission of every single signal one by
	 * one, it would be much better to completly remove the existing handler
	 * which will accept every drop operation.
	 */
	g_signal_stop_emission_by_name(widget, "drag-drop");
	/*
	 * If not in drop zone.
	 */
	found = background_dnd_motion_notify(widget, t->background, drag_context, x, y);
	background_dnd_motion_notify(widget, t->background, drag_context, -1, -1);
	if (!found) {
		//gdk_drop_reply(drag_context, FALSE, time);
		return FALSE;
	}

	gtk_drag_get_data(widget, drag_context,
			  //t->background->match, 
			  GDK_POINTER_TO_ATOM(drag_context->targets->data),
			  time);
	return TRUE;
}


gboolean
on_secondary_widget_drag_drop          (GtkWidget       *widget,
                                        GdkDragContext  *drag_context,
                                        gint             x,
                                        gint             y,
                                        guint            time,
                                        gpointer         user_data)
{
#ifdef DND_DEBUG
	g_message("%s(): Starting.", __func__);
#endif
	return on_main_widget_drag_drop(widget, drag_context, x, y, time, user_data);
}


