%{
/*
 * $Id: du_parser.y,v 1.5 2006/05/27 17:20:03 skinner Exp skinner $
 *
 * $Log: du_parser.y,v $
 * Revision 1.5  2006/05/27 17:20:03  skinner
 * Draws a line for the directory "."
 *
 * Revision 1.4  2006/05/27 17:11:53  skinner
 * Processing humanreadanle filesizes.
 *
 * Revision 1.3  2006/05/27 16:29:22  skinner
 * Drawing size rectangles and the like.
 *
 * Revision 1.2  2006/05/27 14:01:34  skinner
 * need to find out how to draw pretty colour bars for filesizes.
 * Need to fix the space in file or directory names problem.
 *
 * Revision 1.1  2006/05/24 14:54:00  skinner
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "ls_parser_c.h"
#include "cuiterm.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "filename.h"
#include "dnd.h"

#ifdef ALL_DEBUG
#  define DU_DEBUG
#endif

extern CuiWindow *window_for_lex;

/*
 * We use this structure to store all processed lines, so we can go back when we
 * are finished with all the lines.
 */
typedef struct du_line {
  CuiLexElement *size;
  CuiLexElement *filename;
  double hsize;
} du_line;

static GList *list = NULL; /* A list to store the lines in */

static inline void
finalize_parser(void) {
  GList *l;
  du_line *line;
  l = list;
  while(l != NULL) {
    line = l->data;
    cui_lex_element_destroy(line->size, FALSE);
    g_free(line->size);

    cui_lex_element_destroy(line->filename, FALSE);
    g_free(line->filename);

    l = l->next;
  }
  g_list_free(list);
  list = NULL;
}

static inline void
append_line(CuiLexElement *size, CuiLexElement *filename) {
  du_line *new_line = g_new(du_line, 1);
  new_line->size = g_memdup(size, sizeof(CuiLexElement));
  new_line->filename = g_memdup(filename, sizeof(CuiLexElement));

  list = g_list_append(list, new_line);
}

static inline gdouble
get_max(void) {
  GList *le;
  du_line *line;
  gdouble retval = 0;

  le = list;
  /*
   * This is not really efficient.
   */
  while (le != NULL) {
    line = le->data;
    if(strcmp(line->filename->text,".") == 0) {
      retval = line->size->double_val;
    }

    le = le->next;
  }
  return retval;
}

static inline void
process_lines(CuiWindow *window) {
  GList *le;
  du_line *line;
  CuiFilename *fn;
  CuiWidget *widget;
  Background *background;
  gdouble max_size = get_max();
#ifdef DU_DEBUG
  g_message("%s: max_size = %f", __func__, max_size);
#endif

  g_return_if_fail(CUI_IS_WINDOW(window));
  background = cui_widget_get_background(CUI_WIDGET(window));
  /*
   * We go through the lines creating widgets.
   */
  le = list;
  while (le != NULL) {
    line = le->data;

    #ifdef DU_DEBUG
      g_message("%s: %s %s", __func__, line->size->text, line->filename->text);
    #endif

    fn = cui_filename_new(line->filename->text, window->wd->str);
    cui_file_menu_new_for_filename(window, background, fn,
      line->filename->begin,
      line->filename->end);

    /*
     * Colored sizebar.
     */
    if(strcmp(line->filename->text,"total") != 0) {
    widget = cui_rectangle_emboss_new();
    cui_widget_set_background(widget, background);
    cui_widget_set_area_by_marks(widget,line->size->begin,
      line->filename->begin);
    cui_widget_set_width_by_percent(widget,
      line->size->double_val / max_size);
    cui_rectangle_set_color(CUI_RECTANGLE(widget),
      "#8cfbef",
      "#a6fbf2",
      "#55fbe9");
    }
    /*
     * Drag & Drop stuff from ls_parser.y
     */
    GtkTargetList *tlist;
    widget = cui_rectangle_new();
    cui_widget_set_background(widget, background);
    cui_widget_set_area_by_marks(widget,
      line->size->begin,
      line->filename->end);
    cui_widget_set_width_by_percent(widget, 1.00);
    tlist = dnd_get_file_target_list();
    cui_rectangle_set_target_list(CUI_RECTANGLE(widget), tlist);
    cui_rectangle_set_target_object(CUI_RECTANGLE(widget),
      G_OBJECT(fn));
    cui_rectangle_set_is_dnd_target(CUI_RECTANGLE(widget), TRUE);
    cui_widget_append_window_command(widget, WindowRefresh, window);
    
    /*
     * dnd source for filename
     */
    widget = cui_rectangle_new();
    cui_widget_set_background(widget, background);
    cui_widget_set_area_by_marks(widget,
      line->filename->begin,
      line->filename->end);
    cui_rectangle_set_dnd_source_data(CUI_RECTANGLE(widget),
      TargetFileForName, fn, sizeof(CuiFilename));
    cui_rectangle_set_is_dnd_source(CUI_RECTANGLE(widget), TRUE);

next_line:
    le = le->next;
  }
}
%}

%parse-param {CuiWindow *window}
%union {
  CuiLexElement sval;
}

%token <sval> SIZE FILENAME TOTAL
%token END_OF_WINDOW

%%

lines: lines line
     | line
     ;

line: SIZE FILENAME {
      #ifdef DU_DEBUG
        g_message("%s: SIZE FILENAME", __func__);
      #endif
      append_line(&$1, &$2);

      CuiFilename *fn;
      Background *background;
      background = cui_widget_get_background(CUI_WIDGET(window));
      fn = cui_filename_new($2.text, window->wd->str);
      cui_filename_set_filetype_regular(fn);
      /* Action area */
      cui_action_area_new_for_filename(window, background, fn,
        $2.begin, $2.end);
      /* File menu */
      cui_file_menu_new_for_filename(window, background, 
        fn, $2.begin, $2.end);
      /* Filename syntax highlight */
      cui_syntax_highlight_apply_filename(background, fn, 
        $2.begin, $2.end);
      }
    | SIZE TOTAL  {
      #ifdef DU_DEBUG
        g_message("%s: SIZE TOTAL", __func__);
      #endif
     append_line(&$1, &$2);
      cui_window_apply_tag(window_for_lex, "Header", $1.begin, $2.end);
      }
    | END_OF_WINDOW { 
        #ifdef DU_DEBUG
          g_message("%s: END OF WINDOW", __func__);
        #endif
        Background *bg;
        if (list != NULL) {
          process_lines(window_for_lex);
          finalize_parser();
          bg = cui_widget_get_background(CUI_WIDGET(window));
        }
        return 0;
        }
    ;

%%

extern int du_lineno;

int du_error(CuiWindow *window, char *c) {
  gchar *error_message;

  error_message = g_strdup_printf(
    "Disk Usage Error: %s in line %d\n",
    c, du_lineno);
  g_warning(error_message);
  cui_window_set_title(window, error_message);
  g_free(error_message);
  return 1;
}

int du_wrap(void) {
  return 1;
}
