%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "fdisk_parser.h"
#include "terminal.h"
#include "background.h"
#include "cuiwindow.h"

static double device_size = 0;
extern CuiWindow *window_for_lex;

%}

%debug
%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
}

%token <sval> DISK BYTES HEADS ST CYLINDERS COMMENT
%token <sval> DEVICE
%token <sval> NUMBER UNIT
%token <sval> ACTIVE
%token <sval> SYSTEM

%token <sval> HEADER
%token <sval> NAME LINE 
%token <sval> MONTH DATE TIME 

%%

lines
  : lines line
  | line
  ;

line
  : DISK DEVICE ':' NUMBER UNIT ',' NUMBER BYTES
  | NUMBER HEADS ',' NUMBER ST ',' NUMBER CYLINDERS {
     	device_size = $7.double_val;
    }
  | COMMENT
  | HEADER
  | DEVICE active NUMBER NUMBER NUMBER NUMBER SYSTEM {
    	Background *background;
	CuiWidget *widget;

	background = cui_widget_get_background(CUI_WIDGET(window));
	widget = cui_rectangle_emboss_new();
	cui_widget_set_background(widget, background);
	cui_rectangle_set_color(CUI_RECTANGLE(widget), 
				"#76eceb",
				"#36ecea", 
				"#afeceb");
	cui_widget_set_area_by_marks(widget, $1.begin, $7.end);
	cui_widget_set_start_and_stop_by_percent(widget, 
		$3.double_val / device_size,
		$4.double_val / device_size);
    }
  ;

active
  :
  | ACTIVE
  ;
%%

extern int fdisk_lineno;

int fdisk_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	error_message = g_strdup_printf("Error in File: %s in lineno %d",
			c,
			fdisk_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return(1);
}

int fdisk_wrap(void)
{
	g_message("%s(): returning 1", __func__);
	return 1;
}
