/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: filename.h,v 1.1 2006/02/24 17:30:45 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: filename.h,v $
 * Revision 1.1  2006/02/24 17:30:45  pipas
 * Initial revision
 *
 */
#ifndef FILENAME_H
#define FILENAME_H

#define CUI_FILENAME_TYPE (cui_filename_get_type())
#define CUI_FILENAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_FILENAME_TYPE, CuiFilename))
#define CUI_FILENAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_FILENAME_TYPE, CuiFilenameClass))
#define CUI_IS_FILENAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_FILENAME_TYPE))
#define CUI_IS_FILENAME_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), \
			CUI_FILENAME_TYPE))
#define CUI_FILENAME_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), \
			CUI_FILENAME_TYPE, CuiFilenameClass))

typedef enum {
	MODE_UNKNOWN,
	MODE_YES,
	MODE_NO,
	MODE_EXTRA
} CuiFileMode;

typedef enum {
	FILETYPE_UNKNOWN,
	FILETYPE_FILE,
	FILETYPE_DIRECTORY,
	FILETYPE_BLOCK,
	FILETYPE_CHAR,
	FILETYPE_FIFO,
	FILETYPE_LINK,
	FILETYPE_SOCKET
} CuiFileType;

typedef enum {
	FileSizeUnknown,
	FileSizeSame,
	FileSizeIncreasing,
	FileSizeDecreasing
} CuiFileSizeChange;


/**
 * CuiFilename:
 *
 *
 */
typedef struct _CuiFilename CuiFilename;
struct _CuiFilename {
	/*< private >*/
	GObject parent;
	                              /* The mode as separated fields. */
	CuiFileType file_type;
	CuiFileMode user_read;
	CuiFileMode user_write;
	CuiFileMode user_execute;
	CuiFileMode group_read;
	CuiFileMode group_write;
	CuiFileMode group_execute;
	CuiFileMode others_read;
	CuiFileMode others_write;
	CuiFileMode others_execute;
	
	gchar             *filename;
	gchar             *directory;
	gchar             *mode_string;
	
	gchar             *owner;
	gchar             *group_owner;
	
	gint64             size;
	CuiFileSizeChange  sizechange;
	
	GTimeVal          *last_used;
	GTimeVal          *last_modified;
	GTimeVal          *last_inode_op;
};

typedef struct _CuiFilenameClass CuiFilenameClass;
struct _CuiFilenameClass {
	GObjectClass parent_class;
	GTree *dir_repository;
};

GType cui_filename_get_type(void);
CuiFilename *cui_filename_new(const char *file, const char *directory);
gchar *cui_remove_tilde(const gchar *directory);

gchar *cui_filename_get_filename(CuiFilename *file);
gchar *cui_filename_get_canonical_filename(CuiFilename *file, gchar *working_directory);
gchar *cui_filename_get_canonical_for_newname(CuiFilename *file, gchar *working_directory, gchar *newname);

gchar *cui_filename_get_owner(CuiFilename *file);
gchar *cui_filename_get_group(CuiFilename *file);

CuiFileSizeChange cui_filename_get_size_change(CuiFilename *filename);
void cui_filename_set_file_size(CuiFilename *filename, gint64 size);


void cui_filename_set_mode_string_owner(CuiFilename *filename, const gchar *string);
void cui_filename_set_mode_string_group(CuiFilename *filename, const gchar *string);
void cui_filename_set_mode_string_others(CuiFilename *filename, const gchar *string);
void cui_filename_set_mode_string(CuiFilename *filename, const gchar *mode_string);

void cui_filename_set_owner(CuiFilename *filename, const gchar *owner);
void cui_filename_set_group(CuiFilename *filename, const gchar *group);
void cui_filename_set_owner_and_group(CuiFilename *filename, 
		const gchar *owner, 
		const gchar *group);
CuiFileType cui_filename_get_filetype(CuiFilename *filename);
const gchar *cui_filename_get_typename(CuiFilename *filename);
gboolean cui_filename_is_compressed(CuiFilename *filename);
gboolean cui_filename_is_dir(CuiFilename *filename);
gboolean cui_filename_is_block_device(CuiFilename *filename);
gboolean cui_filename_is_character_device(CuiFilename *filename);
gboolean cui_filename_is_user_read(CuiFilename *filename);
gboolean cui_filename_is_user_write(CuiFilename *filename);
gboolean cui_filename_is_user_execute(CuiFilename *filename);
gboolean cui_filename_is_group_read(CuiFilename *filename);
gboolean cui_filename_is_group_write(CuiFilename *filename);
gboolean cui_filename_is_group_execute(CuiFilename *filename);
gboolean cui_filename_is_others_read(CuiFilename *filename);
gboolean cui_filename_is_others_write(CuiFilename *filename);
gboolean cui_filename_is_others_execute(CuiFilename *filename);
void cui_filename_set_filetype_dir(CuiFilename *filename);
void cui_filename_set_filetype_block(CuiFilename *filename);

void cui_filename_set_filetype_regular(CuiFilename *filename);
void cui_filename_set_filetype_link(CuiFilename *filename);
gchar *cui_filename_get_editor(CuiFilename *filename, gchar *long_name);
gchar *cui_filename_get_viewer(CuiFilename *filename, gchar *long_name);
gchar *cui_filename_get_processor(CuiFilename *filename, gchar *long_name);


#endif
