%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: finger_lexer.l,v 1.1 2006/02/24 08:28:04 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: finger_lexer.l,v $
 * Revision 1.1  2006/02/24 08:28:04  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "finger_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%option yylineno

%s LOGINSTATE NAMESTATE FILESTATE PLANSTATE

%%

  if (window_for_lex->parsed_chars == 0)
    BEGIN(0);


^"Login: " {
	#ifdef DEBUG
	g_message("%s(): LOGIN: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	BEGIN(LOGINSTATE);
	return LOGIN;
}

"Name: " {
	#ifdef DEBUG
	g_message("%s(): NAME: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	BEGIN(NAMESTATE);
	return NAME;
}

^"Directory: " {
	#ifdef DEBUG
	g_message("%s(): DIRECTORY: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	BEGIN(FILESTATE);
	return DIRECTORY;
}

"Shell: " {
	#ifdef DEBUG
	g_message("%s(): SHELL: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	BEGIN(FILESTATE);
	return SHELL;
}

("No mail.")|("Mail"[^\n]*) {
	#ifdef DEBUG
	g_message("%s(): SHELL: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "FgRed", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	return NOMAIL;
}

("Plan:")|("No Plan.") {
	#ifdef DEBUG
	g_message("%s(): PLAN: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "unimportant", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	BEGIN (PLANSTATE);
	return PLAN;
}

<PLANSTATE>[^\n]* {
	#ifdef DEBUG
	g_message("%s(): PLANLINE: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "comment", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	return PLANLINE;
}


<LOGINSTATE>[^\t ]+ {
	#ifdef DEBUG
	g_message("%s(): UNAME: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "username", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	BEGIN(0);
	return UNAME;
}

<NAMESTATE>[^\n]+ {
	#ifdef DEBUG
	g_message("%s(): RNAME: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	BEGIN(0);
	return RNAME;
}

<FILESTATE>[^\n ]+ {
	#ifdef DEBUG
	g_message("%s(): FOD: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	BEGIN(0);
	return FOD;
}

("On since"[^\n]*)|("Never logged in.") {
	#ifdef DEBUG
	g_message("%s(): ONS: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Dim", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	return ONS;
}

^[ \t]*[0-9]+[ \t]*("second"|"minute"|"hour")[^\n]* {
	#ifdef DEBUG
	g_message("%s(): IDLE: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Dim", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	return IDLE;
}

^[ \t]*[0-9]+[ \t]*("day"|"days"|"second"|"minute"|"hour")[^\n]* {
	#ifdef DEBUG
	g_message("%s(): IDLE: '%s'", __func__, yytext);
	#endif
	finger_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		finger_lval.sval.text,
		&finger_lval.sval.begin,
		&finger_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Dim", 
		finger_lval.sval.begin, 
		finger_lval.sval.end); 
	return IDLE;
}


[ \t]+ {
	#ifdef DEBUG
	g_message("%s(): FS: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

\n+ {
	#ifdef DEBUG
	//g_message("%s(): \\n: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

. {
	g_message("%s(): Unhandler Character Found: '%s'", __func__, yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

%%
