%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: finger_parser.y,v 1.1 2006/02/24 08:21:03 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: finger_parser.y,v $
 * Revision 1.1  2006/02/24 08:21:03  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "finger_parser.h"
#include "terminal.h"
#include "background.h"
#include "cuiwindow.h"

extern CuiWindow *window_for_lex;

%}

%debug
%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
}
 
%token <sval> LOGIN NAME DIRECTORY SHELL
%token <sval> UNAME RNAME FOD 
%token <sval> ONS IDLE
%token <sval> NOMAIL
%token <sval> PLAN PLANLINE

%%

lines
  : lines line
  | line
  ;

line
  : LOGIN UNAME NAME RNAME {
    		CuiWidget *widget;
		Background *background;
		
		widget = cui_user_menu_new($2.text);
		background = cui_widget_get_background(CUI_WIDGET(window));
	   	cui_widget_set_background(widget, background);
	   	cui_widget_set_area_by_marks(widget, $2.begin, $2.end);
		
		cui_lex_element_destroy(&$1, TRUE);
		cui_lex_element_destroy(&$2, TRUE);
		cui_lex_element_destroy(&$3, TRUE);
		cui_lex_element_destroy(&$4, TRUE);
    	}
  | DIRECTORY FOD SHELL FOD {
		gchar *filename = g_path_get_basename($2.text);
    		gchar *dirname = g_path_get_dirname($2.text);
		Background *background = 
				cui_widget_get_background(CUI_WIDGET(window));
		CuiFilename *fn;
	
		/*
		 * The home directory is always a directory. 
		 */
    		fn = cui_filename_new(filename, dirname); 
		cui_filename_set_filetype_dir(fn);
		cui_action_area_new_for_filename(window, background, fn,
			$2.begin, $2.end);
	   	cui_file_menu_new_for_filename(window, background, fn,
			$2.begin, $2.end);
	   	cui_entry_new_for_filename(window, background, fn, 
			$2.begin, $2.end, $2.text);
	   	cui_syntax_highlight_apply_filename(background, fn,
				$2.begin, $2.end);
		g_free(filename);
		g_free(dirname);
		
		/*
		 * The shell is always a regular executable file. We hope we
		 * have this file in the filename cache, otherwise we don't no
		 * nothing about it.
		 */
		filename = g_path_get_basename($4.text);
		dirname = g_path_get_dirname($4.text);
    		
		fn = cui_filename_new(filename, dirname); 
		cui_filename_set_filetype_regular(fn);
		
	   	cui_syntax_highlight_apply_filename(background, fn,
				$4.begin, $4.end);

		cui_lex_element_destroy(&$1, TRUE);
		cui_lex_element_destroy(&$2, TRUE);
		cui_lex_element_destroy(&$3, TRUE);
		cui_lex_element_destroy(&$4, TRUE);
    	}
  | ONS         { cui_lex_element_destroy(&$1, TRUE);}
  | IDLE        { cui_lex_element_destroy(&$1, TRUE);}
  | NOMAIL      { cui_lex_element_destroy(&$1, TRUE);}
  | PLAN        { cui_lex_element_destroy(&$1, TRUE);}
  | PLANLINE    { cui_lex_element_destroy(&$1, TRUE);}
  | error { window->error_code++; }
  ;

%%

extern int finger_lineno;

int finger_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	error_message = g_strdup_printf(
		"Error in User Information: %s in line %d", 
		c,
		finger_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return(1);
}

int finger_wrap(void)
{
#ifdef DEBUG
	g_message("%s(): returning 1", __func__);
#endif
	return 1;
}
