%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: general_lexer.l,v 1.1 2006/02/24 09:03:21 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: general_lexer.l,v $
 * Revision 1.1  2006/02/24 09:03:21  pipas
 * Initial revision
 *
 */

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include <stdlib.h>
#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "general_parser.h"
#if DEBUG_ALL
#  define DEBUG
#endif

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option yylineno

%x ERROR_LINE ERROR_WORD TRYL COMM
%x PWD CD

%%
	if (window_for_lex->parsed_chars == 0)
		BEGIN(0);

<*>^[-]?"bash:"[^:]+": command not found" {
	#ifdef DEBUG
	g_message("%s: COMMAND_NOT_FOUND: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "error_message",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	return COMMAND_NOT_FOUND;
}

<*>^[-]?"bash:"[^\n]* {
	#ifdef DEBUG
	g_message("%s: GENERAL_BASH_ERROR: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "error_message",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	return GENERAL_BASH_ERROR;
}

<*>"Only root can do that." {
	#ifdef DEBUG
	g_message("%s: ONLY_ROOT: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "error_message",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	return ONLY_ROOT;
}

<*>^[^:]{1,16}/: {
	/*
	 * If the line starts with a name, a ':' cahacter and the name is the
	 * command name we can assume that this is an error message.
	 */
	if (strcmp(yytext, parse_tree_get_producer_program(terminal_for_lex->parse_tree)) != 0){
#ifdef DEBUG
	g_message("'%s' != '%s' REJECT", yytext, 
	parse_tree_get_producer_program(terminal_for_lex->parse_tree));
#endif
		REJECT;
	}
	#ifdef DEBUG
	g_message("%s: APPLICATION_NAME: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "command_name",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	BEGIN(ERROR_LINE);
	return APPLICATION_NAME;
}

<ERROR_LINE>^"Try "/` {
	#ifdef DEBUG
	g_message("%s: TRY: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	BEGIN(TRYL);
	return TRY;
}

<ERROR_LINE>^"Próbáld a "/` {
	#ifdef DEBUG
	g_message("%s: TRY: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	BEGIN(TRYL);
	return TRY;
}

<COMM,TRYL>[`'] {
	#ifdef DEBUG
	g_message("%s: One of [`']: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	if (yytext[0] == '`')
		BEGIN(COMM);
	if (yytext[0] == '\'')
		BEGIN(TRYL);
	return yytext[0];
}

<COMM>[a-zA-Z0-9]+ {
	#ifdef DEBUG
	g_message("%s: COMMAND: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "command_name",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	return COMMAND;
}

<COMM>-{1,2}[-a-zA-Z0-9_]+ {
	#ifdef DEBUG
	g_message("%s: OPTION: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "option",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	return OPTION;
}


<ERROR_LINE>[^ \n:]+ {
	#ifdef DEBUG
	g_message("%s: ERRWORD: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "error_message",
		general_lval.sval.begin, 
		general_lval.sval.end); 
	return ERRWORD;
}

<TRYL>[^ \n`']+ { 
	/* Dropping it away */
	#ifdef DEBUG
	g_message("%s: dropping3: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}


[^ ]+ { 
	/* Dropping it away */
	#ifdef DEBUG
	g_message("%s: dropping1: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<*>[ \t\n]+ { 
	/* Dropping it away */
	#ifdef DEBUG
	g_message("%s: dropping2: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<*>.	{
	#ifdef DEBUG
	g_message("%s: OTHER_CHAR: '%s'", __func__, yytext);
	#endif
	general_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		general_lval.sval.text,
		&general_lval.sval.begin,
		&general_lval.sval.end);
	return yytext[0];
}

%%

