%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: general_parser.y,v 1.1 2006/02/24 08:20:04 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: general_parser.y,v $
 * Revision 1.1  2006/02/24 08:20:04  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "support.h"
#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "widgets.h"
#include "cuiwindow.h"

int general_wrap(void);
int general_error(CuiWindow *window, char *);

extern CuiWindow *window_for_lex;
static CuiLexElement fll;
static CuiFilename *fn;
%}

%debug
%parse-param {CuiWindow *window}
%union {
	CuiLexElement sval;
};

%token <sval> COMMAND_NOT_FOUND GENERAL_BASH_ERROR ONLY_ROOT
%token <sval> APPLICATION_NAME ERRWORD
%token <sval> TRY COMMAND OPTION
%%

lines: 
     | line                        
     | lines line                  
     ;

line: COMMAND_NOT_FOUND             { window->error_code = 1; }
    | GENERAL_BASH_ERROR            { window->error_code = 2; }
    | APPLICATION_NAME ':' errline  { window->error_code = 3; }
    | ONLY_ROOT                     { window->error_code = 4; }
    | errline                       
    | TRY tryline                   
    | error { window->error_code++; }
    ;

errline: ERRWORD              
       | errline ERRWORD
       | errline ':'
       ;
       
tryline: command              
       | tryline command      
       | tryline ERRWORD      
       ;

command: '`' COMMAND OPTION '\'' {
       		GtkWidget *appbar;
		CuiWidget *aa;
		gchar *tt;
		Background *background;
		background = cui_widget_get_background(CUI_WIDGET(window));
		aa = cui_action_area_new();
		cui_widget_set_background(aa, background);
		appbar = lookup_widget(GTK_WIDGET(window->text_view), 
			"appbar1");
		cui_widget_set_appbar(aa, appbar);
		cui_widget_set_area_by_marks(aa, $2.begin, $3.end);
		tt = g_strdup_printf("Double click to run '%s %s'.",
				$2.text, $3.text);
		cui_widget_set_tooltip(aa, tt);
		
		cui_widget_append_window_command_with_arg(aa,
				WindowDirectCommand,
				window,
				g_strconcat($2.text, " ", $3.text, NULL));
		g_free(tt);
       }
       ;

%%

extern int general_lineno;

int general_error(CuiWindow *window, char *c){
	//yydebug = 1;
	g_warning(
		"Error parsing the with the general parser: %s in line %d", 
		c,
		general_lineno);
	return(1);
}

int general_wrap(void)
{
#ifdef DEBUG
	g_message("End of file");
#endif
	return 1;
}

