%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include "cuiwindow.h"
#include "cuilexelements.h"
#include "bash_parser_c.h"
#include "getfacl_parser.h"

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%s FILENAME_STATE OWNERNAME_STATE GROUPOWNERNAME_STATE 
%s GROUPNAME_STATE USERNAME_STATE 
%s RIGHTS_FOLLOWS_STATE RIGHTS_STATE
%s EAT_UP_STATE

%option yylineno

%%
^"# file: " {
	#ifdef DEBUG
	g_message("%s: FILE_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "comment",
		getfacl_lval.sval.begin, 
		getfacl_lval.sval.end); 
	BEGIN(FILENAME_STATE);
	return FILE_LINE;
}

^"# owner: " {
	#ifdef DEBUG
	g_message("%s: OWNER_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "comment",
		getfacl_lval.sval.begin, 
		getfacl_lval.sval.end); 
	BEGIN(OWNERNAME_STATE);
	return OWNER_LINE;
}

^"# group: " {
	#ifdef DEBUG
	g_message("%s: GROUPOWNER_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "comment",
		getfacl_lval.sval.begin, 
		getfacl_lval.sval.end); 
	BEGIN(GROUPOWNERNAME_STATE);
	return GROUPOWNER_LINE;
}

^"user::" {
	#ifdef DEBUG
	g_message("%s: OWNER_RIGHTS_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(RIGHTS_STATE);
	return OWNER_RIGHTS_LINE;
}

^"user:" {
	#ifdef DEBUG
	g_message("%s: USER_RIGHTS_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(USERNAME_STATE);
	return USER_RIGHTS_LINE;
}

^"group:" {
	#ifdef DEBUG
	g_message("%s: GROUP_RIGHTS_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(GROUPNAME_STATE);
	return GROUP_RIGHTS_LINE;
}

^"group::" {
	#ifdef DEBUG
	g_message("%s: GROUPOWNER_RIGHTS_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(RIGHTS_STATE);
	return GROUPOWNER_RIGHTS_LINE;
}

^"other::" {
	#ifdef DEBUG
	g_message("%s: OTHER_RIGHTS_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(RIGHTS_STATE);
	return OTHER_RIGHTS_LINE;
}

^"mask::" {
	#ifdef DEBUG
	g_message("%s: MASK_RIGHTS_LINE: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(RIGHTS_STATE);
	return MASK_RIGHTS_LINE;
}

<FILENAME_STATE>[^\n\r]+ {
	#ifdef DEBUG
	g_message("%s: FILENAME: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(0);
	return FILENAME; 
}

<USERNAME_STATE>[^:]+ {
	#ifdef DEBUG
	g_message("%s: USERNAME: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(RIGHTS_FOLLOWS_STATE);
	return USERNAME; 
}

<GROUPNAME_STATE>[^:]+ {
	#ifdef DEBUG
	g_message("%s: GROUPNAME: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(RIGHTS_FOLLOWS_STATE);
	return GROUPNAME; 
}

<RIGHTS_FOLLOWS_STATE>":" {
	#ifdef DEBUG
	g_message("%s: dropping: <%s>", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(RIGHTS_STATE);
}

<OWNERNAME_STATE>[^\n\r]+ {
	#ifdef DEBUG
	g_message("%s: OWNERNAME: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(0);
	return OWNERNAME; 
}

<GROUPOWNERNAME_STATE>[^\n\r]+ {
	#ifdef DEBUG
	g_message("%s: GROUPNAME: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(0);
	return GROUPOWNERNAME; 
}

<RIGHTS_STATE>[-r][-w][-x] {
	#ifdef DEBUG
	g_message("%s: RIGHTS: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	BEGIN(EAT_UP_STATE);
	return RIGHTS; 
}

<EAT_UP_STATE>[ \t]+#[^\n\r]* {
	/*
 	 * This state will eat up the entire line (eg. "#effective:" lines).
	 */
	#ifdef DEBUG
	g_message("%s: dropping to EOL: <%s>", __func__, yytext);
	#endif
	getfacl_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		getfacl_lval.sval.text,
		&getfacl_lval.sval.begin,
		&getfacl_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "comment",
		getfacl_lval.sval.begin, 
		getfacl_lval.sval.end); 
	BEGIN(0);
}

<EAT_UP_STATE>[\n\r] {
	#ifdef DEBUG
	g_message("%s: dropping: <%s>", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(0);
}


<*>[\n\r]+ {
	#ifdef DEBUG
	g_message("%s: dropping: <%s>", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<*>.|\n {
	//g_warning("%s(): unhandled character: '%s' -> '%d'", 
	//	__func__, yytext, yytext[0]);
}

%%
