/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id:$
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log:$
 */
#ifndef HISTORY_H
#define HISTORY_H

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gtk/gtk.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpathInternals.h>
#include <libxml/tree.h>

#include "cuilexelements.h"

typedef enum ApplicationOpcode {
	OpTryAdd,
	OpReset,
	OpSearch
} ApplicationOpcode;

typedef struct file_read_progress {
	GtkWidget *appbar;
	gint       size;
	gint       red;
	gint       ifile;
} file_read_progress;

typedef struct app_freq {
	gchar *appname;
	gint  *freq;
} app_freq;

#if 0 
typedef struct _program_option program_option;
struct _program_option {
	gchar *option;
	gchar *description;
};
#endif

#if 0 
typedef struct _program_info program_info;
struct _program_info {
	gchar *name;		/* The name of the program. 		*/
	gchar *info;		/* One line description.		*/
	GList *longopts;
	GList *opts;
};
#endif 

typedef enum {
	TypingNotChecked,
	TypingError,
	TypingCorrect
} TypingCheckResult;

/*
 * Handling programs.
 */
gboolean program_add_description(gchar *name, gchar *descr);
gboolean program_add_long_description(gchar *name, gchar *longdesc); 
const gchar *program_get_long_description(gchar *name);
const gchar *program_get_description(gchar *name);

/*
 * Handling options.
 */
xmlNodePtr program_add_long_option(gchar *name, gchar *long_option);
xmlNodePtr program_add_option(gchar *name, gchar *option);

gboolean program_add_option_with_descr( gchar *name, gchar *option, gchar *descr);

gpointer history_read_bash_history(GtkWidget *widget);
gchar *handle_text_editors(gchar *appname, ApplicationOpcode opcode);
gboolean type_check_command_name(CuiLexElement *lexelement);
gboolean type_check_long_option(CuiLexElement *longopt, gchar *name);

GList *program_get_option_list(const gchar *name, const gchar *restraint);
gchar *program_get_option_description( const gchar *name, const gchar *option);

const gchar *program_get_argument_category( const gchar *name, gint position);
gboolean program_get_argument_optional( const gchar *name, gint position);
gboolean program_get_argument_iterate( const gchar *name, gint position);


gboolean program_info_in_cache(gchar *name);
gboolean program_add_longopt(gchar *name, gchar *longopt);
gboolean program_add_longopt_with_descr(
	gchar *name, 
	gchar *longopt,
	gchar *descr);
gboolean program_add_options_with_descr(
	gchar *name,
	gchar *option,
	gchar *longopt,
	gchar *descr);

void program_reset_all_history_counter(void);
void program_increment_option_history_counter(const gchar *name, const gchar *option);


TypingCheckResult 
	program_check_longopt_typing(
			const gchar *name, 
			const gchar *longopt);

void on_history_close_button_clicked(GtkButton *button, gpointer user_data);
void on_history_treeview_realize(GtkWidget *widget, gpointer user_data);

xmlNodePtr synopsis_create_node(const gchar *name);
xmlNodePtr synopsis_create_node_with_child(const gchar *name, xmlNodePtr child);
xmlNodePtr synopsis_create_node_with_option(const gchar *option);
xmlNodePtr synopsis_create_node_with_long_option(const gchar *option);
xmlNodePtr synopsis_create_node_with_option_argument(const gchar *option, const gchar *argument, const gchar *op);
xmlNodePtr synopsis_create_node_with_long_option_argument(const gchar *option, const gchar *argument, const gchar *op);

xmlNodePtr synopsis_create_operator_node(
		gchar operator,
		xmlNodePtr arg1,
		xmlNodePtr arg2);

xmlNodePtr synopsis_create_alternative(xmlNodePtr arg1, xmlNodePtr arg2);

xmlNodePtr synopsis_catenate_nodes(xmlNodePtr node1, xmlNodePtr node2);

xmlNodePtr synopsis_set_node_optional(xmlNodePtr node);
xmlNodePtr synopsis_set_node_iterate(xmlNodePtr node);

void synopsis_add_node_to_program(const gchar *name, xmlNodePtr node);
void synopsis_remove_nodes_from_program(const gchar *name);



#endif
