/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: icons.c,v 1.1 2006/02/24 17:19:21 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: icons.c,v $
 * Revision 1.1  2006/02/24 17:19:21  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif
#ifdef ALLDEBUG
#  define ICON_DEBUG
#endif

#include <stdlib.h>
#include <gnome.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include "configure.h"
#include "icons.h"

/*****************************************************************************
 * Class: CuiIcon a picture object representing files.                       *
 *                                                                           *
 *****************************************************************************/
static void
cui_icon_set_property(GObject *object, 
		guint           property_id, 
		const GValue    *value, 
		GParamSpec      *pspec);
static void
cui_icon_get_property(GObject *object, 
		guint            property_id, 
		GValue           *value, 
		GParamSpec       *pspec);

static void cui_icon_load(CuiIcon *icon);
static void set_icon_width(CuiIcon *icon);

GnomeThumbnailFactory *gnome_thumbnail_factory = NULL;




#define PROPERTY_FILE 1
#define PROPERTY_MAXSIZE 2
#define PROPERTY_MAX_HEIGHT 3
#define PROPERTY_PIXBUF 4

static void 
cui_icon_class_init(
		gpointer g_class, 
		gpointer g_class_data) 
{
	GObjectClass *gobject_class = G_OBJECT_CLASS(g_class);
	CuiIconClass *klass = CUI_ICON_CLASS(g_class);
	GParamSpec *pspec;
	/*
	 * Properties of the CuiIcon class.
	 */
	gobject_class->set_property = cui_icon_set_property;
  	gobject_class->get_property = cui_icon_get_property;
	pspec = g_param_spec_pointer("file", "filename", 
			"The CuiFilename this icon is representing.", 
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROPERTY_FILE, pspec);
	
	pspec = g_param_spec_int("max_size", "max-size",
			"The maximum height and width in pixels.",
			10, 1024, 32, 
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROPERTY_MAXSIZE, pspec);
	
	pspec = g_param_spec_int("max_height", "max-height",
			"The maximum height in pixels.",
			10, 1024, 32, 
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROPERTY_MAX_HEIGHT, pspec);
			
	pspec = g_param_spec_pointer("pixbuf", "gdkpixbuf", 
			"The icon in GdkPixbuf format.", 
			G_PARAM_READWRITE);
	g_object_class_install_property(gobject_class, PROPERTY_PIXBUF, pspec);
	
	klass->icon_theme = gtk_icon_theme_get_default();
}

static void
cui_icon_base_class_init(CuiIconClass *klass)
{
}

static void
cui_icon_instance_init(
		GTypeInstance   *instance, 
		gpointer         g_class)
{
	CuiIcon *self = (CuiIcon *)instance;
	CuiIconClass *klass = (CuiIconClass *)g_class;

	self->pixbuf = NULL;
	self->max_size = -1;
	self->max_height = -1;
	self->filename = NULL;
}

static void
cui_icon_set_property(GObject *object, 
		guint           property_id, 
		const GValue    *value, 
		GParamSpec      *pspec)
{
	CuiIcon *self = (CuiIcon *)object;
	
	switch(property_id){
		case PROPERTY_FILE:
			//g_return_if_fail(!CUI_IS_FILENAME(g_value_get_pointer(value)));
			if (self->filename != NULL)
				g_object_unref(G_OBJECT(self->filename));
			self->filename = g_value_get_pointer(value);
			g_object_ref(G_OBJECT(self->filename));
			cui_icon_load(self);
			break;
			
		case PROPERTY_MAXSIZE:
			self->max_size = g_value_get_int(value);
			set_icon_width(self);
			break;

		case PROPERTY_MAX_HEIGHT:
			self->max_height = g_value_get_int(value);
			set_icon_width(self);
			break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
	}
}

static void
cui_icon_get_property(GObject *object, 
		guint            property_id, 
		GValue           *value, 
		GParamSpec       *pspec)
{
	CuiIcon *self = (CuiIcon *) object;
	switch(property_id){
		case PROPERTY_FILE:
			g_value_set_pointer(value, self->filename);
			break;
		
		case PROPERTY_PIXBUF:
			g_value_set_pointer(value, self->pixbuf);
			break;

		default:
			G_OBJECT_WARN_INVALID_PROPERTY_ID(object, property_id, pspec);
	}

}

GType 
cui_icon_get_type(void) 
{
	static GType type = 0;
	
	if (type == 0) {
		static const GTypeInfo info = {
			sizeof(CuiIconClass),
			(GBaseInitFunc) cui_icon_base_class_init, 
			(GBaseFinalizeFunc) NULL, 
			(GClassInitFunc) cui_icon_class_init, 
			NULL,
			NULL,
			sizeof(CuiIcon),
			0,
			(GInstanceInitFunc) cui_icon_instance_init 
		};
		type = g_type_register_static(G_TYPE_OBJECT, 
				"CuiIcon", &info, 0);
	}
	return type;
}

static void
set_icon_width(CuiIcon *icon)
{
	gint      width, height;
	gint      new_width, new_height;
	gdouble   ratio;
	
	/*
	 * If we have a max height we set the size according that.
	 */
	if (icon->max_height > 0) {
		width = gdk_pixbuf_get_width(icon->pixbuf);
		height = gdk_pixbuf_get_height(icon->pixbuf);
		if (height <= icon->max_height)
			return;
		ratio = (gdouble)width / height;
		new_height = icon->max_height;
		new_width = ratio * new_height;
		icon->pixbuf = gnome_thumbnail_scale_down_pixbuf(icon->pixbuf, 
				new_width, new_height);
		return;
	}
	
	if (icon->max_size < 0)
		return;
	
	width = gdk_pixbuf_get_width(icon->pixbuf);
	height = gdk_pixbuf_get_height(icon->pixbuf);
	if (width > height) {
		ratio = (gdouble)height / width;
		new_width = icon->max_size;
		new_height = icon->max_size * ratio;
	}else{
		ratio = (gdouble)width / height;
		new_width = icon->max_size * ratio;
		new_height = icon->max_size;
	}
	/*
	 * The gnome_thumbnail_scale_down_pixbuf can not magnify pictures. 
	 */
	if (new_width <= width && new_height <= height)
		icon->pixbuf = gnome_thumbnail_scale_down_pixbuf(icon->pixbuf, 
				new_width, new_height);
}


static void
init_icon_by_file(CuiIcon *icon)
{
	GdkPixbuf       *pixbuf;
	CuiIconClass    *klass = g_type_class_peek(CUI_ICON_TYPE);
	gchar           *filename;
	gchar           *uri;
	gchar           *mime;
	gchar           *icon_name;
	gchar           *thumbfile;
	gint             size;
	struct stat      stat_buff;
	GnomeIconLookupResultFlags results;
#ifdef DEBUG
g_message("%s(): Start", __func__);
#endif
	filename = cui_filename_get_canonical_filename(icon->filename, "");
	/* 
	 * FIXME: this should be somewhere else...
	 * 
	 */
	if (filename[0] == '~'){
		gchar *tmp;
		tmp = g_build_filename(getenv("HOME"), filename + 2, NULL);
		g_free(filename);
		filename = tmp;
	}
	
	/*
	 * For the thumbnail we have to stat the file to get the modification
	 * time. If the file can't be stat-ed, we simply jump over, since it
	 * must be a problem to access the file anyway.
	 */
	if (stat(filename, &stat_buff) != 0)
		goto then_again;
	/*
	 * First we try to use a thumbnail. If we can, we are happy with it.
	 */
	uri = gnome_vfs_get_uri_from_local_path(filename);
	mime = gnome_vfs_get_mime_type(uri);
	thumbfile = gnome_thumbnail_factory_lookup(gnome_thumbnail_factory, uri, stat_buff.st_mtime);
	if (thumbfile != NULL) {
		icon->pixbuf = gdk_pixbuf_new_from_file(thumbfile, NULL);
		if (icon->pixbuf != NULL) {
			g_free(thumbfile);
			g_free(uri);
			g_free(mime);
			return;
		}else{
			g_free(thumbfile);
		}
	}
	if (gnome_thumbnail_factory_can_thumbnail(gnome_thumbnail_factory, uri, mime, stat_buff.st_mtime)) {
		icon->pixbuf = gnome_thumbnail_factory_generate_thumbnail(gnome_thumbnail_factory, uri, mime);
		if (icon->pixbuf != NULL){
			gnome_thumbnail_factory_save_thumbnail(gnome_thumbnail_factory, 
					icon->pixbuf, uri, stat_buff.st_mtime);
			g_free(uri);
			g_free(mime);
			g_free(filename);
			return;
		}else{
			g_free(mime);
		}
	}
	/*
	 * If we can't make a thumbnail, we get an icon.
	 */
then_again:
	size = icon->max_size < 0 ? 256 : icon->max_size;
	icon_name = gnome_icon_lookup_sync(klass->icon_theme, gnome_thumbnail_factory, uri, NULL, 
			0, //GNOME_ICON_LOOKUP_FLAGS_EMBEDDING_TEXT,
			&results);
	icon->pixbuf = gtk_icon_theme_load_icon(klass->icon_theme, icon_name, size, 
			GTK_ICON_LOOKUP_USE_BUILTIN, 
			NULL);	
}

static void
cui_icon_load(CuiIcon *icon)
{
	if (icon->filename != NULL)
		init_icon_by_file(icon);
	else
		g_warning("%s(): Can't init icon, no source file set.", __func__);

	set_icon_width(icon);
}

/**
 * cui_icon_new:
 * @file: a #CuiFilename 
 * @returns: a new #CuiIcon object 
 *
 * Creates a new icon for the given file. FIXME: this function should be
 * completed.
 */
CuiIcon *
cui_icon_new(CuiFilename *file)
{
	CuiIcon *icon;
	
#ifdef DEBUG
g_message("%s(): Start", __func__);
#endif
	g_return_val_if_fail(CUI_IS_FILENAME(file), NULL);
	icon = g_object_new(CUI_ICON_TYPE, NULL);
	g_object_set(G_OBJECT(icon), "file", file, NULL);
	return icon;
}

/**
 * cui_icon_get_pixbuf:
 *
 */
GdkPixbuf *
cui_icon_get_pixbuf(CuiIcon *icon)
{
	GdkPixbuf *pixbuf;
#ifdef DEBUG
g_message("%s(): Start", __func__);
#endif
	g_return_val_if_fail(CUI_IS_ICON(icon), NULL);
	g_object_get(G_OBJECT(icon), "pixbuf", &pixbuf, NULL);
	return pixbuf;
}

