%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: make_lexer.l,v 1.1 2006/02/24 09:01:49 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: make_lexer.l,v $
 * Revision 1.1  2006/02/24 09:01:49  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

/****************************************************************************
 *                                                                          *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/
#include <gnome.h>
#include <string.h>
#include "cuiterm.h"
#include "cuilexelements.h"
#include "cuiwindow.h"
#include "make_parser.h"

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option yylineno
%x MESSAGE_STATE FILENAME_STATE FILENAME1_STATE

%%
\r|\n|(\r\n)|(\n\r) {
#ifdef DEBUG
	g_message("skipping: '%s'", yytext);
#endif
	/* Just for skipping to the next token */
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);	
}

(" "+)|":" {
#ifdef DEBUG
	g_message("skipping: '%s'", yytext);
#endif
	/* Just for skipping to the next token */
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);	
}

<*>[\t\n\r] {
#ifdef DEBUG
	g_message("skipping: '%s'", yytext);
#endif
	/* Just for skipping to the next token */
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

^("gcc"|"make"|"cd") {
#ifdef DEBUG
	g_message("COMMAND: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "command_name", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return(COMMAND);
}

^\/[^\ ]+ {
#ifdef DEBUG
	g_message("COMMAND: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "command_name", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return(COMMAND);
}

^make\[[0-9]+\]: {
#ifdef DEBUG
	g_message("MAKE_COMMAND: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
		#if 0
		// FIXME: What color should we use for these message ids?
	cui_window_apply_tag(window_for_lex, "command_name", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
		#endif
	BEGIN(MESSAGE_STATE);
  	return(MAKE_COMMAND);
}

<MESSAGE_STATE>[\t ]*"Entering directory"[ \t]* {
#ifdef DEBUG
	g_message("ENTERING: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "important", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
	BEGIN(FILENAME_STATE);
  	return ENTERING;
}


<MESSAGE_STATE>[\t ]*"Leaving directory"[ \t]* {
#ifdef DEBUG
	g_message("LEAVING: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "important", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
	BEGIN(FILENAME_STATE);
  	return LEAVING;
}


<MESSAGE_STATE>[\t ]*"Nothing to be done for"[^\n]* {
#ifdef DEBUG
	g_message("NOTHING_TO_DO: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "important", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
	BEGIN(0);
  	return NOTHING_TO_DO;
}

<FILENAME_STATE>"/" {
#ifdef DEBUG
	g_message("ROOT_DIR: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	BEGIN(FILENAME1_STATE);
  	return ROOT_DIR;
}

<FILENAME1_STATE>[^/\n\r'`]+ {
#ifdef DEBUG
	g_message("DIRNAME: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
  	return DIRNAME;
}

<FILENAME1_STATE>"/" {
#ifdef DEBUG
	g_message("DIRSEP: '%s'", yytext);
#endif
	make_lval.sval.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
  	return DIRSEP;
}

<FILENAME_STATE,FILENAME1_STATE>[`] {
#ifdef DEBUG
	g_message("delimiter: '%s'", yytext);
#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	return DELIM_START;
}

<FILENAME_STATE,FILENAME1_STATE>['] {
#ifdef DEBUG
	g_message("delimiter: '%s'", yytext);
#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(0);
	return DELIM_END;
}


"error:" {
#ifdef DEBUG
	g_message("ERROR: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "error", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return(ERROR);
}

"warning:" {
#ifdef DEBUG
	g_message("WARNING: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "warning_message", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return(WARNING);
}

^[a-zA-Z0-9_]+\.[chylf]/: {
#ifdef DEBUG
	g_message("SOURCE_FILE: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "filename", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
	return SOURCE_FILE;
}	

[0-9]+ {
#ifdef DEBUG
	g_message("NUMBER: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
  	return NUMBER;
}

"assignment makes pointer from integer without a cast" {
#ifdef DEBUG
	g_message("MESSAGE_POINT_INT: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "error_message", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return MESSAGE_POINT_INT;
}

"comparison between pointer and integer" {
#ifdef DEBUG
	g_message("MESSAGE_POINT_INT: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "error_message", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return MESSAGE_POINT_INT;
}

[\`\‘][a-zA-Z0-9_]+[\'\’]" undeclared (first use in this function)" {
#ifdef DEBUG
	g_message("MESSAGE_UNDECLARED: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "error_message", 
		make_lval.sval.begin, 
		make_lval.sval.end); 
  	return MESSAGE_UNDECLARED;
}

[^\t \n\r:]+ {
#ifdef DEBUG
	g_message("STRING: '%s'", yytext);
#endif
	make_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		make_lval.sval.text,
		&make_lval.sval.begin,
		&make_lval.sval.end);
  	return STRING;
}


%%
