%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: man_lexer.l,v 1.1 2006/02/24 08:58:09 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: man_lexer.l,v $
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpathInternals.h>
#include <libxml/tree.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "man_parser.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"

static GList *program_names = NULL;
static gchar *section = NULL;
static gchar *header_name = NULL;

static void 
add_program_name(const gchar *name)
{
	program_names = g_list_append(program_names, g_strdup(name));
}

static gboolean
is_program_name(const gchar *word)
{
	GList    *li;
	gchar    *basename;
	gboolean  same;
	
	g_assert(word != NULL);
	
	li = program_names;
	while (li != NULL) {
		if (li->data != NULL && strcmp(li->data, word) == 0)
			return TRUE;
		basename = g_path_get_basename(word);
		same = strcmp(li->data, basename) == 0;
		g_free(basename);
		if (same)
			return TRUE;
		li = li->next;
	}
	return FALSE;
}

static void
drop_program_names(void)
{
	GList *li;
	
	li = program_names;
	while (li != NULL) {
		g_free(li->data);
		li = li->next;
	}

	g_list_free(program_names);
	program_names = NULL;
}

#define PUSH_BACK_ALL \
{\
  gint  n;\
  gchar *copy = g_strdup(yytext);\
  for (n = yyleng - 1; n >= 0; --n){\
    unput(copy[n]);\
  }\
  g_free(copy);\
}



gint paragraph_indent;

//#define DEBUG_SYN
//#define DEBUG_HEADER
//#define DEBUG_UNKNOWN
//#define DEBUG_PARAGRAPH
//#define DEBUG_DESCRIPTION
//#define DEBUG

#undef YY_INPUT
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))

%}

%option yylineno
%option stack

%x  ST_START ST_UNKNOWN_START
%x  ST_ERROR
%x  ST_HEADER ST_HEADER_SECTION ST_HEADER_EAT
%x  ST_SHORT_DESCRIPTION
%x  ST_DESCRIPTION ST_NAME ST_SYNOPSIS ST_SYNOPSIS_CC 
%x  ST_SENTENCE ST_SENTENCE_STARTING 
%x  ST_PARAGRAPH_STARTING ST_PARAGRAPH ST_PARAGRAPH_REMAINDER
%x  ST_WORDS
%x  ST_SYN_ARG

FS                [ \t]+
CAPITAL           [A-ZÍÖÜÓŐÚÉÁŰ]
LETTER            [A-ZÍÖÜÓŐÚÉÁŰa-zíöüóőúéáű]
NOTLETTER         [A-ZÍÖÜÓŐÚÉÁŰa-zíöüóőúéáű]
ALPHNUM           [A-ZÍÖÜÓŐÚÉÁŰa-zíöüóőúéáű0-9]
CHAPTER_TITLE     [A-ZÍÖÜÓŐÚÉÁŰa-zíöüóőúéáű][- A-ZÍÖÜÓŐÚÉÁŰa-zíöüóőúéáű]+

%%
	if (window_for_lex->parsed_chars == 0){
		drop_program_names();
		cui_window_set_title(window_for_lex, _("Manual Page"));
		BEGIN(ST_HEADER);
	}
	if (window_for_lex->error_code > 0)
		BEGIN(ST_ERROR);

	/* 
	 * The header. We want to analyze the header for we have to know the section number. 
	 */
<ST_HEADER>^{ALPHNUM}+ {
	#ifdef DEBUG_HEADER
	g_message("%s(): HEADER_NAME: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", man_lval.sval.begin, man_lval.sval.end); 
	cui_window_apply_tag(window_for_lex, "command_name", man_lval.sval.begin, man_lval.sval.end); 
	
	g_free(header_name);
	header_name = g_strdup(yytext);
	return HEADER_NAME;
}

<ST_HEADER>\( {
	#ifdef DEBUG_HEADER
	g_message("%s(): HEADER'(': '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", man_lval.sval.begin, man_lval.sval.end); 
	BEGIN(ST_HEADER_SECTION);
	return '(';
}

<ST_HEADER_SECTION>[^\)]+ {
	#ifdef DEBUG_HEADER
	g_message("%s(): HEADER_SECTION: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", man_lval.sval.begin, man_lval.sval.end); 
	
	g_free(section);
	section = g_strdup(yytext);
	
	BEGIN(ST_HEADER_EAT);
	return HEADER_SECTION;
}

<ST_HEADER_EAT>\) {
	#ifdef DEBUG_HEADER
	g_message("%s(): ST_HEADER_EAT: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", man_lval.sval.begin, man_lval.sval.end); 
	return ')';
}

<ST_HEADER_EAT>[^\n \)\(]+ {
	#ifdef DEBUG_HEADER
	g_message("%s(): HEADER_WORD?: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", man_lval.sval.begin, man_lval.sval.end); 
	if (header_name != NULL && strcmp(yytext, header_name) == 0) {
		cui_window_apply_tag(window_for_lex, "command_name", man_lval.sval.begin, man_lval.sval.end); 
		BEGIN(ST_HEADER);
		return HEADER_NAME;
	}else {	
		return HEADER_WORD;
	}
}

<ST_HEADER_EAT>[\ \t] {
	#ifdef DEBUG_HEADER
	g_message("%s(): dropped(ST_HEADER_EAT): '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<ST_HEADER_EAT>[\n] {
	#ifdef DEBUG_HEADER
	g_message("%s(): (\\n)ST_HEADER_EAT: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	if (section != NULL && 
		(*section == '1' || *section == '8')) {
		BEGIN(ST_START);
	}else{
		cui_window_set_title(window_for_lex, _("Unparsed Manual Page"));
		BEGIN(ST_UNKNOWN_START);
	}
}

	/*
	 *
	 */
<ST_WORDS>.+ {
	#ifdef DEBUG
	g_message("%s(): state: ST_WORDS: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	//cui_window_apply_tag(window_for_lex, "Header", 
	//	man_lval.sval.begin, 
	//	man_lval.sval.end); 
	return MORE_WORD;
}

<ST_WORDS>^({CAPITAL}|[-\ ])+$ {
	#ifdef DEBUG
	g_message("%s(): state: ST_WORDS MORE_CHAPTER: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return MORE_CHAPTER;
}



	/*
	 * The error messages. If the man can't come up with a man page, 
	 * it writes things like this.
	 */
<ST_HEADER>"Cannot open"(.|\n)* {
	#ifdef DEBUG
	g_message("%s(): MAN_ERROR: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "error_message", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return MAN_ERROR;
}

<ST_ERROR>.* {
	/*
	 * In this state we eat up everything underlining the text
	 * for we encountered an error.
	 */
	#ifdef DEBUG_ERROR
	g_message("%s(): ST_ERROR...", __func__);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "underline-error", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return MAN_ERROR_LINE;
}





	/*
	 * Strong rules to recognize the known sections of the manual page.
	 */
<ST_SYNOPSIS,ST_SYNOPSIS_CC>^({CAPITAL}|[ ])+$ {
	/*
	 * We try to excape from ths state as soon as possible. This is because
	 * the syntax of the synopsis chapter is very strict.
	 */
	#ifdef DEBUG
	g_message("%s(): CHAPTER_NAME: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_DESCRIPTION);
	return CHAPTER;
}

<*>^{CAPITAL}({CAPITAL}|[-\ ])+$ {
	#ifdef DEBUG
	g_message("%s(): CHAPTER_NAME: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	if (strcmp(yytext, "NAME") == 0 || strcmp(yytext, "NÉV") == 0) {
		BEGIN(ST_NAME);
		return CHAPTER_NAME;
	}
	if (strcmp(yytext, "SYNOPSIS") == 0 || strcmp(yytext, "ÁTTEKINTÉS") == 0) {
		BEGIN(ST_SYNOPSIS);
		return CHAPTER_SYNOPSIS;
	}
	if (strcmp(yytext, "DESCRIPTION") == 0 || strcmp(yytext, "LEÍRÁS") == 0) {
		BEGIN(ST_DESCRIPTION);
		return CHAPTER_DESCRIPTION;
	}
	// We should process file list (at least).
	if (strcmp(yytext, "COPYRIGHT") == 0 || 
			strcmp(yytext, "AUTHOR") == 0 ||
			strcmp(yytext, "BUGS") == 0 ||
			strcmp(yytext, "FILES") == NULL) {
		BEGIN(ST_WORDS);
		return MORE_CHAPTER;
	}
	return CHAPTER;
}

<*>^"COPYRIGHT"$ {
	#ifdef DEBUG
	g_message("%s(): CHAPTER_NAME: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_WORDS);
	return CHAPTER;
}





	/*
	 * In this start condition virually anything can happen. This is a chapter
	 * in the man page which can hold textual information such as paragraphs,
	 * option lists followed by paragraphs etc.
	 */
<ST_DESCRIPTION>--[^ \n\t,=\[\]]+ {
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION LONGOPT: '%s'", __func__, yytext);
	g_message("%s(): => LONGOPT", __func__);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return LONGOPT;
}

<ST_DESCRIPTION>-[^ \n\t,-][^ \n\t,]* {
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION OPT: '%s'", __func__, yytext);
	g_message("%s(): => OPT", __func__);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return OPT;
}

<ST_DESCRIPTION>^[ ]+{LETTER} {
	/*
	 * Now we think this is a paragraph coming.
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION: '%s'", __func__, yytext);
	g_message("%s(): => ST_DESCRIPTION -> ST_PARAGRAPH_STARTING", __func__);
	#endif
	PUSH_BACK_ALL
	yy_push_state(ST_PARAGRAPH_STARTING);
}

<ST_DESCRIPTION>{LETTER}[^\n]{10,} {
	/*
	 * Now we think this is a paragraph coming. FIXME: if this sentence is starting
	 * with the following words, we should reject:
	 *   o or
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION: '%s'", __func__, yytext);
	g_message("%s(): => ST_DESCRIPTION -> ST_PARAGRAPH_STARTING", __func__);
	#endif
	PUSH_BACK_ALL
	yy_push_state(ST_PARAGRAPH_STARTING);
}

<ST_DESCRIPTION>[\(][^\n]{5,} {
	/*
	 * A paragraph can be started by a '('
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION: '%s'", __func__, yytext);
	g_message("%s(): => ST_DESCRIPTION -> ST_PARAGRAPH_STARTING", __func__);
	#endif
	PUSH_BACK_ALL
	yy_push_state(ST_PARAGRAPH_STARTING);
}

<ST_DESCRIPTION>","|"or" {
	/*
	 * The options can be separated with with various operators.
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION list separator: '%s'", __func__, yytext);
	g_message("%s(): => '%c')", __func__, yytext[0]);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "FgRed", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return ',';
}

<ST_DESCRIPTION>[=\[] {
	/*
	 * Extra characters at the beginning of the paragraph. They show
	 * how to give ann argument after an option.
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION extra char: '%s'", __func__, yytext);
	g_message("%s(): => '%c')", __func__, yytext[0]);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "FgRed", 
		man_lval.sval.begin, 
		man_lval.sval.end);
	yy_push_state(ST_SYN_ARG);
	return yytext[0];
}

<ST_DESCRIPTION>[\]] {
	/*
	 * Extra characters at the beginning of the paragraph. They show
	 * how to give ann argument after an option.
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION extra char: '%s'", __func__, yytext);
	g_message("%s(): => '%c')", __func__, yytext[0]);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "FgRed", 
		man_lval.sval.begin, 
		man_lval.sval.end);
	return yytext[0];
}

<ST_DESCRIPTION>[A-Za-z0-9][-A-Za-z0-9_]*/(\n|=) {
	/*
	 * An argument for the option at the beginning of the paragraph.
	 */
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION SYN_ARG: '%s'", __func__, yytext);
	g_message("%s(): => ", __func__, yytext[0]);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "nonterminal", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return SYN_ARG;
}



	/*
	 * In this state we accept expressions of a meta language describing
	 * the basics of the command line option syntax.
	 */
<ST_SYNOPSIS>^[\t ]* {
	#ifdef DEBUG_SYN
	g_message("%s(): Dropping(ST_SYNOPSIS): '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<ST_SYNOPSIS>[a-zA-Z0-9\/]+ {
	#ifdef DEBUG_SYN
	g_message("%s(): COMMAND(ST_SYNOPSIS): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "command_name", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_SYNOPSIS_CC);
	return COMMAND;
}

<ST_SYNOPSIS_CC>[\n]+ {
	#ifdef DEBUG_SYN
	g_message("%s(): Dropping(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	//BEGIN(ST_SYNOPSIS);
}

<ST_SYNOPSIS_CC>[\[\]\|\.=,-] {
	CuiLexElement lex;
	#ifdef DEBUG_SYN
	g_message("%s(): returning(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	lex.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		lex.text, &lex.begin, &lex.end);
	cui_window_apply_tag(window_for_lex, "FgRed", lex.begin, lex.end); 
	cui_lex_element_destroy(&lex, TRUE);
	return yytext[0];
}

<ST_SYNOPSIS_CC>("."{2,})|(\[\.\.\.\]) {
	#ifdef DEBUG_SYN
	g_message("%s(): SYN_DOTS(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "FgRed", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return SYN_DOTS;
}


<ST_SYNOPSIS_CC>"ARG"[0-9]* {
	#ifdef DEBUG_SYN
	g_message("%s(): SYN_ARG(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "nonterminal", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return SYN_ARG;
}


<ST_SYNOPSIS_CC>[^- \n\t\]\[\.=\|,][^ \n\t\]\[\.\|,]* {
	#ifdef DEBUG_SYN
	g_message("%s(): WORD(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	if (is_program_name(yytext)){
		cui_window_apply_tag(window_for_lex, "command_name", 
			man_lval.sval.begin, 
			man_lval.sval.end); 
		return COMMAND;
	}else{
		cui_window_apply_tag(window_for_lex, "nonterminal", 
			man_lval.sval.begin, 
			man_lval.sval.end); 
		return WORD;
	}
}

<ST_SYNOPSIS_CC>\{[^\}*]\} {
	/*
	 * FIXME: Actually this is a list of alternative terminal words.
	 */
	#ifdef DEBUG_SYN
	g_message("%s(): WORD(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "nonterminal", 
			man_lval.sval.begin, 
			man_lval.sval.end); 
	return WORD;
}

<ST_SYNOPSIS_CC>-[a-zA-Z0-9_][-a-zA-Z0-9_]* {
	#ifdef DEBUG_SYN
	g_message("%s(): SYN_OPTION(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return SYN_OPTION;
}

<ST_SYNOPSIS_CC>--[a-zA-Z0-9_][-a-zA-Z0-9_]* {
	#ifdef DEBUG_SYN
	g_message("%s(): SYN_LONGOPT(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return SYN_LONGOPT;
}

<ST_SYNOPSIS_CC>\([^\(]+\) {
	/*
	 * Comments in synopsis are text inside parenthesis.
	 */
	CuiLexElement lex;
	#ifdef DEBUG_SYN
	g_message("%s(): dropping(ST_SYNOPSIS_CC): '%s'", __func__, yytext);
	#endif
	lex.text = g_strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		lex.text, &lex.begin, &lex.end);
	cui_window_apply_tag(window_for_lex, "comment", lex.begin, lex.end); 
	cui_lex_element_destroy(&lex, TRUE);
}














	/*
	 * The states for discovering a paragraph. It is very complicated to 
	 * find a paragraph in a man page, since it is written for human reader.
	 * These states are using the flex stack to restore the state after a 
	 * complete paragraph found or the input doesn't contain any paragraph
	 * at all.
	 */
<ST_PARAGRAPH_STARTING>^{CHAPTER_TITLE}$ {
	/*
	 *
	 */
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): state=ST_PARAGRAPH_STARTING: '%s'", __func__, yytext);
	g_message("%s(): => this is a chapter title: yymore() and error", __func__);
	#endif
	yymore();
	BEGIN(ST_ERROR);
}

<ST_PARAGRAPH_STARTING>--[^ \n\t,]+ {
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): state=ST_PARAGRAPH_STARTING: '%s'", __func__, yytext);
	g_message("%s(): => this is a long option: yymore() and error ", __func__);
	#endif
	yymore();
	BEGIN(ST_ERROR);
}

<ST_PARAGRAPH_STARTING>[\n\t ]* {
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): state=ST_PARAGRAPH_STARTING dropping: '%s'", __func__, yytext);
	#endif
	paragraph_indent = g_utf8_strlen(yytext, -1);
	//g_message("%s(): => paragraph_indent = %d", __func__, paragraph_indent);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL); 
}

<ST_PARAGRAPH_STARTING>[^-] {
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): state=ST_PARAGRAPH_STARTING->ST_PARAGRAPH: '%s'", __func__, yytext);
	#endif
	yymore();
	BEGIN(ST_PARAGRAPH);
}

<ST_PARAGRAPH_STARTING>\([^\)]+\) {
	/*
	 * Maybe this is not a good idea, but I saw some '(ignored)' line (e.g. touch)
	 */
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): state=ST_PARAGRAPH_STARTING: '%s'", __func__, yytext);
	g_message("%s(): => complete paragraph", __func__, paragraph_indent);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	yy_pop_state();
	return PARAGRAPH;
}

<ST_PARAGRAPH>[^-\n][^\n]* {
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): 1 state=ST_PARAGRAPH yymore: '%s'", __func__, yytext);
	#endif
	yymore();
}

<ST_PARAGRAPH>\n/[^\n] {
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): 2 state=ST_PARAGRAPH yymore: '%s'", __func__, yytext);
	#endif
	yymore();
}

<ST_PARAGRAPH>\n\n {
	#ifdef DEBUG_PARAGRAPH
	g_message("%s(): 3 state=ST_PARAGRAPH: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	yy_pop_state();
	return PARAGRAPH;
}









<ST_NAME>[_a-zA-Z0-9][-_a-zA-Z0-9]* {
	#ifdef DEBUG
	g_message("%s(): COMMAND: '%s'", __func__, yytext);
	#endif
	
	add_program_name(yytext);
	
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "command_name", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return COMMAND;
}

<ST_NAME>"-"|"—" {
//	#ifdef DEBUG
	g_message("%s(): other char: '%s'", __func__, yytext);
//	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(ST_SHORT_DESCRIPTION);
	return '-';
}

<ST_SHORT_DESCRIPTION>{ALPHNUM}[^-\n]* {
//	#ifdef DEBUG
	g_message("%s(): state=ST_SHORT_DESCRIPTION (1): '%s'", __func__, yytext);
	g_message("%s(): => yymore()", __func__);
//	#endif
	yymore();
}

<ST_SHORT_DESCRIPTION>"-"|(-\n\ *) {
//	#ifdef DEBUG
	g_message("%s(): state=ST_SHORT_DESCRIPTION (2): '%s'", __func__, yytext);
	g_message("%s(): => yymore()", __func__);
//	#endif
	yymore();
}

<ST_SHORT_DESCRIPTION>"\n" {
	g_message("%s(): state=ST_SHORT_DESCRIPTION (3): '%s'", __func__, yytext);
	g_message("%s(): => DESCRIPTION", __func__);
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "comment", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	
	gchar *title = g_strdup_printf("Man Page: %s", yytext);
	cui_window_set_title(window_for_lex, title);
	
	BEGIN(ST_NAME);
	return DESCRIPTION;
}











<ST_START,ST_PARAGRAPH_STARTING>^{CAPITAL}[- A-Z,]*$ {
	#ifdef DEBUG
	g_message("%s(): CHAPTER: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return CHAPTER;
}

<ST_START,ST_PARAGRAPH_STARTING>^[ ]+[- A-Z,]+:?$ {
	#ifdef DEBUG
	g_message("%s(): SECTION: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return SECTION;
}


<ST_SENTENCE_STARTING>--[^ \n\t,]+ {
	#ifdef DEBUG
	g_message("%s(): LONGOPT: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return LONGOPT;
}

<ST_SENTENCE_STARTING>-[^ \n\t,]+ {
	#ifdef DEBUG
	g_message("%s(): OPT: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	return OPT;
}

<ST_PARAGRAPH_STARTING,ST_SENTENCE_STARTING>-[a-zA-Z0-9?]+ {
	#ifdef DEBUG
	g_message("%s(): OPT_AT_BEGIN: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "option", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	BEGIN(ST_SENTENCE_STARTING);
	return OPT_AT_BEGIN;
}

<ST_SENTENCE_STARTING>[\n ][\t ]* {
	#ifdef DEBUG
	g_message("%s(): state=ST_SENTENCE_STARTING dropping: '%s'",
			__func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL); 
	BEGIN(ST_SENTENCE);
}

<ST_SENTENCE_STARTING>,[ ]* {
	#ifdef DEBUG
	g_message("%s(): 1 state=ST_SENTENCE_STARTING other char: '%s'", 
		__func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	return yytext[0];
}

<ST_SENTENCE>[^\., ][^\n\.]* {
	#ifdef DEBUG
	g_message("%s(): 2 state=ST_SENTENCE yymore: '%s'", __func__, yytext);
	#endif
	yymore();
}

<ST_SENTENCE>\n {
	#ifdef DEBUG
	g_message("%s(): 3 state=ST_SENTENCE yymore: '%s'", __func__, yytext);
	#endif
	yymore();
}

<ST_SENTENCE>\. {
	#ifdef DEBUG
	g_message("%s(): state=ST_SENTENCE->ST_PARAGRAPH_REMAINDER yymore: '%s'", 
			__func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	BEGIN(ST_PARAGRAPH_REMAINDER);
	return SENTENCE;
}

<ST_SENTENCE>\n\n {
	#ifdef DEBUG
	g_message("%s(): SENTENCE(ST_SENTENCE): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	BEGIN(ST_PARAGRAPH_STARTING);
	return SENTENCE;
}

<ST_PARAGRAPH_REMAINDER>.*|\n {
	#ifdef DEBUG
	g_message("%s(): 1 state=ST_PARAGRAPH_REMAINDER yymore: '%s'", 
			__func__, yytext);
	#endif
	yymore();
}

<ST_PARAGRAPH_REMAINDER>\n\n {
	#ifdef DEBUG
	g_message("%s(): 2 state=ST_PARAGRAPH_REMAINDER: '%s'", 
			__func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	BEGIN(ST_PARAGRAPH_STARTING);
	return PARAGRAPH;
}



<*>{FS} {
	#ifdef DEBUG
	g_message("%s(): {FS}: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}


	/*
	 * We enter this state if the man section is unknown for us.
	 * 
	 */
<ST_UNKNOWN_START>^[ \t]*[-A-Z\ ]+$ {
	#ifdef DEBUG_UNKNOWN
	g_message("%s(): IGNORED_CHAPTER_NAME: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "Header", man_lval.sval.begin, man_lval.sval.end); 
	return IGNORED_CHAPTER_NAME;
}

<ST_UNKNOWN_START>[^\n]+ {
	#ifdef DEBUG_UNKNOWN
	g_message("%s(): IGNORED_LINE: '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, man_lval.sval.text, &man_lval.sval.begin, &man_lval.sval.end);
	return IGNORED_LINE;
}

<ST_UNKNOWN_START>[\n]+ {
	#ifdef DEBUG_UNKNOWN
	g_message("%s(): dropping(ST_UNKNOWN_START): '%s'", __func__, yytext);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

	
	/*
	 * In this state we positive we have to read at least one option
	 * argument. When we ready we have to return to the previous state.
	 */
<ST_SYN_ARG>({LETTER}|[_])+ {
	#ifdef DEBUG
	g_message("%s(): state=ST_SYN_ARG SYN_ARG: '%s'", __func__, yytext);
	g_message("%s(): => ", __func__);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "nonterminal", 
		man_lval.sval.begin, 
		man_lval.sval.end); 
	yy_pop_state();
	return SYN_ARG;
}

<ST_SYN_ARG>[=] {
	#ifdef DEBUG_DESCRIPTION
	g_message("%s(): state=ST_DESCRIPTION extra char: '%s'", __func__, yytext);
	g_message("%s(): => '%c')", __func__, yytext[0]);
	#endif
	man_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		man_lval.sval.text,
		&man_lval.sval.begin,
		&man_lval.sval.end);
	cui_window_apply_tag(window_for_lex, "FgRed", 
		man_lval.sval.begin, 
		man_lval.sval.end);
	return yytext[0];
}

















<*>\n {
	#ifdef DEBUG
	g_message("%s(): \\n", __func__);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<*>. {
	#ifdef DEBUG
	printf("<%s>", yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

%%
