%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: man_parser.y,v 1.1 2006/02/24 08:15:09 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: man_parser.y,v $
 * Revision 1.1  2006/02/24 08:15:09  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <libxml/xmlmemory.h>
#include <libxml/parser.h>
#include <libxml/xpathInternals.h>
#include <libxml/tree.h>

#include "cuiterm.h"
#include "bash_parser_c.h"
#include "cuilexelements.h"
#include "man_parser.h"
#include "terminal.h"
#include "background.h"
#include "cuiwindow.h"
#include "history.h"

static gchar *command_name = NULL;
extern CuiWindow *window_for_lex;
static Background *background = NULL;


%}

%debug
%parse-param {CuiWindow *window}
%union { 
	CuiLexElement sval;
	xmlNodePtr    node;
}

%token <sval> HEADER_NAME HEADER_SECTION HEADER_WORD
%token <sval> IGNORED_CHAPTER_NAME IGNORED_LINE

%token <sval> MAN_ERROR MAN_ERROR_LINE

%token <sval> MORE_CHAPTER MORE_WORD
%token <sval> CHAPTER SECTION
%token <sval> CHAPTER_NAME CHAPTER_DESCRIPTION CHAPTER_SYNOPSIS

%token <sval> SYN_ARG 
%token <sval> SYN_DOTS 
%token <sval> SYN_OPTION SYN_LONGOPT

%token <sval> COMMAND DESCRIPTION
%type  <sval> command_list
%token <sval> LONGOPT LONGOPT_AT_BEGIN
%token <sval> OPT OPT_AT_BEGIN
%token <sval> WORD LINE
%token <sval> PARAGRAPH SENTENCE

%type  <node> synopsis_full synopsis_element synopsis_expression synopsis_expression_list
%type  <node> option option_list

%nonassoc     '='
%nonassoc     '.'
%left         ','
%left         '|'

%%
chapters
   : header
   | MAN_ERROR         { window->error_code++; cui_lex_element_destroy(&$1, TRUE); }
   | MAN_ERROR_LINE
   | chapters MAN_ERROR_LINE
   | chapters chapter       
   | error             { window->error_code++; }
   ;

header
   : HEADER_NAME '(' HEADER_SECTION ')'           { 
		background = cui_widget_get_background(CUI_WIDGET(window));
	}
   | header HEADER_WORD                           
   | header HEADER_NAME '(' HEADER_SECTION ')'    
   ;

chapter
   : chapter_name
   | ignored_man_page
   | chapter_description
   | chapter_synopsis
   | more_chapter
   ;

more_chapter
   : MORE_CHAPTER
   | more_chapter MORE_CHAPTER
   | more_chapter MORE_WORD
   ;

ignored_man_page
   : IGNORED_CHAPTER_NAME
   | IGNORED_LINE
   | ignored_man_page IGNORED_CHAPTER_NAME
   | ignored_man_page IGNORED_LINE
   ;

chapter_name
   : CHAPTER_NAME command_list '-' DESCRIPTION {
		program_add_description($2.text, $4.text);
		cui_lex_element_destroy(&$1, TRUE); 
		cui_lex_element_destroy(&$2, TRUE); 
		cui_lex_element_destroy(&$4, TRUE); 
	}
   ;

command_list
   : COMMAND {
   		g_free(command_name);
		command_name = g_strdup($1.text);
	}
   | command_list COMMAND
   ;

chapter_description
   : CHAPTER_DESCRIPTION PARAGRAPH {
		/*
		 * The first paragraph is important.
		 */
		cui_window_apply_tag(window, "important", $2.begin, $2.end); 
		program_add_long_description(command_name, $2.text);

		cui_lex_element_destroy(&$1, TRUE);
		cui_lex_element_destroy(&$2, TRUE);
	}
   | CHAPTER_DESCRIPTION option_paragraph {
		cui_lex_element_destroy(&$1, TRUE);
	}
   | chapter_description option_paragraph
   | chapter_description PARAGRAPH {
		cui_lex_element_destroy(&$2, TRUE);
	}
   | chapter_description SECTION {
		/*This chapter can contain sections.*/
		cui_lex_element_destroy(&$2, TRUE);
	}
   | CHAPTER                      
   | chapter_description CHAPTER  
   ;


option_paragraph
   : option_list PARAGRAPH {
   		program_set_node_description($1, $2);
	}
   ;

option_list
   : option {
		$$ = $1;
	}
   | option_list ',' option {
		$$ = program_set_options_equal($1, $3);
	}
   | option_list option {
		$$ = program_set_options_equal($1, $2);
	}
   ;

option
  : OPT {
		$$ = program_add_option(command_name, $1.text);
		cui_dnd_source_new_with_string(window, background, TargetStringOption, $1.text, $1.begin, $1.end);
	}
  | LONGOPT {
		$$ = program_add_long_option(command_name, $1.text);
		cui_dnd_source_new_with_string(window, background, TargetStringLongOption, $1.text, $1.begin, $1.end);
	}
  | OPT SYN_ARG {
		$$ = program_add_option(command_name, $1.text);
		cui_dnd_source_new_with_string(window, background, TargetStringOption, $1.text, $1.begin, $1.end);
	}
  | LONGOPT SYN_ARG {
		$$ = program_add_long_option(command_name, $1.text);
		program_set_node_argument($$, $2);
		cui_dnd_source_new_with_string(window, background, TargetStringLongOption, $1.text, $1.begin, $1.end);
	}
  | LONGOPT '=' SYN_ARG {
		$$ = program_add_long_option(command_name, $1.text);
		program_set_node_argument($$, $3);
		cui_dnd_source_new_with_string(window, background, TargetStringLongOption, $1.text, $1.begin, $1.end);
	}
  | LONGOPT '[' '=' SYN_ARG ']' {
		$$ = program_add_long_option(command_name, $1.text);
		program_set_node_argument($$, $4);
		cui_dnd_source_new_with_string(window, background, TargetStringLongOption, $1.text, $1.begin, $1.end);
	}
  | LONGOPT SYN_ARG '=' SYN_ARG {
		$$ = program_add_long_option(command_name, $1.text);
		program_set_node_argument($$, $2);
		cui_dnd_source_new_with_string(window, background, TargetStringLongOption, $1.text, $1.begin, $1.end);
	}
  ;

chapter_synopsis
   : CHAPTER_SYNOPSIS         { cui_lex_element_destroy(&$1, TRUE); }
   | chapter_synopsis synopsis_full {
		gchar *command;
		command = parse_tree_get_first_argument(window->parse_tree);
		synopsis_remove_nodes_from_program(command);
		if ($2 != NULL)
			synopsis_add_node_to_program(command, $2);
	}
   ;

synopsis_full
   : COMMAND synopsis_expression_list {
   		#ifdef DEBUG
		g_message("----> <synopsis_full: COMMAND synopsis_expression_list>");
		#endif
		$$ = synopsis_create_node_with_child("argument-list", $2);
		cui_dnd_source_new_with_string(window, background, TargetStringProgram, $1.text, $1.begin, $1.end);
	}
   | COMMAND {
   		#ifdef DEBUG
   		g_message("----> <synopsis_full: COMMAND>");
		#endif
		$$ = NULL;
	}
   | synopsis_full COMMAND synopsis_expression_list {
   		xmlNodePtr node;
		#ifdef DEBUG
		g_message("----> <COMMAND synopsis_expression_list>");
		#endif
		node = synopsis_create_node_with_child("argument-list", $3);
		$$ = synopsis_catenate_nodes($1, node);
		//$$ = synopsis_create_alternative($1, $3);
	}
   ;

synopsis_expression_list
   : synopsis_expression {
		$$ = $1;
		#ifdef DEBUG
		g_message("----> <synopsis_expression_list: synopsis_expression>");
		#endif

	}
   | synopsis_expression_list synopsis_expression {
		$$ = synopsis_catenate_nodes($1, $2);
		#ifdef DEBUG
		g_message("----> <synopsis_expression_list: synopsis_expression_list synopsis_expression>"); 
		#endif
	}
   ;

synopsis_expression
   : synopsis_element { 
		$$ = $1; 
		#ifdef DEBUG
		g_message("----> <synopsis_expression: synopsis_element>"); 
		#endif
	}
   | synopsis_expression SYN_DOTS { 
		$$ = synopsis_set_node_iterate($1); 
		#ifdef DEBUG
		g_message("----> <synopsis_expression: synopsis_expression SYN_DOTS>"); 
		#endif
	}
   | '[' synopsis_expression ']' { 
		$$ = synopsis_set_node_optional($2); 
		#ifdef DEBUG
		g_message("----> <synopsis_expression: '[' synopsis_expression ']'>"); 
		#endif
	}
   | '[' synopsis_expression_list ']' { 
		$$ = synopsis_set_node_optional($2); 
		#ifdef DEBUG
		g_message("----> <synopsis_expression: '[' synopsis_expression_list ']'>"); 
		#endif
	}
   | '[' ',' synopsis_expression ']' { 
		$$ = $3; 
		#ifdef DEBUG
		g_message("----> <'[' synopsis_expression ']'>"); 
		#endif
	}
   | synopsis_expression '|' synopsis_expression     { 
		$$ = synopsis_create_operator_node('|', $1, $3); 
		#ifdef DEBUG
		g_message("----> <synopsis_expression '|' synopsis_expression>"); 
		#endif
	}
   | synopsis_expression ',' synopsis_expression     { 
		$$ = synopsis_create_operator_node(',', $1, $3); 
		#ifdef DEBUG
		g_message("----> <synopsis_expression ',' synopsis_expression>"); 
		#endif
	}
   ;

synopsis_element
   : WORD { 
		$$ = synopsis_create_node($1.text); 
		#ifdef DEBUG
		g_message("----> <synopsis_element: WORD>"); 
		#endif
	}
   | SYN_OPTION { 
		$$ = synopsis_create_node_with_option($1.text); 
		#ifdef DEBUG
		g_message("----> <SYN_OPTION>"); 
		#endif
	}
   | SYN_OPTION WORD {
		$$ = synopsis_create_node_with_option_argument($1.text, $2.text, " "); 
		#ifdef DEBUG
		g_message("----> <SYN_OPTION WORD>"); 
		#endif
	}
   | SYN_OPTION '=' WORD { 
		$$ = synopsis_create_node_with_option_argument($1.text, $3.text, "="); 
		#ifdef DEBUG
		g_message("----> <SYN_LONGOPT '=' WORD>"); 
		#endif
	}
   | SYN_LONGOPT { 
		$$ = synopsis_create_node_with_long_option($1.text); 
		#ifdef DEBUG
		g_message("----> <SYN_LONGOPT>"); 
		#endif
	}
   | SYN_LONGOPT WORD { 
		$$ = synopsis_create_node_with_long_option_argument($1.text, $2.text, " ");
		#ifdef DEBUG
		g_message("----> <SYN_LONGOPT WORD>"); 
		#endif
	}
   | SYN_LONGOPT '=' WORD { 
		$$ = synopsis_create_node_with_long_option_argument($1.text, $3.text, "=");
		#ifdef DEBUG
		g_message("----> <SYN_LONGOPT '=' WORD>"); 
		#endif
	}
   | '-' { 
		$$ = synopsis_create_node("minus"); 
		#ifdef DEBUG
		g_message("----> <'-'>"); 
		#endif
	}
   | SYN_ARG { 
		$$ = synopsis_create_node($1.text); 
		#ifdef DEBUG
		g_message("----> <SYN_ARG>"); 
		#endif
	}
   ;

%%

extern int man_lineno;

int man_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	error_message = g_strdup_printf("Error in File: %s in lineno %d",
			c,
			man_lineno);
	g_warning(error_message);
	//g_error(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	return(1);
}

int man_wrap(void)
{
	//g_message("%s(): returning 1", __func__);
	return 1;
}
