/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: popup_menus.c,v 1.1 2006/02/24 17:16:17 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: popup_menus.c,v $
 * Revision 1.1  2006/02/24 17:16:17  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>

#include <fnmatch.h>
#include "callbacks.h"
#include "support.h"
#include "interface.h"
#include "terminal.h"
#include "cuiterm.h"
#include "cuiwindow.h"
#include "widgets.h"
#include "filename.h"

/*
 * Clipdoard operations. I decided to handle all of the clipboard operations
 * here, not in the shell which would be much more nice and cool. To do the work
 * here is a lot more simple, so this how we handle the problem. However, this
 * is a little bit too simple.
 */
#define CLIP_NOTHING 0
#define CLIP_COPY    1
#define CLIP_CUT     2

static gint clip_last_operation = CLIP_NOTHING;
static CuiFilename *clip_last_filename = NULL;

/**
 * drop_file_command:
 * @menuitem: the activated menuitem
 * @user_data: user data from the signal connection
 * @command: the opcode of the command
 * @menuname: the name of the popup menu as named in the Glade
 * @refresh: if TRUE the function will execute the CuiWidget commands 
 *
 * A simple utility function to simplify the execution of menu commands.
 */
static void
drop_file_command(GtkMenuItem *menuitem,
		gpointer user_data,
		CuiCommandOpcode command,
		gchar *menuname,
		gboolean refresh)
{
	GtkWidget *menu;
	CuiWidget *cui_widget;
	Terminal *terminal;
	CuiGenerator *command_generator;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), menuname);
	command_generator = g_object_get_data(G_OBJECT(menu), "command_generator");
	cui_widget = g_object_get_data(G_OBJECT(menu), "cui_widget");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) && CUI_IS_WIDGET(cui_widget));

	/*
	 * First we emit the command associated with the menu item. We have this
	 * command from the menu item callback function.
	 */
	if (CUI_IS_FILE_MENU(cui_widget))
		cui_generator_emit_file_command(command_generator,
				command,
				CUI_FILE_MENU(cui_widget)->file);
	if (CUI_IS_MODE_MENU(cui_widget))
		cui_generator_emit_file_command(command_generator,
				command,
				CUI_MODE_MENU(cui_widget)->file);
	
	/*
	 * Then we emit the commands associated with the CuiWidget of the popup
	 * menu.
	 */
	if (refresh)
		cui_generator_emit_commands(command_generator, 
				cui_widget->commands);
}

static inline CuiFilename *
file_menu_item_get_file(GtkMenuItem *menuitem)
{
	GtkWidget *menu;
	CuiWidget *cui_file_menu;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "file_popup_menu");
	cui_file_menu = g_object_get_data(G_OBJECT(menu), "cui_widget");
	g_return_val_if_fail(CUI_IS_WIDGET(cui_file_menu), NULL);
	return cui_file_menu_get_filename(CUI_FILE_MENU(cui_file_menu));
}

/***************************************************************************
 * Callbacks for the application popup menu.                               *
 *                                                                         *
 **************************************************************************/
static void
application_menu_drop_command(GtkMenuItem *menuitem, const gchar *command)
{
	GtkWidget *menu;
	Terminal  *terminal;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "application_popup_menu");
	terminal = (Terminal *) g_object_get_data(G_OBJECT(menu), "terminal");
	terminal_send_command(terminal, (gchar *)command, TRUE);
}
	

void
on_application_about_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_about1_activate(menuitem, user_data);
}


void
on_application_new_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_application_new_directory_activate  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;

	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	background_store_command(t->background, CommandNewDirectory);
	application_menu_drop_command(menuitem, "mkdir NewDirectory");
}


void
on_application_new_file_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;

	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	background_store_command(t->background, CommandNewFile);
	application_menu_drop_command(menuitem, "touch NewFile");
}


void
on_files_and_directories_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *command;
	
	command = g_strdup_printf("ls%s", bg_conf_get_default_ls_options(TRUE));
	application_menu_drop_command(menuitem, command);
	g_free(command);
}

void
on_mounted_devices1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "df -h");
}

void
on_logged_on_users_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "who -H");
}

void
on_processes_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "ps aux");
}



void
on_tree_file_view_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "tree");
}


void
on_tree_processes_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "pstree");
}

void
on_application_cut_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}

void
on_application_copy_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}

void
on_application_paste_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	Terminal         *terminal;
	CuiGenerator     *generator;
	GtkWidget        *menu;
	gchar            *wd;
	CuiCommandOpcode  opcode;
	gchar            *command;
	
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "application_popup_menu");
	terminal = g_object_get_data(G_OBJECT(menu), "terminal");
	generator = g_object_get_data(G_OBJECT(menu), "command_generator");
	
	g_assert(terminal != NULL);
	g_assert(generator != NULL);

	wd = (gchar *)terminal_get_working_directory(terminal);
	if (clip_last_operation == CLIP_CUT)
		opcode = FileMove;
	else 
		opcode = FileCopy;

	cui_generator_emit_file_command_with_arg(
			generator, opcode, clip_last_filename, wd);	
	/*
	 * If the last operation is the cut, we set it to nothing since the file
	 * is no more available to paste from the old place.
	 */
	if (clip_last_operation == CLIP_CUT)
		clip_last_operation = CLIP_NOTHING;

	/*
	 * Refreshing the file list.
	 */
	command = g_strdup_printf("ls%s", bg_conf_get_default_ls_options(TRUE));
	application_menu_drop_command(menuitem, command);
	g_free(command);
}
	
void
on_logout_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "logout");
}

void
on_shut_down_computer1_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "poweroff");
}

void
on_reboot_computer_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	application_menu_drop_command(menuitem, "reboot");
}

void
on_app_popup_quit_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	on_quit1_activate(menuitem, user_data);
}

/***************************************************************************
 * Callbacks for the window menu (CuiWindow, embedded windows).            *
 *                                                                         *
 *                                                                         *
 *                                                                         *
 **************************************************************************/
static void
window_menu_drop_command(GtkMenuItem *menuitem, 
		const gchar *command,
		gboolean refresh)
{
	GtkWidget *menu;
	CuiWidget *cui_widget;
	Terminal  *terminal;
	CuiGenerator *command_generator;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "window_menu");
	terminal = (Terminal *) g_object_get_data(G_OBJECT(menu), "terminal");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	cui_widget = g_object_get_data(G_OBJECT(menu), "cui_widget");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			 CUI_IS_WIDGET(cui_widget));
	terminal_send_command(terminal, (gchar *)command, TRUE);
	/*
	 * If we need to refresh the window we emit the associated commands.
	 */
	if (refresh)
		cui_generator_emit_commands(command_generator, 
				cui_widget->commands);
}


void
on_window_menu_realize                 (GtkWidget       *widget,
                                        gpointer         user_data)
{
	GtkWidget *menuitem;
	menuitem = lookup_widget(widget, "window_cut");
	gtk_widget_set_sensitive(menuitem, FALSE);
	/* menuitem = lookup_widget(widget, "window_copy");
	gtk_widget_set_sensitive(menuitem, FALSE);*/
	menuitem = lookup_widget(widget, "window_paste");
	gtk_widget_set_sensitive(menuitem, clipboard_get_paste());
}

static void
do_save_window_as_png_picture(CuiWindow *window, 
		const gchar *filename)
{
	Background *bg = cui_widget_get_background(CUI_WIDGET(window));
	GdkRectangle *area = cui_widget_get_area_window_coords(CUI_WIDGET(window));
	GdkPixbuf *pixbuf;
	
	pixbuf = gdk_pixbuf_get_from_drawable(
			NULL,               // GdkPixbuf *dest
			bg->window, // GdkDrawable *src
			NULL,               // GdkColormap *colormap
			area->x,    // src x
			area->y,    // src y
			0,          // dest x
			0,          // dest y
			area->width + 5, 
			area->height + 5);
	
	gdk_pixbuf_save (pixbuf, filename, "png", NULL, NULL);

	g_free(area);
	g_object_unref(G_OBJECT(pixbuf));
}

void
on_save_window_as_png_picture_activate (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *menu;
	GtkTextView *dialog;
	CuiWindow *window;
	gchar *filename;
	/*
	 * Getting data.
	 */
	menu = lookup_widget(GTK_WIDGET(menuitem), "window_menu");
	window = g_object_get_data(G_OBJECT(menu), "window");
	
	dialog = gtk_file_chooser_dialog_new("Save window as PNG image...", 
			NULL, 
			GTK_FILE_CHOOSER_ACTION_SAVE, 
			GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, 
			GTK_STOCK_OK, GTK_RESPONSE_OK, NULL);
  
	if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_OK){
  		gtk_widget_destroy (dialog);
		return;
	}
  
	filename = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
	gtk_widget_destroy (dialog);
  
	if (filename == NULL){
		return;
	}
	
	do_save_window_as_png_picture(window, filename);
	g_free (filename);
}


void
on_save_window_as_unicode_text_activate
                                        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}

void 
on_detach_window_activate(GtkMenuItem *menuitem, 
		gpointer user_data)
{
	GtkWidget *menu;
	CuiWindow *window;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "window_menu");
	window = g_object_get_data(G_OBJECT(menu), "window");
	g_return_if_fail(window != NULL);
	/*
	 * Detaching the window.
	 */
	cui_window_detach(window);
}


void
on_refresh_window1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *menu;
	CuiWidget *cui_widget;
	CuiGenerator *command_generator;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "window_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	cui_widget = g_object_get_data(G_OBJECT(menu), "cui_widget");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			 CUI_IS_WIDGET(cui_widget));
	/*
	 * If we need to refresh the window we emit the associated commands.
	 */
	cui_generator_emit_commands(command_generator, cui_widget->commands);
}


void
on_window_close_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}


void
on_window_new_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_window_new_directory_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;

	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	background_store_command(t->background, CommandNewDirectory);
	window_menu_drop_command(menuitem, "mkdir NewDirectory", TRUE);
}


void
on_window_new_file_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *widget;
	Terminal *t;

	widget = lookup_widget(GTK_WIDGET(menuitem), "main_widget");
	t = g_object_get_data(G_OBJECT(widget), "terminal");
	background_store_command(t->background, CommandNewFile);
	window_menu_drop_command(menuitem, "touch NewFile", TRUE);
}


void
on_files_and_directories1_activate     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	gchar *command;
	
	command = g_strdup_printf("ls%s", bg_conf_get_default_ls_options(TRUE));
	window_menu_drop_command(menuitem, command, FALSE);
	g_free(command);
}


void
on_logged_on_users1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "who -H", FALSE);
}


void
on_processes1_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "ps aux", FALSE);
}


void
on_tree_file_view1_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "tree", FALSE);
}


void
on_tree_process1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "pstree", FALSE);
}


void
on_exit_propram2_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "exit", FALSE);
}


void
on_logout1_activate                    (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "logout", FALSE);
}


void
on_shut_down_computer2_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "poweroff", FALSE);
}


void
on_reboot_computer1_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "reboot", FALSE);
}


void
on_mounted_devices2_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "df -h", FALSE);
}



void
on_window_cut_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	g_warning("%s(): Not implemented.", __func__);
}


void
on_window_copy_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkClipboard *clipboard;
	GdkAtom atom;
	GtkWidget *menu;
	CuiWindow *window;
	GtkWidget *text_view;
	GtkTextBuffer *text_buffer;
	GtkTextIter begin, end;
	gchar *text;
		
	menu = lookup_widget(GTK_WIDGET(menuitem), "window_menu");
	window = g_object_get_data(G_OBJECT(menu), "window");
	g_return_if_fail(window != NULL);
	/*
	 * Loading the original text.
	 */
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, 
			&begin, window->begin);
	gtk_text_buffer_get_iter_at_mark(window->text_buffer, 
			&end, window->end);
	text = gtk_text_buffer_get_text(window->text_buffer, 
			&begin, &end, TRUE);
	
	atom = gdk_atom_intern("CLIPBOARD", FALSE);
	clipboard = gtk_clipboard_get(atom);
	gtk_clipboard_set_text(clipboard, text, -1);
}


void
on_window_paste_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *menu;
	CuiGenerator *command_generator;
	CuiWindow *window;
	
	g_assert(clip_last_filename != NULL);
	g_message("%s(): filename: '%s'", __func__, 
			clip_last_filename->filename);

	menu = lookup_widget(GTK_WIDGET(menuitem), "window_menu");
	command_generator = g_object_get_data(G_OBJECT(menu),
			"command_generator");
	window = g_object_get_data(G_OBJECT(menu), "window");
	g_return_if_fail(window != NULL);

	if (clip_last_operation == CLIP_COPY)
		cui_generator_emit_file_command_with_arg(
				command_generator,
				FileCopy,
				clip_last_filename,
				window->wd->str);
	else if (clip_last_operation == CLIP_CUT)
		cui_generator_emit_file_command_with_arg(
				command_generator,
				FileMove,
				clip_last_filename,
				window->wd->str);
	else
		g_warning("%s(): Not copy and not cut?", __func__);
		
	cui_generator_emit_window_command(command_generator, 
			WindowRefresh,
			window);
}


void
on_window_quit_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	window_menu_drop_command(menuitem, "exit", FALSE);
}

/***************************************************************************
 * Callback functions for Device popup menus.                              *
 *                                                                         *
 *                                                                         *
 *                                                                         *
 **************************************************************************/
void
on_show_mounted_devices1_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mount_device_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_unmount_device_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileOpenForEdit, "block_device_menu", FALSE);
}


void
on_mount_floppy_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_umount_floppy_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mount_cdrom1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_umount_cdrom1_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_mount_usb_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;

}


void
on_umount_usb_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}



/***************************************************************************
 * Callback functions for File popup menus.                                *
 *                                                                         *
 *                                                                         *
 *                                                                         *
 **************************************************************************/
void
on_file_open_activate(GtkMenuItem *menuitem,
                      gpointer    user_data)
{
	UNIMPLEMENTED;
	// FIXME: we have to create a new class for this blcok_device_menu
	//drop_file_command(menuitem, user_data, 
	//		FileOpenForEdit, "file_popup_menu",
	//		FALSE);
}


void
on_file_delete_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	
	drop_file_command(menuitem, user_data, FileDelete, "file_popup_menu",
			TRUE);
}


void
on_file_move_to_trash_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileMoveToTrash, 
			"file_popup_menu", TRUE);
}


void
on_file_edit_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}

/**
 * clipboard_get_paste:
 * @returns: TRUE if we can use the 'paste' menuitem
 *
 * We can use this function to find out if the "Paste" menu item can be set to
 * sensitive since we have had used the "Copy" or the "Cut" menu item.
 */
gboolean
clipboard_get_paste(void)
{
	return clip_last_operation != CLIP_NOTHING;
}

void
on_file_cut_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	clip_last_operation = CLIP_CUT;
	clip_last_filename = file_menu_item_get_file(menuitem);
}


void
on_file_copy_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	clip_last_operation = CLIP_COPY;
	clip_last_filename = file_menu_item_get_file(menuitem);
}


void
on_file_paste_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	Terminal     *terminal;
	CuiFileMenu  *filemenu;
	CuiGenerator *generator;
	GtkWidget    *menu;
	gchar        *wd;
	gchar        *newname;
	CuiCommandOpcode opcode;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "file_popup_menu");
	filemenu = g_object_get_data(G_OBJECT(menu), "cui_widget");
	terminal = g_object_get_data(G_OBJECT(menu), "terminal");
	generator = g_object_get_data(G_OBJECT(menu), "command_generator");
	
	g_assert(filemenu != NULL && terminal != NULL && generator != NULL);

	wd = (gchar *)terminal_get_working_directory(terminal);
	newname = cui_filename_get_canonical_filename(filemenu->file, wd);
	if (clip_last_operation == CLIP_CUT)
		opcode = FileMove;
	else 
		opcode = FileCopy;
	
	cui_generator_emit_file_command_with_arg(
			generator, opcode, clip_last_filename, newname);
	
	g_free(newname);

	/*
	 * If the last operation is the cut, we set it to nothing since the file
	 * is no more available to paste from the old place.
	 */
	if (clip_last_operation == CLIP_CUT)
		clip_last_operation = CLIP_NOTHING;
}


void
on_file_create_archive_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileNewTarArchive, 
			"file_popup_menu",
			TRUE);
}


void
on_file_create_zip_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileNewTarGzArchive, 
			"file_popup_menu", TRUE);
}


void
on_file_create_bzip_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileNewTarBzArchive, 
			"file_popup_menu", TRUE);
}


void 
on_file_new_activate(GtkMenuItem * menuitem, gpointer user_data)
{
	drop_file_command(menuitem, user_data, FileNewRegularFile, 
			"file_popup_menu", TRUE);
}


void
on_file_new_directory_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *menu;
	Terminal *t;

	menu = lookup_widget(GTK_WIDGET(menuitem), "file_popup_menu");
	t = g_object_get_data(G_OBJECT(menu), "terminal");
	background_store_command(t->background, CommandNewDirectory);
	drop_file_command(menuitem, user_data, FileNewDirectory, 
			"file_popup_menu", TRUE);
}


void
on_file_new_file_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget *menu;
	Terminal *t;

	menu = lookup_widget(GTK_WIDGET(menuitem), "file_popup_menu");
	t = g_object_get_data(G_OBJECT(menu), "terminal");
	background_store_command(t->background, CommandNewFile);
	drop_file_command(menuitem, user_data, FileNewRegularFile, 
			"file_popup_menu", TRUE);
}


void
on_file_gvim_text_editor_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileOpenWithVim, 
			"file_popup_menu", FALSE);
}


void
on_emacs_text_editor_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileOpenWithEmacs, 
			"file_popup_menu", FALSE);
}


void
on_file_uncompress_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileUncompress, 
			"file_popup_menu", TRUE);
}


void
on_file_properties_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileShowProperties, 
			"file_popup_menu", FALSE);
}



void
on_file_view_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileOpenForView, 
			"file_popup_menu", FALSE);
}


void
on_process_file_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileProcess, 
			"file_popup_menu", TRUE);
}


/****************************************************************************
 * File mode menu callbacks.                                                *
 *                                                                          *
 *                                                                          *
 *                                                                          *
 ****************************************************************************/
void
on_mode_file_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileSetModeToFile, 
			"mode_popup_menu", TRUE);
}


void
on_mode_dir_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileSetModeToDir, 
			"mode_popup_menu", TRUE);
}


void
on_mode_program_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileSetModeToProgram, 
			"mode_popup_menu", TRUE);
}


void
on_grant_read_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantReadUser, 
			"mode_popup_menu", TRUE);
}


void
on_revoke_read_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeReadUser, 
			"mode_popup_menu", TRUE);
}


void
on_grant_write_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantWriteUser, 
			"mode_popup_menu", TRUE);
}


void
on_revoke_write_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeWriteUser, 
			"mode_popup_menu", TRUE);
}


void
on_grant_execute_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantExecuteUser, 
			"mode_popup_menu", TRUE);
}


void
on_revoke_execute_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeExecuteUser, 
			"mode_popup_menu", TRUE);
}


void
on_mode_group_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}


void
on_group_grant_read_activate           (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantReadGroup, 
			"mode_popup_menu", TRUE);
}


void
on_group_revoke_read_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeReadGroup, 
			"mode_popup_menu", TRUE);
}


void
on_group_revoke_write_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeWriteGroup, 
			"mode_popup_menu", TRUE);
}


void
on_group_grant_execute_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantExecuteGroup, 
			"mode_popup_menu", TRUE);
}


void
on_group_revoke_execute_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeExecuteGroup, 
			"mode_popup_menu", TRUE);
}


void
on_mode_others_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
}


void
on_others_grant_read_activate          (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantReadOthers, 
			"mode_popup_menu", TRUE);
}


void
on_others_revoke_read_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeReadOthers, 
			"mode_popup_menu", TRUE);
}


void
on_others_grant_write_activate         (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantWriteOthers, 
			"mode_popup_menu", TRUE);
}


void
on_others_revoke_write_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeWriteOthers, 
			"mode_popup_menu", TRUE);
}


void
on_others_grant_execute_activate       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileGrantExecuteOthers, 
			"mode_popup_menu", TRUE);
}


void
on_others_revoke_execute_activate      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	drop_file_command(menuitem, user_data, FileRevokeExecuteOthers, 
			"mode_popup_menu", TRUE);
}


/***************************************************************************
 * Callback functions for User popup menus.                                *
 *                                                                         *
 *                                                                         *
 *                                                                         *
 **************************************************************************/
void
on_user_home_directory_activate        (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("cd ~%s", username);
	cui_generator_send_shell_command(command_generator, command);
	/* FIXME: this neglects default ls options */
	cui_generator_send_shell_command(command_generator, "ls -lh");
	
	g_free(command);
}


void
on_user_delete_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("userdel %s", username);
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);
}


void
on_user_add_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	UNIMPLEMENTED;
}


void
on_user_properties_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("finger %s", username);
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);
}


void
on_loggen_in_users_activate            (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator));
	command = g_strdup_printf("who -H");
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);

}


void
on_user_processes_activate             (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("ps aux | grep \"^%s \"", username);
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);
}


void
on_user_quotas_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("quota %s", username);
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);
}


void
on_user_lock_activate                  (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("passwd -l %s", username);
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);
}

void
on_user_unlock_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget 	*menu;
	Terminal 	*terminal;
	CuiGenerator 	*command_generator;
	gchar 		*username;
	gchar 		*command;
	
	menu = lookup_widget(GTK_WIDGET(menuitem), "user_popup_menu");
	command_generator = g_object_get_data(G_OBJECT(menu), 
			"command_generator");
	username = g_object_get_data(G_OBJECT(menu),
			"username");
	g_return_if_fail(CUI_IS_GENERATOR(command_generator) &&
			username != NULL);
	command = g_strdup_printf("passwd -u %s", username);
	cui_generator_send_shell_command(command_generator, command);
	g_free(command);
}

