%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: ps_lexer.l,v 1.1 2006/02/24 09:01:07 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: ps_lexer.l,v $
 * Revision 1.1  2006/02/24 09:01:07  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include <stdlib.h>
#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "ps_parser.h"

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option yylineno
%s STARTSTATE TIMESTATE COMMANDSTATE

%%
"USER".*"PID".* {
	#ifdef DEBUG
	g_message("%s: HEADER: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "Header",
		ps_lval.sval.begin, 
		ps_lval.sval.end); 
	return HEADER;
}

<*>^[^ \t]+ {
	#ifdef DEBUG
	g_message("%s: USERNAME: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "username",
		ps_lval.sval.begin, 
		ps_lval.sval.end); 
	BEGIN(STARTSTATE);
	return USERNAME;
}

[0-9]+ {
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	return NUMBER;
}

[0-9]+\.[0-9]+ {
	#ifdef DEBUG
	g_message("%s: FLOAT: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	ps_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	return FLOAT;
}

"?" {
	#ifdef DEBUG
	g_message("%s: Special character: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	return yytext[0];
}

("tty"[0-9]+)|("pts"\/[0-9]+) {
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	ps_lval.sval.double_val = g_ascii_strtod(yytext, NULL);
	ps_lval.sval.double_val *= 1024 * 1024;
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	return TERMINAL;
}

[A-Z][A-Za-z+<]* {
	#ifdef DEBUG
	g_message("%s: STAT: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	return STAT;
}

[A-ZÖÜÓŐÚÁŰÉ][a-zöüóőúéáű]+[0-9]+ {
	#ifdef DEBUG
	g_message("%s: DATE: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	BEGIN(TIMESTATE);
	return DATE;
}

<STARTSTATE>[0-9][0-9]?:[0-9][0-9]? {
	#ifdef DEBUG
	g_message("%s: TIME(1): '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	BEGIN(TIMESTATE);
	return TIME;
}

<TIMESTATE>[0-9][0-9]?:[0-9][0-9]? {
	#ifdef DEBUG
	g_message("%s: TIME(2): '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	BEGIN(COMMANDSTATE);
	return TIME;
}


<COMMANDSTATE>"["[^\]]*"]" {
	#ifdef DEBUG
	g_message("%s: KERNELPR: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "kernel_process",
		ps_lval.sval.begin, 
		ps_lval.sval.end); 
	BEGIN(STARTSTATE);
	return KERNELPR;
}

<COMMANDSTATE>"init"[ \t]*\[[0-9S]\]\ * {
	#ifdef DEBUG
	g_message("%s: INITPR: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "kernel_process",
		ps_lval.sval.begin, 
		ps_lval.sval.end); 
	BEGIN(STARTSTATE);
	return INITPR;
}

<COMMANDSTATE>[^ ][^\n]* {
	#ifdef DEBUG
	g_message("%s: PROGRAM: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	BEGIN(STARTSTATE);
	return PROGRAM;
}

[ \t\n]+ { 
	/* Dropping it away */
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}
	

.	{
	#ifdef DEBUG
	g_message("%s: OTHER_CHAR: '%s'", __func__, yytext);
	#endif
	ps_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		ps_lval.sval.text,
		&ps_lval.sval.begin,
		&ps_lval.sval.end);
	return yytext[0];
}

%%

