%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: simple_lexer.l,v 1.1 2006/02/24 09:04:26 pipas Exp $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: simple_lexer.l,v $
 * Revision 1.1  2006/02/24 09:04:26  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "ls_parser_c.h"
#include "simple_parser.h"

// #define DEBUG

#undef YY_INPUT
extern CuiWindow *window_for_lex;
extern Terminal  *terminal_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option stack
%option yylineno

%x WHOAMI PWD LOCATE CP
%x ST_QUOTEDFILE_START ST_QUOTEDFILE
%x EAT_ALL
%%
  	if (window_for_lex->parsed_chars == 0) {
		gchar *program;

		program = parse_tree_get_producer_program(terminal_for_lex->parse_tree);
		g_assert(program != NULL);

		if (strcmp(program, "whoami") == 0) {
			cui_window_set_title(window_for_lex, _("Effective UserID"));
			BEGIN(WHOAMI);
		} else if (strcmp(program, "pwd") == 0 || strcmp(program, "popd") == 0) {
			cui_window_set_title(window_for_lex, _("Working Directory"));
			BEGIN(PWD);
		} else if (strcmp(program, "locate") == 0) {
			cui_window_set_title(window_for_lex, _("File Search"));
			BEGIN(LOCATE);
		} else if (strcmp(program, "cp") == 0) {
			cui_window_set_title(window_for_lex, _("Copy Files"));
			BEGIN(CP);
		}
	}

	/*
	 * Error messages.
	 */
<*>"bash:"(.|\n)* {
	#ifdef DEBUG
	g_message("%s: SIMPLE_ERROR: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	cui_window_apply_tag(window_for_lex, "error_message", 
		simple_lval.sval.begin, 
		simple_lval.sval.end); 
	return SIMPLE_ERROR;
}


	/*
	 * The cp program. It can produce error messages and verbose 
	 * progress messages when started with the -v option.
	 */
<CP>"cp: ".*\n {
	#ifdef DEBUG
	g_message("%s: CP_ERROR: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	cui_window_apply_tag(window_for_lex, "error_message", 
		simple_lval.sval.begin, 
		simple_lval.sval.end); 
	return CP_ERROR;
}

<CP>"`"|"'" {
	#ifdef DEBUG
	g_message("%s: QUOTE: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	//cui_window_apply_tag(window_for_lex, "FgRed", 
	//	simple_lval.sval.begin, 
	//	simple_lval.sval.end); 
	yy_push_state(ST_QUOTEDFILE_START);
	return QUOTE;
}

<CP>"->" {
	#ifdef DEBUG
	g_message("%s: ARROW: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	//cui_window_apply_tag(window_for_lex, "FgRed", 
	//	simple_lval.sval.begin, 
	//	simple_lval.sval.end); 
	return ARROW;
}




	/*
	 * This state will handle quoted files, filenames and full path
	 * filenames between single quotes. The state will use a pop
	 * operation to restore the state when the quoted file name
	 * ends (see CP state for example). 
	 */
<ST_QUOTEDFILE_START>"/"|"."|"~" {
	#ifdef DEBUG
	g_message("%s: ROOT_DIR: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	BEGIN(ST_QUOTEDFILE);
	return ROOT_DIR;
}

<ST_QUOTEDFILE>[^/\n\r`']+/\/ {
	#ifdef DEBUG
	g_message("%s: DIRNAME: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	return DIRNAME;
}

<ST_QUOTEDFILE>[^/\n\r`']+ {
	#ifdef DEBUG
	g_message("%s: FILENAME: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	return FILENAME;
}

<ST_QUOTEDFILE>"/" {
	#ifdef DEBUG
	g_message("%s: DIRSEP: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
		simple_lval.sval.text = g_strdup(yytext);
	return DIRSEP;
}

<ST_QUOTEDFILE>"`"|"'" {
	#ifdef DEBUG
	g_message("%s: QUOTE: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	//cui_window_apply_tag(window_for_lex, "FgRed", 
	//	simple_lval.sval.begin, 
	//	simple_lval.sval.end); 
	yy_pop_state();
	return QUOTE;
}


<WHOAMI>[^\n\r]+ {
	#ifdef DEBUG
	g_message("%s: USERNAME: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
		simple_lval.sval.text = g_strdup(yytext);
	BEGIN(EAT_ALL);
	return USERNAME;
}

<PWD,LOCATE>^[/~] {
	#ifdef DEBUG
	g_message("%s: ROOT_DIR: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	return ROOT_DIR;
}

<PWD>[^/\n\r]+ {
	#ifdef DEBUG
	g_message("%s: DIRNAME: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
		simple_lval.sval.text = g_strdup(yytext);
	return DIRNAME;
}

<PWD,LOCATE>"/" {
	#ifdef DEBUG
	g_message("%s: DIRSEP: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
		simple_lval.sval.text = g_strdup(yytext);
	return DIRSEP;
}

<PWD>[\n\r] {
	#ifdef DEBUG
	g_message("%s: starting eater: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(EAT_ALL);
}

<LOCATE>[^/\n\r]+/\/ {
	#ifdef DEBUG
	g_message("%s: DIRNAME(LOCATE): '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
		simple_lval.sval.text = g_strdup(yytext);
	return DIRNAME;
}

<LOCATE>[^/\n\r]+ {
	#ifdef DEBUG
	g_message("%s: FILENAME(LOCATE): '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, 
		yytext,
		&simple_lval.sval.begin,
		&simple_lval.sval.end);
	simple_lval.sval.text = g_strdup(yytext);
	return FILENAME;
}


	/*
	 * The characters we drop.
	 */
<CP>" "+ {
	#ifdef DEBUG
	g_message("%s: dropping: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<LOCATE,CP>[\n\r]+ {
	#ifdef DEBUG
	g_message("%s: dropping: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

<EAT_ALL>(.|\n)+ {
	#ifdef DEBUG
	g_message("%s: eating: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}


%%
