%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: stat_lexer.l,v 1.1 2006/02/24 09:00:21 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: stat_lexer.l,v $
 * Revision 1.1  2006/02/24 09:00:21  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
                             
#include <string.h>
#include <stdlib.h>
#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "stat_parser.h"

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}


%x STRING
DATE [0-9]{4,4}-[0-9]{1,2}-[0-9]{1,2}
TIME [0-9]{1,2}:[0-9]{1,2}:[0-9]{1,2}.[0-9]{9,9}
TIME_ZONE [-+]{1,1}[0-9]{4,4}

%option yylineno

%%
	cui_window_set_title(window_for_lex, _("File State"));


^"  File:" {
	/* Just for skipping to the next token */
	#ifdef DEBUG
	g_message("%s: FILE_LINE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return FILE_LINE;
}

^"  Size:" {
	#ifdef DEBUG
	g_message("%s: SIZE_LINE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return SIZE_LINE;
}

^"Device:" {
	#ifdef DEBUG
	g_message("%s: DEVICE_LINE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return DEVICE_LINE;
}

^"Access:" {
	#ifdef DEBUG
	g_message("%s: ACCESS_LINE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return ACCESS_LINE;
}

^"Modify:" {
	#ifdef DEBUG
	g_message("%s: MODIFY_LINE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return MODIFY_LINE;
}

^"Change:" {
	#ifdef DEBUG
	g_message("%s: CHANGE_LINE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return CHANGE_LINE;
}

"Blocks:" {
	#ifdef DEBUG
	g_message("%s: BLOCKS: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return BLOCKS;
}

"IO Block:" {
	#ifdef DEBUG
	g_message("%s: IO_BLOCKS: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return IO_BLOCK;
}

"Inode:" {
	#ifdef DEBUG
	g_message("%s: INODE: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return INODE;
}

"Links:" {
	#ifdef DEBUG
	g_message("%s: LINKS: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return LINKS;
}

"Uid:" {
	#ifdef DEBUG
	g_message("%s: UID: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return UID;
}

"Gid:" {
	#ifdef DEBUG
	g_message("%s: GID: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	terminal_apply_tag(terminal_for_lex, "unimportant",
		stat_lval.sval.begin, 
		stat_lval.sval.end); 
	return GID;
}


	/*
	 * Quoted strings.
	 */
[\`\"]      {
	#ifdef DEBUG
	g_message("%s: QUOTE_START: <%s>", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(STRING);
	return '"';
}

<STRING>[\'\"] {
	#ifdef DEBUG
	g_message("%s: QUOTE_END: <%s>", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	BEGIN(0);
	return '"';
}

<STRING>[^\'\"]+ {
	#ifdef DEBUG
	g_message("%s: FILENAME: <%s>", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return FILENAME; 
}

[0-9]+[hd]? 	{
	#ifdef DEBUG
	g_message("%s: NUMBER: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return NUMBER;
}


[a-z]{2,2}\/[0-9]{2,2}[a-z]{1,1}	{
	#ifdef DEBUG
	g_message("%s: DEVICE: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return DEVICE;
}

{DATE}		{
	#ifdef DEBUG
	g_message("%s: DATE: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return DATE;
}

{TIME}		{
	#ifdef DEBUG
	g_message("%s: TIME: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return TIME;
}

{TIME_ZONE}     {
	#ifdef DEBUG
	g_message("%s: ZONE: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return ZONE;
}

		
[-dlpcb]([-r][-w][-xst]){3,3} {
	#ifdef DEBUG
	g_message("%s: MODE_STRING: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return MODE_STRING;
}

[-a-zA-Z0-9öüóőúéáűÖÜÓŐÚÉÁŰ]*	{
	#ifdef DEBUG
	g_message("%s: WORD: '%s'", __func__, yytext);
	#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
	return WORD;
}

\n { 
	/* Dropping it away */
	//g_message("%s: spaces: <%s>", __func__, yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

[ \t]+ { 
	/* Dropping it away */
	//g_message("%s: spaces: <%s>", __func__, yytext);
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}
	

.	{
	#ifdef DEBUG
	g_message("%s: OTHER_CHAR: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
	return yytext[0];
}

^"stat: "[^\n\r]+ {
	/*
	 * These are error messages.
	 */
#ifdef LS_DEBUG
	g_message("%s: STAT_ERROR_MESSAGE: '%s'", __func__, yytext);
#endif
	stat_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, 
		stat_lval.sval.text,
		&stat_lval.sval.begin,
		&stat_lval.sval.end);
  	return(STAT_ERROR_MESSAGE);
}

%%

