%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: terminal_lexer.l,v 1.1 2006/02/24 08:59:36 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: terminal_lexer.l,v $
 * Revision 1.1  2006/02/24 08:59:36  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include "cuiterm.h"
#include "terminal.h"
#include "terminal_parser.h"

#if DEBUG_ALL
#  define DEBUG
#endif


#undef YY_INPUT
#define YY_INPUT(b, r, ms) (r = terminal_get_buffer_char(b, ms))

extern gint attributes[];
extern gint nattributes;
extern gint attributes_new[];
extern gint nattributes_new;

static void attributes_reset(void){
	nattributes = 0;
}

static void attributes_add(gchar *str){
	attributes_new[nattributes_new++] = atoi(str);
}

static void attributes_validate(void){
	int n;
	for (n = 0; n < nattributes_new; ++n)
		attributes[nattributes++] = attributes_new[n];
	nattributes_new = 0;
}

/*
 * Unknown sequences:
 * \033[12l
 * \033[25h
 */
%}

%x WDSTATE PRSTATE PRHOSTSTATE PRDIRSTATE
%x ESCSTATE

%%
	//g_message("-------->%d", YY_START);

"\033c"    { g_message("%s(): Unsupported 'Reset device'", __func__);          }
"\033[c"   { g_message("%s(): Unsupported 'Query Device Code'", __func__);     }
"\033[5n"  { g_message("%s(): Unsupported 'Query Device Status'", __func__);   }
"\033[6n"  { g_message("%s(): Unsupported 'Query Cursor Position'", __func__); }
"\033[7h"  { g_message("%s(): Unsupported 'Enable Line Wrap'", __func__);      }
"\033[7l"  { g_message("%s(): Unsupported 'Disable Line Wrap'", __func__);     }
"\033[mB"  { g_message("%s(): Unsupported unknown code", __func__); }
"\033[?12l" { 
	g_message("%s(): Unsupported 'Show cursor'", __func__);           
}

"\033[?25h" { 
	g_message("%s(): Unsupported 'Hide cursor'", __func__);           
}

"\033[1@" {
#ifdef DEBUG
	g_message("%s(): INSERT_ONCE", __func__);
#endif
	return INSERT_ONCE;
}

"\033[4h" { 
#ifdef DEBUG
	g_message("%s(): INSERT", __func__);
#endif
	return INSERT;     
}

"\033[4l" { 
#ifdef DEBUG
	g_message("%s(): OVERWRITE", __func__);
#endif
	return OVERWRITE;     
} 

"\033(B"  { g_message("%s(): Unsupported 'End alternative character set'", __func__);        }
"\033("   { g_message("%s(): Unsupported 'Font Set G0'", __func__);        }
"\033)"   { g_message("%s(): Unsupported 'Font Set G1'", __func__);        }

"\007"   { 
#ifdef DEBUG
	g_message("%s(): BEEP", __func__);
#endif
	return BEEP;      
}

"\010"   { 
#ifdef DEBUG
	g_message("%s(): BACKSPACE", __func__);
#endif
	return BACKSPACE; 
}

"\033[H" { 
#ifdef DEBUG
	g_message("%s(): CURSOR_HOME", __func__);
#endif
	return CURSOR_HOME;     
}

"\033[A" { 
#ifdef DEBUG
	g_message("%s(): CURSOR_UP", __func__);
#endif
	return CURSOR_UP;       
}

"\033[B" { 
#ifdef DEBUG
	g_message("%s(): CURSOR_DOWN", __func__);
#endif
	return CURSOR_DOWN;     
}

"\033[C" { 
#ifdef DEBUG
	g_message("%s(): CURSOR_FORWARD", __func__);
#endif
	return CURSOR_FORWARD;  
}

"\033[D" { 
#ifdef DEBUG
	g_message("%s(): CURSOR_BACKWARD", __func__);
#endif
	return CURSOR_BACKWARD; 
}

"\033[s" { 
#ifdef DEBUG
	g_message("%s(): SAVE_CURSOR", __func__);
#endif
	return SAVE_CURSOR;     
}

"\033[u" { 
#ifdef DEBUG
	g_message("%s(): UNSAVE_CURSOR", __func__);
#endif
	return UNSAVE_CURSOR;   
}

"\033"7  { 
#ifdef DEBUG
	g_message("%s(): SAVE_CUR_ATTR", __func__);
#endif
	return SAVE_CUR_ATTR;   
}

"\033"8  { 
#ifdef DEBUG
	g_message("%s(): UNSAVE_CUR_ATTR", __func__);
#endif
	return UNSAVE_CUR_ATTR; 
}

"\033[K"        { 
#ifdef DEBUG
	g_message("%s(): DELETE_TO_END", __func__);
#endif
	return DELETE_TO_END;   
}

"\033[1K"       { 
#ifdef DEBUG
	g_message("%s(): DELETE_TO_BEGIN", __func__);
#endif
	return DELETE_TO_BEGIN; 
}

"\033[2K"       { 
#ifdef DEBUG
	g_message("%s(): DELETE_LINE", __func__);
#endif
	return DELETE_LINE;     
}

"\033[P"        { 
#ifdef DEBUG
	g_message("%s(): DELETE_ONE", __func__);
#endif
	return DELETE_ONE;      
}

"\033[2J"       { 
#ifdef DEBUG
	g_message("%s(): ERASE_SCREEN", __func__);
#endif
	return ERASE_SCREEN;    
}

"\033["[0-9]+B  { 
#ifdef DEBUG
	g_message("%s(): DOWN_N", __func__);
#endif
	terminal_lval.ival = atoi(yytext + 2); 
	return DOWN_N;  
}

"\033["[0-9]+P  { 
#ifdef DEBUG
	g_message("%s(): DELETE_N", __func__);
#endif
	terminal_lval.ival = atoi(yytext + 2); 
	return DELETE_N;  
}

"\033["[0-9]+C  { 
#ifdef DEBUG
	g_message("%s(): FORWARD_N", __func__);
#endif
	terminal_lval.ival = atoi(yytext + 2); 
	return FORWARD_N; 
}

"\033["[0-9]+D  { 
#ifdef DEBUG
	g_message("%s(): BACKWARD_N", __func__);
#endif
	terminal_lval.ival = atoi(yytext + 2); 
	return BACKWARD_N;
}

\n {
#ifdef DEBUG
	g_message("%s(): '\\n'", __func__);
#endif
	terminal_lval.sval = g_strdup(yytext);
	return NEW_LINE;
}

\r {
#ifdef DEBUG
	g_message("%s(): '\\r'", __func__);
#endif
	terminal_lval.sval = g_strdup(yytext);
	return RETURN;
}

"\033["27m {
#ifdef DEBUG
	g_message("%s(): attributes reset?: '%s'", __func__, yytext);
#endif
	attributes_reset();
}

"\033["0*m {
#ifdef DEBUG
	g_message("%s(): attributes reset: '%s'", __func__, yytext);
#endif
	attributes_reset();
}

"\033["   { 
#ifdef DEBUG
	g_message("%s(): ESC[", __func__);
#endif
	BEGIN(ESCSTATE);
}

<ESCSTATE>[0-9]+ {
#ifdef DEBUG
	g_message("%s(): attribute(escstate): '%s'", __func__, yytext);
#endif
	attributes_add(yytext);
}

<ESCSTATE>; {
#ifdef DEBUG
	g_message("%s(): parameter separator(escstate) ';'", __func__);
#endif
}

<ESCSTATE>m {
#ifdef DEBUG
	g_message("%s(): attributes validate", __func__);
#endif
	attributes_validate();
	BEGIN(INITIAL);
}

<ESCSTATE>H {
#ifdef DEBUG
	g_message("%s(): GOTO", __func__);
#endif
	BEGIN(INITIAL);
	return GOTO;
}

<ESCSTATE>[a-zA-Z] {
#ifdef DEBUG
	g_message("%s(): dropped char(escstate): '%c'", __func__, yytext[0]);
#endif
	BEGIN(INITIAL);
}

"\033]0;"    { 
#ifdef DEBUG
	g_message("%s(): WD_BEGIN", __func__);
#endif
	BEGIN(WDSTATE);
	return WD_BEGIN;
}

"\r\033]0;"    { 
#ifdef DEBUG
	g_message("%s(): WD_BEGIN_AGAIN", __func__);
#endif
	BEGIN(WDSTATE);
	return WD_BEGIN_AGAIN;
}

<WDSTATE>[^\007]+ {
#ifdef DEBUG
	g_message("%s(): WD: \"%s\"", __func__, yytext);
#endif
	terminal_lval.sval = g_strdup(yytext);
	return WD;
}

<WDSTATE>\007 {
#ifdef DEBUG
	g_message("%s(): WD_END", __func__);
#endif
	BEGIN(PRSTATE);
	return WD_END;
}

<PRSTATE>[#$][ \t]* {
#ifdef DEBUG
	g_message("%s(): PREND: '%s'", __func__, yytext);
#endif
	terminal_lval.sval = g_strdup(yytext);
	BEGIN(INITIAL);
	return PREND;
}

<PRSTATE>[^@\[\]#]+ {
#ifdef DEBUG
	g_message("%s(): USER (prompt): \"%s\"", __func__, yytext);
#endif
	terminal_lval.sval = g_strdup(yytext);
	BEGIN(PRHOSTSTATE);
	return USER;
}

<PRHOSTSTATE>[^ @\[\]#]+ {
#ifdef DEBUG
	g_message("%s(): HOST (prompt): \"%s\"", __func__, yytext);
#endif
	terminal_lval.sval = g_strdup(yytext);
	BEGIN(PRDIRSTATE);
	return HOST;
}

<PRDIRSTATE>[^ ][^\[\]#]* {
#ifdef DEBUG
	g_message("%s(): DIRECTORY (prompt): \"%s\"", __func__, yytext);
#endif
	terminal_lval.sval = g_strdup(yytext);
	BEGIN(PRSTATE);
	return DIRECTORY;
}

<PRSTATE,PRHOSTSTATE,PRDIRSTATE>. {
#ifdef DEBUG
	g_message("%s(): other char(prompt): '%c'", __func__, yytext[0]);
#endif
	terminal_lval.sval = g_strdup(yytext);
	return yytext[0];
}

"\033"    { 
#ifdef DEBUG
	g_message("%s(): ESC(simple)", __func__);
#endif
	return ESC;
}

([^[:cntrl:]]|\t)+ {
#ifdef DEBUG
	g_message("%s(): STRING: '%s'", __func__, yytext);
#endif
	terminal_lval.sval = g_strdup(yytext);
	return STRING;
}

"\000" {
#ifdef DEBUG
	g_message("%s(): '\\0'", __func__);
#endif
}


<*>. {
#ifdef DEBUG
	g_message("%s(): dropped other char: '%s'", __func__, yytext);
#endif
}

%%
