%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: terminal_parser.y,v 1.1 2006/02/24 08:12:43 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: terminal_parser.y,v $
 * Revision 1.1  2006/02/24 08:12:43  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "cuiterm.h"
#include "cuilexelements.h"
#include "terminal_parser.h"
#include "terminal.h"
#include "background.h"
#include "cuiwindow.h"

#if DEBUG_ALL
#  define DEBUG
#endif

gint attributes[128];
gint nattributes = 0;
gint attributes_new[128];
gint nattributes_new = 0;

%}

%parse-param {Terminal *terminal}
%union { 
	gchar *sval;
	gint   ival;
}
 
%token <sval> STRING USER HOST DIRECTORY
%token <sval> WD_BEGIN WD WD_END
%token <sval> WD_BEGIN_AGAIN
%token <sval> PREND
%token        ESC
%token        BEEP BACKSPACE
%token        INSERT OVERWRITE INSERT_ONCE
%token <sval> NEW_LINE RETURN CR_NL
%token        ERASE_SCREEN
%token        CURSOR_HOME GOTO 
%token        CURSOR_UP CURSOR_DOWN CURSOR_FORWARD CURSOR_BACKWARD
%token        SAVE_CURSOR UNSAVE_CURSOR SAVE_CUR_ATTR UNSAVE_CUR_ATTR
%token        DELETE_TO_END DELETE_TO_BEGIN DELETE_LINE DELETE_ONE
%token <ival> DELETE_N FORWARD_N BACKWARD_N DOWN_N
%%

tags
   : tags tag 
   | tag      
   ;

tag 
   : cursor_move
   | delete_chars 
   | begin        
   | wd prompt
   | single_escape
   | string 
   | error        { g_message("%s(): Terminal syntax error.", __func__); }
   ;

single_escape
   : BEEP         { gdk_beep(); }
   | BACKSPACE    { terminal_sequence_cursor_left(terminal); } 
   | RETURN       { terminal_sequence_text(terminal, "\r");  }
   | NEW_LINE     { terminal_sequence_text(terminal, "\n");  }
   | ERASE_SCREEN { terminal_sequence_erase(terminal);       }
   | OVERWRITE    { terminal_sequence_overwrite(terminal);   }
   | INSERT       { terminal_sequence_insert(terminal);      }
   | ESC
   ;

string
   : STRING  { if (nattributes == 0)
   			terminal_sequence_text(terminal, $1); 
		    else
			terminal_sequence_text_with_attributes(terminal, 
				$1, attributes, nattributes); }
   | INSERT_ONCE  { terminal_sequence_insert(terminal);      }
       string     { terminal_sequence_overwrite(terminal);   }
   ;

begin
   : RETURN NEW_LINE {
     		#ifdef DEBUG
			g_message("%s(): Terminal Enter came", __func__);
			terminal_debug_print_status(terminal);
		#endif
		if (terminal->status == TermSendingUserEnter){
			terminal_get_command(terminal);
			terminal_command_starting(terminal);
			terminal->status = TermExecuting;	
		} else if (terminal->status == TermSendingCommand){
			terminal_get_command(terminal);
			terminal_command_starting(terminal);
			terminal->status = TermExecuting;	
		}
     		#ifdef DEBUG
			g_message("%s(): Terminal sending Enter", __func__);
			terminal_debug_print_status(terminal);
		#endif
		/*
		 * Yes, it's really strange. We have a \r\n but we put only \n
		 * into the terminal. I hate MS-DOS...
		 */
                terminal_sequence_text(terminal, "\n"); 
	     }
   ;

wd
   : WD_BEGIN WD WD_END {
  		GString *working_directory = g_string_new($2);
		terminal_set_working_directory(terminal, working_directory);
  		
		#ifdef DEBUG
			g_message("%s(): Terminal Working Dir: '%s'", __func__, 
				working_directory->str);
			terminal_debug_print_status(terminal);
		#endif
		g_string_free(working_directory, TRUE);
	}
   | WD_BEGIN_AGAIN WD WD_END {
  		GString *working_directory = g_string_new($2);
		//terminal->command_begin_cursor = NULL;
		terminal_set_working_directory(terminal, working_directory);
  		
		#ifdef DEBUG
			g_message("%s(): Terminal Working Dir (again): '%s'", __func__, working_directory->str);
			terminal_debug_print_status(terminal);
		#endif
		g_string_free(working_directory, TRUE);
		terminal_sequence_cursor_home(terminal);
	}
   ; 

prompt
   : '[' USER '@' HOST ' ' DIRECTORY ']' PREND {
      		gchar *prompt;
		#ifdef DEBUG
			g_message("%s(): Prompt came", __func__);
			terminal_debug_print_status(terminal);
		#endif

		prompt = g_strconcat("[", $2, "@", $4, " ", $6, "]", $8, NULL);
		terminal_sequence_prompt(terminal, prompt);
		/*
		 * Storing some information.
		 */
		terminal->username  = g_string_assign(terminal->username, $2);
		terminal->hostname  = g_string_assign(terminal->hostname, $4);
		terminal->directory = g_string_assign(terminal->directory, $6);
		terminal->prompt_length = g_utf8_strlen(prompt, -1);
		if (terminal->status == TermUninitialized){
			terminal->status = TermReady;
			terminal_send_init_commands(terminal);
		} else if (terminal->status == TermExecuting){
			terminal->status = TermReady;
			terminal_command_ended(terminal);
			terminal->command = g_string_truncate(terminal->command, 0);
		} else if (terminal->status == TermReady) {
			/* What then? */
		}
		#ifdef DEBUG
			g_message("%s(): Prompt processed", __func__);
			terminal_debug_print_status(terminal);
		#endif
		g_free(prompt);
	}
   ;

cursor_move
   : CURSOR_HOME     { terminal_sequence_cursor_xy(terminal, 0, 0); }
   | GOTO            { 
		terminal_sequence_cursor_xy(terminal, attributes_new[1], attributes_new[0]); 
		nattributes_new = 0;
	      }
   | FORWARD_N       { terminal_sequence_cursor_right_n(terminal, $1); }
   | BACKWARD_N      { terminal_sequence_cursor_left_n(terminal, $1);  }
   | CURSOR_UP       { terminal_sequence_cursor_up(terminal);          }
   | CURSOR_DOWN     { terminal_sequence_cursor_down(terminal);        }
   | DOWN_N          { terminal_sequence_cursor_down_n(terminal, $1);  }
   | CURSOR_FORWARD  { terminal_sequence_cursor_right(terminal);  }
   | CURSOR_BACKWARD { terminal_sequence_cursor_left(terminal);   }
   | SAVE_CURSOR     { g_warning("Unsupported: SAVE_CURSOR");     }
   | UNSAVE_CURSOR   { g_warning("Unsupported: UNSAVE_CURSOR");   }
   | SAVE_CUR_ATTR   { g_warning("Unsupported: SAVE_CUR_ATTR");   }
   | UNSAVE_CUR_ATTR { g_warning("Unsupported: UNSAVE_CUR_ATTR"); }
   ;

delete_chars
   : DELETE_TO_END   { terminal_sequence_delete_to_end(terminal);   }
   | DELETE_TO_BEGIN { terminal_sequence_delete_to_begin(terminal); }
   | DELETE_LINE     { terminal_sequence_delete_line(terminal);     }
   | DELETE_ONE      { terminal_sequence_delete_n(terminal, 1);     }
   | DELETE_N        { terminal_sequence_delete_n(terminal, $1);    }
   ;

%%

int terminal_error(Terminal *terminal, char *c)
{
	g_message("%s(): terminal error: %s", __func__, c);
	return(1);
}

int terminal_wrap(void)
{
#ifdef DEBUG
	g_message("%s(): returning 1", __func__);
#endif
	return 1;
}
