%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: tree_lexer.l,v 1.1 2006/02/24 08:58:46 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: tree_lexer.l,v $
 * Revision 1.1  2006/02/24 08:58:46  pipas
 * Initial revision
 *
 */
#ifdef HAVE_CONFIG_H
# include <config.h>
#endif

#define _GNU_SOURCE
#include <gnome.h>
#include <string.h>
#include "ls_parser_c.h"
#include "terminal.h"
#include "cuiwindow.h"
#include "bash_parser_c.h"
#include "tree_parser.h"

#undef TREE_DEBUG

#undef YY_INPUT
extern Terminal *terminal_for_lex;
extern CuiWindow *window_for_lex;
#define YY_INPUT(b, r, ms) (r = cui_window_get_char(window_for_lex, b, ms))
%}

%option yylineno

%%

	if (window_for_lex->parsed_chars == 0)
		BEGIN(0);

"|   " {
	#ifdef TREE_DEBUG
		g_message("%s: INDENT: '%s'", __func__, yytext);
	#endif
	tree_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, tree_lval.sval.text,
		&tree_lval.sval.begin,
		&tree_lval.sval.end);
	return INDENT;
  }

"    "|"|-- "|"`-- " {
	#ifdef TREE_DEBUG
		g_message("%s: BRANCH: '%s'", __func__, yytext);
	#endif
	tree_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, tree_lval.sval.text,
		&tree_lval.sval.begin,
		&tree_lval.sval.end);
	return BRANCH;
  }

[^\t \n|`][^\t\n]* {
	#ifdef TREE_DEBUG
		g_message("%s: FILENAME: '%s'", __func__, yytext);
	#endif
	tree_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, tree_lval.sval.text,
		&tree_lval.sval.begin,
		&tree_lval.sval.end);
	return FILENAME;
  }

[^\t \n|`][^\t\n]/\/ {
	#ifdef TREE_DEBUG
		g_message("%s: DIRNAME: '%s'", __func__, yytext);
	#endif
	tree_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, tree_lval.sval.text,
		&tree_lval.sval.begin,
		&tree_lval.sval.end);
	return DIRNAME;
  }

[0-9]+" directories, "[0-9]+" files" {
	#ifdef TREE_DEBUG
		g_message("%s: LAST_LINE: '%s'", __func__, yytext);
	#endif
	tree_lval.sval.text = strdup(yytext);
	cui_window_get_token_marks(window_for_lex, tree_lval.sval.text,
		&tree_lval.sval.begin,
		&tree_lval.sval.end);
	return LAST_LINE;
}

.|\n {
	#ifdef TREE_DEBUG
		g_message("%s: dropping: '%s'", __func__, yytext);
	#endif
	cui_window_get_token_marks(window_for_lex, yytext, NULL, NULL);
}

%%



