%{
/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: tree_parser.y,v 1.1 2006/02/24 08:13:24 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: tree_parser.y,v $
 * Revision 1.1  2006/02/24 08:13:24  pipas
 * Initial revision
 *
 */
#include <gnome.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <math.h>

#include "ls_parser_c.h"
#include "cuiterm.h"
#include "terminal.h"
#include "cuiwindow.h"

extern CuiWindow *window_for_lex;
GList *nl = NULL; 	/* name list used to construct pathnames 	*/
GList *ul = NULL;       /* the unused elements needed to free the mem.  */
gint  ld = 0;           /* last depth used to detect tree movement	*/
gchar *fullname = NULL;

/*
 * Creates a filename from the GList which has all elements a 
 * CuiLexElement pointer as data.
 */
static gchar *
build_filename_from_list(GList *l)
{
	GList *listelement;
	CuiLexElement *le;
	gchar *fullname = g_strdup("");
	gchar *tmp;
	
	listelement = l;
	while (listelement != NULL){
		le = (CuiLexElement *)listelement->data;
		tmp = fullname;
		fullname = g_build_filename(fullname, le->text, NULL);
		g_free(tmp);
next_element:
		listelement = listelement->next; 
	}
	/*
	 * If the full-name starts with "./" we have to remove these two
	 * characters.
	 */
	if (strncmp(fullname, "./", 2) == 0){
		tmp = fullname;
		fullname = g_strdup(fullname + 2);
		g_free(tmp);
	}
	return fullname;
}

%}

%parse-param {CuiWindow *window}
%union { 
	CuiLexElement 	sval;
	gint 		ival;
}
 
%type  <ival> depth
%type  <sval> filename
%token <sval> LAST_LINE
%token <sval> STRING DIRNAME FILENAME LINKNAME
%token <sval> INDENT BRANCH
%token <sval> ERROR_MESSAGE
/* 
 * This grammar describes the language the 'tree' command uses to print a
 * tree like directory structure.
 *
 */
%%
lines: lines line
     | line
     ;

line: LAST_LINE {
    			/* 
			 * When the last line, a greand total of the tree
			 * program comes, we have to process the file name we
			 * last saw.
			 */
			CuiFilename *fn;	
			GList *l = g_list_last(nl);
			CuiLexElement *le = (CuiLexElement *)l->data;
			Background *background;
			background = cui_widget_get_background(CUI_WIDGET(window));
			
			fn = cui_filename_new(fullname, window->wd->str);
			//if (is_dir)
			//	cui_filename_set_filetype_dir(fn);
			/* Action area */
			cui_action_area_new_for_filename(
				window, background, 
				fn, le->begin, le->end);
			/* Filename menu */
			cui_file_menu_new_for_filename(window, background, fn, 
				le->begin, le->end);
			/* Filename entry */
			cui_entry_new_for_filename(window, background, 
				fn, le->begin, le->end, le->text);
			/* Syntay highlight */
			cui_syntax_highlight_apply_filename(background, fn, le->begin, le->end);
		} 
    | depth filename { 
		/* 
		 * We always process the element before the actual one, so we
		 * can see, if the given element is a directory. The tree is
		 * going deeper after a directory, at least after a non-empty
		 * directory.
		 */
		CuiLexElement *le = NULL;         /*  */
		GList *l = g_list_last(nl);       /*  */
		gboolean is_dir = FALSE;
		Background *background;
		background = cui_widget_get_background(CUI_WIDGET(window));
		
		if (l != NULL)
			l = l->prev;
		if (l != NULL)
			le = (CuiLexElement *)l->data;
		
		if (ld < $1){
		       /*
		 	* If we go one deeper, the line before this was a directory
		 	* for sure, and this name is in that directory.
			*/
			is_dir = TRUE;
		}else{
			/*
			 * Otherwise we dont need this element anymore, so we
			 * remove it from the end of the list. However we can't 
			 * destroy the CuiLexElement, because we are going to use 
			 * it, so we add to the unused list. 
			 */
			if (l != NULL)
				ul = g_list_prepend(ul, l->data);
			nl = g_list_remove_link(nl, l);
		}
		
		/*
		 * If we go back, we remove the element before the last element, 
		 * since that is no more the part of the name.
		 */
		if (ld > $1){
			l = g_list_last(nl);
			l = l->prev;
			nl = g_list_remove_link(nl, l);
		}
		
		/*
		 * Here we create the widgets if the element before this last
		 * one is an existing one, so this is not the first element.
		 */
		CuiFilename *fn;
		if(le != NULL){
			fn = cui_filename_new(fullname, window->wd->str);
			if (is_dir)
				cui_filename_set_filetype_dir(fn);
	   		/* Action area */
			cui_action_area_new_for_filename(
				window,
				background, fn, le->begin, le->end);
			/* Filename menu */
			cui_file_menu_new_for_filename(window, background, fn, 
				le->begin, le->end);
			/* Filename entry */
			cui_entry_new_for_filename(window, background, 
				fn, le->begin, le->end, le->text);
			/* Syntay highlight */
			cui_syntax_highlight_apply_filename(background, fn, 
				le->begin, le->end);
		
		/********************************************************
		 * Drag&Drop target and source area.                    *
		 ********************************************************/
		GtkTargetList *tlist;
		CuiWidget     *widget;
		widget = cui_rectangle_new();
	   	cui_widget_set_background(widget, background);
	   	cui_widget_set_area_by_marks(widget, le->begin, le->end);
		cui_widget_set_width_by_percent(widget, 1.00);
		if (cui_filename_is_dir(fn))
			tlist = dnd_get_dir_target_list();
		else
			tlist = dnd_get_file_target_list();
		cui_rectangle_set_target_list(CUI_RECTANGLE(widget), tlist);
		cui_rectangle_set_target_object(CUI_RECTANGLE(widget), 
			G_OBJECT(fn));
		cui_rectangle_set_is_dnd_target(CUI_RECTANGLE(widget), TRUE);
	        cui_widget_append_window_command(widget, WindowRefresh, window);
		/* DnD source for filename */	
		widget = cui_rectangle_new();
	   	cui_widget_set_background(widget, background);
	   	cui_widget_set_area_by_marks(widget, le->begin, le->end);
		cui_rectangle_set_dnd_source_data(CUI_RECTANGLE(widget),
			TargetFileForName, fn, sizeof(CuiFilename));
		cui_rectangle_set_is_dnd_source(CUI_RECTANGLE(widget), TRUE);
		}
		
		/*
		 * Saving the full name of this file for we need it processing
		 * the next line.
		 */
		g_free(fullname);
		fullname = build_filename_from_list(nl);
		ld = $1;
        }
    | error { window->error_code++; }
    ;

depth:              { $$ = 0;      } 
     | INDENT       { $$ = 1;      }
     | BRANCH       { $$ = 1;      }
     | depth INDENT { $$ = $1 + 1; }
     | depth BRANCH { $$ = $1 + 1; }
     ;

filename: STRING   { nl = g_list_append(nl, g_memdup(&$1, sizeof(CuiLexElement))); }
        | DIRNAME  { nl = g_list_append(nl, g_memdup(&$1, sizeof(CuiLexElement))); }
        | FILENAME { nl = g_list_append(nl, g_memdup(&$1, sizeof(CuiLexElement))); }
        | LINKNAME { nl = g_list_append(nl, g_memdup(&$1, sizeof(CuiLexElement))); }
        ;
%%

/**
 * Frees the allocated resources and resets the parser.
 */
static void
finalize_parser(void)
{
	GList *l;
	CuiLexElement *le;
	/*
	 * Destroying the list. This will not destroy only the few elements we 
	 * left in the list after the last filename has been processed.
	 */
	l = nl;
	while (l != NULL){
		le = l->data;
		cui_lex_element_destroy(le, FALSE);
		g_free(le);
		
		l = l->next;
	}
	g_list_free(nl);
	nl = NULL;
	/*
	 * Destroying the unused list.
	 */
	l = ul;
	while (l != NULL){
		le = l->data;
		cui_lex_element_destroy(le, FALSE);
		g_free(le);
		l = l->next;
	}
	g_list_free(ul);
	ul = NULL;
	/*
	 * Other thingy...
	 */
	g_free(fullname);
	fullname = NULL;
	ld = 0;
}

extern int tree_lineno;

int tree_error(CuiWindow *window, char *c )
{
	gchar *error_message;
	
	error_message = g_strdup_printf(
		"File Tree View Error: %s in line %d", 
		c,
		tree_lineno);
	g_warning(error_message);
	cui_window_set_title(window, error_message);
	g_free(error_message);
	
	finalize_parser();
	return(1);
}

int tree_wrap(void)
{
	finalize_parser();
	return 1;
}
