/*
 * Copyright (C) 2004 Laszlo Pere
 * $Id: widgets.h,v 1.1 2006/02/24 17:29:03 pipas Exp pipas $
 *
 * This is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Library General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * $Log: widgets.h,v $
 * Revision 1.1  2006/02/24 17:29:03  pipas
 * Initial revision
 *
 */
#ifndef WIDGETS_H
#define WIDGETS_H

#include <gnome.h>
#include "cuiterm.h"
#include "filename.h"
#include "terminal.h"
#include "command_generator.h"
#include "dnd.h"
#include "icons.h"

typedef enum {
	DecorNone            = 1 << 0,
	DecorIncrease        = 1 << 1,
	DecorDecrease        = 1 << 2
} Decoration;


/******************************************************************************
 * CuiWidget class                                                            *
 *                                                                            *
 ******************************************************************************/
#define CUI_WIDGET_TYPE (cui_widget_get_type())
#define CUI_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_WIDGET_TYPE, CuiWidget))
#define CUI_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_WIDGET_TYPE, CuiWidgetClass))
#define CUI_IS_WIDGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_WIDGET_TYPE))
#define CUI_IS_WIDGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), \
			CUI_WIDGET_TYPE))
#define CUI_WIDGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), \
			CUI_WIDGET_TYPE, CuiWidgetClass))


/**
 * CuiWidget:
 *
 * This is a base class cor the widgets in the CUI environment.
 */
typedef struct _CuiWidget CuiWidget;
/**
 * CuiWidget:
 * @name: the name of the widget instance if any
 * @groupname: the name of the group if any
 * @area:  the location and the size of the widget
 * @active: if the widget can be activated
 * @tooltip: some descriptive text for the user
 * @directory: the working directory of the command
 * @background:  the background this widgets belongs to
 * @terminal: the terminal to send the commands to
 * @appbar: for the tooltip information
 * @commands: The list with the commands we want to execite when the widget 
 *  is activated. 
 * @mouse_cursor_active: The mouse cursor we want to see over the widget.
 * @mouse_cursor_inactive: The mouse cursor we want to see when the mouse 
 *  leaves the widget.
 * @mouse_is_over: This field is TRUE if the mouse cursor is currently over 
 *  the widget.
 * @visible: if the widget is currently visible (FIXME: do we use this?)
 *
 * This is a base class for the widgets in the CUI environment.
 * Currently I'm marking some fields as public since I didn't write the
 * accessing functions for them.
 *
 */
struct _CuiWidget 
{
	GObject parent;
	/*< public >*/
	gchar            *name;        
	gchar            *groupname; 
	GdkRectangle     *area; 
	gboolean          active;    
	gchar            *tooltip;     
	gchar            *directory;   
	Background       *background;
	Terminal         *terminal; 
	GtkWidget        *appbar;  
	gboolean          visible;
	GList            *commands;
	GdkCursor        *mouse_cursor_active;
	GdkCursor        *mouse_cursor_inactive;
	gboolean          mouse_is_over;

	GtkTextMark      *begin_mark;
	GtkTextMark      *end_mark;
	gchar            *active_style;
};

typedef struct {
	GObjectClass parent_class;
	GdkCursor *default_inactive_cursor;
	GdkCursor *default_active_cursor;
} CuiWidgetClass;

GType cui_widget_get_type(void);

/*
 * Public functions of the CuiWidget.
 */
void cui_widget_set_name(CuiWidget *widget, const gchar *name);
gchar *cui_widget_get_name(CuiWidget *widget);
gboolean cui_widget_get_active(CuiWidget *widget);
void cui_widget_set_active(CuiWidget *widget, gboolean active);
void cui_widget_set_directory(CuiWidget *widget, const gchar *directory);
gchar *cui_widget_get_directory(CuiWidget *widget);
void cui_widget_set_background(CuiWidget *widget, const Background *background);
Background * cui_widget_get_background(CuiWidget *widget);
void cui_widget_set_appbar(CuiWidget *widget, GtkWidget *appbar);
void cui_widget_set_tooltip(CuiWidget *widget, const gchar *tooltip);
gboolean cui_widget_is_mouse_over(CuiWidget *widget, gint x, gint y);
void cui_widget_mouse_enter(CuiWidget *widget);
void cui_widget_mouse_leave(CuiWidget *widget);
void cui_widget_set_area_by_coord( CuiWidget *widget, 
		gint x, gint y,
		gint dx, gint dy);
void cui_widget_set_area_by_marks(CuiWidget *widget, 
		GtkTextMark *begin,
		GtkTextMark *end);
void cui_widget_set_area_by_iter( CuiWidget *widget, GtkTextIter *iter);
void cui_widget_set_area(CuiWidget *widget, const GdkRectangle *rectangle);
void cui_widget_move(CuiWidget *widget, gint dx, gint dy);
gint cui_widget_get_size(CuiWidget *widget);
GdkRectangle *cui_widget_get_area(CuiWidget *widget);
GdkRectangle *cui_widget_get_area_window_coords(CuiWidget *widget);
void cui_widget_append_file_command(CuiWidget *widget, 
		CuiCommandOpcode opcode,
		CuiFilename *file_arg);
void cui_widget_append_file_command_with_arg(CuiWidget *widget, 
		CuiCommandOpcode opcode,
		CuiFilename *file_arg,
		gpointer     arg);
void cui_widget_append_window_command(CuiWidget *widget,
		CuiCommandOpcode opcode,
		gpointer window);
void cui_widget_append_window_command_with_arg(CuiWidget *widget,
		CuiCommandOpcode opcode,
		gpointer window,
		gpointer arg);
void cui_widget_activate(CuiWidget *widget);
void cui_widget_set_width_by_percent(
		CuiWidget *widget, 
		gdouble percent);
void cui_widget_set_start_and_stop_by_percent(
		CuiWidget *widget, 
		gdouble start,
		gdouble stop);
void cui_widget_set_terminal(CuiWidget *widget, const Terminal *terminal);
void cui_widget_set_active_style(CuiWidget *widget, const gchar *style);
void cui_widget_add_tag_by_name(CuiWidget *widget, const gchar *tag);


Terminal *cui_widget_get_terminal(CuiWidget *widget);

/******************************************************************************
 * Instantiable classed type: CuiActionArea                                   *
 *                                                                            *
 ******************************************************************************/
#define CUI_ACTION_AREA_TYPE (cui_action_area_get_type ())
#define CUI_ACTION_AREA(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_ACTION_AREA_TYPE, CuiActionArea))
#define CUI_IS_ACTION_AREA(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_ACTION_AREA_TYPE))
#define CUI_ACTION_AREA_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_ACTION_AREA_TYPE, CuiActionAreaClass))

typedef struct {
	CuiWidget parent;
	/* instance members */
} CuiActionArea;

typedef struct {
	CuiWidgetClass parent_class;
	/* class members */
} CuiActionAreaClass;

GType cui_action_area_get_type(void);
CuiWidget *cui_action_area_new(void);

/*
 * Public functions of the CuiActionArea
 */
gboolean cui_action_area_redraw(CuiActionArea *area, gint debug);

/******************************************************************************
 * Instantiable classed type: CuiPixmap                                       *
 *                                                                            *
 ******************************************************************************/
#define CUI_PIXMAP_TYPE (cui_pixmap_get_type ())
#define CUI_PIXMAP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_PIXMAP_TYPE, CuiPixmap))
#define CUI_IS_PIXMAP(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_PIXMAP_TYPE))
#define CUI_PIXMAP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_PIXMAP_TYPE, CuiPixmapClass))


/**
 * CuiPixmap:
 *
 * A widget capaple of showing icons and other graphical elements on the CUI
 * area.
 */
typedef struct _CuiPixmap CuiPixmap;

struct _CuiPixmap {
	CuiWidget  parent;
	gchar     *filename;
	GdkPixbuf *pixbuf;
	GdkPixmap *pixmap;
	GdkBitmap *bitmap;
	CuiIcon   *icon;
};

typedef struct _CuiPixmapClass CuiPixmapClass;

struct _CuiPixmapClass {
	CuiWidgetClass parent_class;
	/*< public >*/
};

GType cui_pixmap_get_type(void);
CuiWidget *cui_pixmap_new_empty(void);
CuiWidget *cui_pixmap_new_from_package_file(const gchar *filename);
CuiWidget *cui_pixmap_new_thumbnail(CuiFilename *file);
void cui_pixmap_set_max_size(CuiPixmap *cui_pixmap, gint max_size);
void cui_pixmap_set_max_height(CuiPixmap *cui_pixmap, gint max_height);

gboolean cui_pixmap_redraw(CuiPixmap *pixmap, gint debug);
void cui_pixmap_set_xy(CuiPixmap *cui_pixmap, gint x, gint y);

/******************************************************************************
 * Instantiable classed type: CuiEntry                                        *
 *                                                                            *
 ******************************************************************************/
#define CUI_ENTRY_TYPE (cui_entry_get_type ())
#define CUI_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_ENTRY_TYPE, CuiEntry))
#define CUI_IS_ENTRY(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_ENTRY_TYPE))
#define CUI_ENTRY_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_ENTRY_TYPE, CuiEntryClass))

/**
 * CuiEntry:
 *
 * I'm setting every field to public because I'm working on this part of the
 * program.
 */
typedef struct {
	CuiWidget parent;
	gchar *default_text;
	GtkWidget *box;
	GtkWidget *window;
	GtkWidget *entry;
	GtkWidget *ok;
	GtkWidget *cancel;
	GtkWidget *ok_pixmap;
	GtkWidget *cancel_pixmap;
} CuiEntry;

typedef struct {
	CuiWidgetClass parent_class;
} CuiEntryClass;

GType cui_entry_get_type(void);

CuiWidget *cui_entry_new(void);
void cui_entry_set_default_text(CuiEntry *entry, const gchar *text);
void cui_entry_realize(CuiEntry *entry);

/******************************************************************************
 * Instantiable classed type: CuiCalendar                                     *
 *                                                                            *
 ******************************************************************************/
#define CUI_CALENDAR_TYPE (cui_calendar_get_type ())
#define CUI_CALENDAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_CALENDAR_TYPE, CuiCalendar))
#define CUI_IS_CALENDAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_CALENDAR_TYPE))
#define CUI_CALENDAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_CALENDAR_TYPE, CuiCalendarClass))

/**
 * CuiCalendar:
 *
 * This widget is similar to the CuiEntry but it shows a calendar to select a
 * date and a time index.
 */
typedef struct {
	CuiWidget parent;
	guint default_year;
	guint default_month;
	guint default_day;
	gboolean default_date_set;
	guint year;
	guint month;
	guint day;
	gboolean date_chosen;
	GtkWidget *hbox;
	GtkWidget *vbox;
	GtkWidget *window;
	GtkWidget *entry;
	GtkWidget *calendar;
	GtkWidget *ok;
	GtkWidget *cancel;
	GtkWidget *ok_pixmap;
	GtkWidget *cancel_pixmap;
} CuiCalendar;

typedef struct {
	CuiWidgetClass parent_class;
} CuiCalendarClass;

GType cui_calendar_get_type(void);
CuiWidget *cui_calendar_new(void);
void cui_calendar_realize(CuiCalendar *calendar);
void cui_calendar_set_default_date(CuiCalendar *calendar, gchar *date);

/******************************************************************************
 * Instantiable classed type: CuiPopupMenu                                    *
 *                                                                            *
 ******************************************************************************/
#define CUI_POPUP_MENU_TYPE (cui_popup_menu_get_type ())
#define CUI_POPUP_MENU(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_POPUP_MENU_TYPE, CuiPopupMenu))
#define CUI_IS_POPUP_MENU(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_POPUP_MENU_TYPE))
#define CUI_POPUP_MENU_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_POPUP_MENU_TYPE, CuiPopupMenuClass))

/**
 * CuiPopupMenu:
 * @menu: a #GtkWidget menu object we use, when every instance has different
 * menus, mainly in inherited classes with dynamically generated menus.
 *
 */
typedef struct {
	CuiWidget parent;
	GtkWidget *menu;
} CuiPopupMenu;

/**
 * CuiPopupMenuClass:
 */
typedef struct {
	CuiWidgetClass parent_class;
} CuiPopupMenuClass;

GType cui_popup_menu_get_type(void);
CuiWidget *cui_popup_menu_new(GtkWidget *gtk_widget);
void cui_popup_menu_popup(CuiPopupMenu *cui_popup_menu,
		GtkWidget *parent_widget,
		GdkEventButton *event);

/******************************************************************************
 * Instantiable classed type: CuiFileMenu                                     *
 *                                                                            *
 ******************************************************************************/
#define CUI_FILE_MENU_TYPE (cui_file_menu_get_type ())
#define CUI_FILE_MENU(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_FILE_MENU_TYPE, CuiFileMenu))
#define CUI_IS_FILE_MENU(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_FILE_MENU_TYPE))
#define CUI_FILE_MENU_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_FILE_MENU_TYPE, CuiFileMenuClass))

typedef struct {
	CuiPopupMenu parent;
	GtkWidget   *menu;
	CuiFilename *file;
} CuiFileMenu;

typedef struct {
	CuiPopupMenuClass parent_class;
	GtkWidget *menu;
} CuiFileMenuClass;

GType cui_file_menu_get_type(void);
CuiWidget *cui_file_menu_new(void);
void cui_file_menu_set_filename(CuiFileMenu *file_menu, CuiFilename *filename);
CuiFilename *cui_file_menu_get_filename(CuiFileMenu *file_menu);
void cui_file_menu_popup(CuiFileMenu *cui_file_menu,
		GtkWidget *parent_widget,
		GdkEventButton *event);

/******************************************************************************
 * Instantiable classed type: CuiModeMenu                                     *
 *                                                                            *
 ******************************************************************************/
#define CUI_MODE_MENU_TYPE (cui_mode_menu_get_type ())
#define CUI_MODE_MENU(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_MODE_MENU_TYPE, CuiModeMenu))
#define CUI_IS_MODE_MENU(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_MODE_MENU_TYPE))
#define CUI_MODE_MENU_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_MODE_MENU_TYPE, CuiModeMenuClass))

typedef struct {
	CuiPopupMenu parent;
	GtkWidget   *menu;
	CuiFilename *file;
} CuiModeMenu;

typedef struct {
	CuiPopupMenuClass parent_class;
	GtkWidget *menu;
} CuiModeMenuClass;

GType cui_mode_menu_get_type(void);
CuiWidget *cui_mode_menu_new(void);
void cui_mode_menu_set_filename(CuiModeMenu *mode_menu, CuiFilename *filename);
void cui_mode_menu_popup(CuiModeMenu *cui_mode_menu,
		GtkWidget *parent_widget,
		GdkEventButton *event);

/******************************************************************************
 * Instantiable classed type: CuiUserMenu                                     *
 *                                                                            *
 ******************************************************************************/
#define CUI_USER_MENU_TYPE (cui_user_menu_get_type ())
#define CUI_USER_MENU(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_USER_MENU_TYPE, CuiUserMenu))
#define CUI_IS_USER_MENU(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_USER_MENU_TYPE))
#define CUI_USER_MENU_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_USER_MENU_TYPE, CuiUserMenuClass))

typedef struct {
	CuiPopupMenu parent;
	GtkWidget   *menu;
	gchar       *username;
} CuiUserMenu;

typedef struct {
	CuiPopupMenuClass parent_class;
	GtkWidget *menu;
} CuiUserMenuClass;

GType cui_user_menu_get_type(void);
CuiWidget *cui_user_menu_new(const gchar *username);
void cui_user_menu_popup(CuiUserMenu *cui_user_menu,
		GtkWidget *parent_widget,
		GdkEventButton *event);

/******************************************************************************
 * Instantiable classed type: CuiRectangle                                    *
 *                                                                            *
 ******************************************************************************/
#define CUI_RECTANGLE_TYPE (cui_rectangle_get_type ())
#define CUI_RECTANGLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_RECTANGLE_TYPE, CuiRectangle))
#define CUI_IS_RECTANGLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_RECTANGLE_TYPE))
#define CUI_RECTANGLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_RECTANGLE_TYPE, CuiRectangleClass))

typedef struct _CuiRectangleClass CuiRectangleClass;
struct _CuiRectangleClass {
	CuiWidgetClass parent_class;
};

/**
 * CuiRectangle:
 * @is_dnd_target: TRUE if the rectangle can be a Drag&Drop target
 * @is_dnd_source: TRUE if the rectangle can be a Drag&Drop source
 * @is_highlighted: TRUE if the rectangle is highlighted (drag-motion)
 * @source_type: the type of the data we drag from here
 * @source_data: the data we drag from here
 * @source_size: the size of the data we drag from here
 * @target_list: the #GtktargetList of acceptable DnD types
 * @target_object: the G_OBJECT which act as a the target for the DnD
 *
 * The base class for rectangular passive widgets such as frames, embossed color
 * rectangles.
 */
typedef struct _CuiRectangle CuiRectangle;
struct _CuiRectangle {
	CuiWidget parent;
	GdkColor *color;
	GdkColor *hcolor;
	GdkColor *lcolor;
	/*< public >*/
	gboolean        is_dnd_target;
	gboolean        is_dnd_source;
	gboolean        is_highlighted;
	TargetType      source_type;
	gpointer        source_data;
	gsize           source_size;	
	GtkTargetList  *target_list;
	gpointer        target_object;
};

GType cui_rectangle_get_type(void);

guint cui_rectangle_get_expose_signal_id(void);
CuiWidget *cui_rectangle_new(void);
void cui_rectangle_set_color(CuiRectangle *rec, 
		const char *color_name,
		const char *high_color_name,
		const char *low_color_name);
GdkColor *cui_rectangle_get_color(CuiRectangle *rec);
GdkColor *cui_rectangle_get_high_color(CuiRectangle *rec);
GdkColor *cui_rectangle_get_low_color(CuiRectangle *rec);
void cui_rectangle_set_is_dnd_source(CuiRectangle *rectangle, 
		gboolean is_dnd_source);
void cui_rectangle_set_dnd_source_data(
		CuiRectangle *rectangle,
		TargetType    source_type,
		gpointer      source_data,
		gsize         source_size);
void cui_rectangle_set_is_dnd_target(CuiRectangle *rectangle, 
		gboolean is_dnd_target);
gboolean cui_rectangle_is_dnd_source(CuiRectangle *rectangle);
gboolean cui_rectangle_is_dnd_target(CuiRectangle *rectangle);
gboolean cui_rectangle_set_highlighted(CuiRectangle *rectangle, 
		gboolean highlighted);
void cui_rectangle_set_target_list(CuiRectangle *rec, GtkTargetList *list);
void cui_rectangle_set_target_object(CuiRectangle *rec, GObject *object);
GtkTargetList *cui_rectangle_get_target_list(CuiRectangle *rec);
gboolean cui_rectangle_redraw(CuiRectangle *rectangle, gint debug);

/******************************************************************************
 * Class: CuiRectangleEmboss:                                                 *
 *                                                                            *
 ******************************************************************************/
#define CUI_RECTANGLE_EMBOSS_TYPE (cui_rectangle_emboss_get_type ())
#define CUI_RECTANGLE_EMBOSS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_RECTANGLE_EMBOSS_TYPE, CuiRectangleEmboss))
#define CUI_IS_RECTANGLE_EMBOSS(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_RECTANGLE_EMBOSS_TYPE))
#define CUI_RECTANGLE_EMBOSS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_RECTANGLE_EMBOSS_TYPE, CuiRectangleEmbossClass))

typedef struct {
	CuiRectangle parent;
	/* instance members */
} CuiRectangleEmboss;

typedef struct {
	CuiRectangleClass parent_class;
	/* class members */
} CuiRectangleEmbossClass;

GType cui_rectangle_emboss_get_type(void);
CuiWidget *cui_rectangle_emboss_new(void);


/******************************************************************************
 * Class: CuiSizeBar:                                                         *
 *                                                                            *
 ******************************************************************************/
#define CUI_SIZE_BAR_TYPE (cui_size_bar_get_type ())
#define CUI_SIZE_BAR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_SIZE_BAR_TYPE, CuiSizeBar))
#define CUI_IS_SIZE_BAR(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_SIZE_BAR_TYPE))
#define CUI_SIZE_BAR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_SIZE_BAR_TYPE, CuiSizeBarClass))

typedef struct {
	CuiRectangleEmboss parent;
	/* instance members */
	Decoration        decor;      /* What kind of decoration we draw.         */
	gdouble           percent;    /* The width in percent.                    */
	gdouble           turnpoint;  /* What percent we turn to red from green.  */
	GdkColor          colors[5];
} CuiSizeBar;

typedef struct {
	CuiRectangleEmbossClass parent_class;
	/* class members */
} CuiSizeBarClass;

GType cui_size_bar_get_type(void);
CuiWidget *cui_size_bar_new(void);


/******************************************************************************
 * Class: CuiFrame                                                            *
 *                                                                            *
 ******************************************************************************/
#define CUI_FRAME_TYPE (cui_frame_get_type ())
#define CUI_FRAME(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), \
			 CUI_FRAME_TYPE, CuiFrame))
#define CUI_IS_FRAME(obj) (G_TYPE_CHECK_INSTANCE_TYPE((obj), \
			CUI_FRAME_TYPE))
#define CUI_FRAME_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), \
			CUI_FRAME_TYPE, CuiFrameClass))

typedef struct {
	CuiRectangle parent;
	/* instance members */
} CuiFrame;

typedef struct {
	CuiRectangleClass parent_class;
	/* class members */
} CuiFrameClass;

GType cui_frame_get_type(void);
CuiWidget *cui_frame_new(void);
#endif
